#!/usr/bin/perl

$VERBOSE = 0;

$SENDMAIL_PATH = "/usr/lib/sendmail";

# We're assuming that this script is running on the same machine as the
# webserver:
$SERVER_IP      = '127.0.0.1';
$PORT           = 80;

$WEB_REQUEST_TIMEOUT = 30;  # seconds
$WAKE_UP_FREQUENCY = 30;  # seconds
$ALERT_EMAIL_FROM = "loco_pm_watcher\@leverage.com";

# add whatever other email addresses (e.g. any pagers) you want here:
$ALERT_EMAIL_TO  = " ck\@leverage.com chris\@vivid.com";

$request_text_1 = "GET /cb/locolink_p5421/watch2locos HTTP/1.0\n\n";
$request_text_2 = "GET /cb/locolink_p5422/watch2locos HTTP/1.0\n\n";

# default: start out at locolink #1.
$request_text = $request_text_1;

$expected_result = "Unknown Handler";
##########################################################################
# Begin main
##########################################################################

$loco_response = `/usr/local/cgi-bin/locolink_p5421`;
if ($loco_response !~ /default handler/)
{
    print ("The Loco on port 5421 is not running!  Please start it.\n");
    exit(-1);
}

$loco_response = `/usr/local/cgi-bin/locolink_p5422`;
if ($loco_response !~ /default handler/)
{
    print ("The Loco on port 5422 is not running!  Please start it.\n");
    exit(-1);
}

$loco_response = `/usr/local/cgi-bin/locolink`;
if ($loco_response !~ /default handler/)
{
    print ("The default Loco is not running!  Please start it.\n");
    exit(-1);
}

##### End of initialization checks #######



for (;;)
{

$datestr = `date`;
chop($datestr);

# set the flag that says whether it was the loco that malfunctioned
(timeout
# or crash), as opposed to the web server.
$loco_malfunctioned = "";

# set the flag that says whether the problem was a timeout to 'false'.
$did_time_out = "";

# get the protocol identifier
($name, $aliases, $proto) = getprotobyname ('tcp');

# get a new socket (af_inet, socket_type = stream)
if (! socket (SFD, 2, 2, $proto))
{
    $error_msg .= "INTERNAL ERROR: socket allocation failed\n";
    &send_alert();
    exit(-1);
}

# do a connect

print ("connecting via regular_connect\n") if ($VERBOSE);
if (! &regular_connect (SFD, $SERVER_IP, $PORT))
{
    $error_msg .= "ERROR: regular_connect: failed\n";
    &send_alert();
    exit(-1);
}

# if we've made it this far we're connected
    
print ("connection established\n") if ($VERBOSE);



$error_msg = "";

# set alarm and handler (cause interrupt on blocking read/write below)
$which_timeout = "write_to_server";
$SIG {'ALRM'} = 'socket_timeout';
alarm ($WEB_REQUEST_TIMEOUT);

# write request to socket

print ("request string: " . $request_text . "\n") if ($VERBOSE);

$numwritten = 0;
$total_written = 0;
$msg_length = length ($request_text);
while ($total_written < $msg_length)
{
    $numwritten = syswrite (SFD, $request_text, 
                            $msg_length - $total_written, $total_written);
    
    if (! defined ($numwritten))
    {
        $error_msg .= "ERROR: write to webserver failed: $!\n";
        &send_alert();
        exit(-1);
    }
    
    $total_written = $total_written + $numwritten;
}

print ("request_written\n") if ($VERBOSE);

# try and read the results 
$which_timeout = "read_response_from_server";
($response, $size) = &read_until_eof (SFD);
close (SFD);

print ("response_received\n") if ($VERBOSE);
print ($response) if ($VERBOSE);
# turn off the clock
alarm(0);

# do results comparison
if (! ($response =~ /$expected_result/))
{
#    we don't need to mail the error to the pager now, because we're going
#    to automatically detect what the error is.
#    $error_msg .= "ERROR: server returns bogus results: \n";
#    $error_msg .= substr ($response, 0, 512);
    $loco_malfunctioned = "true";
}
else
{
    print ("response_passed_analysis\n") if ($VERBOSE);
}

print ($response . "\n") if ($VERBOSE == 2);

if ($loco_malfunctioned eq "true")
{
    &handle_crash();
    &send_alert();
}

# execute this test every 30 seconds
sleep($WAKE_UP_FREQUENCY);

# we should alternate between checking the real locos and the spare locos,
# so that we know we can fail over successfully:
if ($request_text eq $request_text_1)
{
    $request_text = $request_text_2;
}
else
{
    $request_text = $request_text_1;
}

}
# end of main for (;;) loop


############################################################
# BEGIN SUBROUTINES
############################################################

sub regular_connect
{
    local ($SFD, $remote_host_ipaddr_string, $remote_host_port) = @_;
    local ($sock_addr_in) = pack ("C C n C4 x8", 14, 2, $remote_host_port, 
                                  split(/\./, $remote_host_ipaddr_string));

    $which_timeout = "direct connect";
    $SIG {'ALRM'} = "socket_timeout";
    alarm ($WEB_REQUEST_TIMEOUT);
    if (! connect ($SFD, $sock_addr_in))
    {
        $error_msg .= "ERROR: connect failed: $!\n";
        return ("");
    }
    alarm (0);
    return ("true");
}

sub send_alert
{
    if (! open (SMFH, "|$SENDMAIL_PATH $ALERT_EMAIL_TO"))
    {
        print ("ERROR: failed to open sendmail fh!\n");
        exit (-1);
    }
    print (SMFH "To: sky.thede\@leverage.com\n");
    print (SMFH "From: $ALERT_EMAIL_FROM\n");
    print (SMFH "Subject: Watcher Alert\n\n");
    print (SMFH "$error_msg\n at time: $datestr\n");
    close (SMFH);
}


sub socket_timeout
{       
    local ($sig) = @_;
    $error_msg .= "SIGNAL: ALRM: connection timeout: $which_timeout\n";
    $did_time_out = $which_timeout;
    $loco_malfunctioned = "true";
}  

sub read_until_eof
{
    local ($SFD) = @_;
    local ($done, $numread, $totalread) = (0, 0, 0);
    local ($buf);
    while ($done != 1)
    {
        $numread = sysread ($SFD, $buf, 4096, $totalread);

        if (! defined ($numread))
        {
            $error_msg .= "ERROR: read response: $!\n";
            $loco_malfunctioned = "true";
        }
        if ($numread != 0)
        {
            $totalread += $numread;
        }
        else
        {
            $done = 1;
        }
    }
    return ($buf, $totalread);

}

sub handle_crash
{
    local($timestamp) = `date +%Y%m%d.%H%M%S`;
    chop $timestamp;
    local($crashlog) = "/home/loco/locoroot/logs/crash-$timestamp.log";

    # Here's where we switch over the locolinks.
    # BUT we switch over only when we know that the REAL loco failed;  in
    # other words, we might have been testing the spare loco this time.

    $which_locolink = readlink "/usr/local/cgi-bin/locolink";
    if ($which_locolink =~ /5421/ && $request_text =~ /5421/)
    {
        system ("cd /usr/local/cgi-bin; ln -s locolink_p5422 locolink_new; mv locolink_new
locolink");
        $error_msg = "NOTE: locolink has switched to 2.\n" . $error_msg;
    }
    elsif ($which_locolink =~ /5422/ && $request_text =~ /5422/)
    {
        system ("cd /usr/local/cgi-bin; ln -s locolink_p5421 locolink_new; mv locolink_new
locolink");
        $error_msg = "NOTE: locolink has switched to 1.\n" . $error_msg;
    }
    
    # Okay, now we've got some time to bring the server back up.
    
    local ($core_file) = "/home/loco/locoroot/config/core";
    local ($which_loco);
    local ($bad_loco_pid) = 0;
    
    if ($request_text =~ /542([0-9])/)
    {
        $which_loco = $1;
        print "Loco that failed: $which_loco.\n" if ($VERBOSE);
    }
    else
    {
        $error_msg = "request_string, '$request_text,' is wrong!\n" . $error_msg;
        return;
    }
    
    
    # if we couldn't connect to the server, it probably core dumped (Oracle).
    if ($response =~ /Connection refused/)
    {
        if (-e $core_file)
        {
            system("mv $core_file $core_file-$which_loco.$timestamp");
            $error_msg = "Loco $which_loco core dumped. " . $error_msg;
        }
        
        &restart_loco($which_loco);
    }
    elsif ($did_time_out eq "read_response_from_server")
    {
        # if there was no core dump, then it was probably a hang.
        $bad_loco_pid = `/usr/ucb/ps -auxwww | grep '^loco.*java -ms2$which_loco' | grep -v
grep`;
        
        if ($bad_loco_pid =~ /^loco[ ]+([0-9]+)/)
        {
            $bad_loco_pid = $1;
            
            # first, try kill -QUIT, because Javasoft says that's what we
            # need to do to generate a thread dump.
            kill ("QUIT", $bad_loco_pid);

            # sleep after this for 3 seconds, so that kill -QUIT has time
            # to act.
            sleep (3);
            
            # if it doesn't respond to kill -QUIT, because it's hung, then
            # try killing it more brutally.
            kill ("TERM", $bad_loco_pid);
            print "Killed $bad_loco_pid.\n" if ($VERBOSE);
            
            &restart_loco($which_loco);
            
            $error_msg = "Loco $which_loco hung.  Killed it. " . $error_msg;
        }
        else
        {
            $error_msg = "could not kill Loco $which_loco because " .
                "could not find its PID! " .
                "Call Thede 362-2272, Ben 334-2971, or " .
                    "Piyush 776-1971 to log on and kill it." . $error_msg;
        }           
    }
    else
    {
        $error_msg = "unknown problem with Loco $which_loco! " .
            "Call Thede 362-2272, Ben 334-2971, or " .
            "Piyush 776-1971 to log on and fix it." . $error_msg;
    }
    
    open (CRLOG, ">>$crashlog");
    print (CRLOG "$error_msg at time: $datestr\n\n");
    print (CRLOG "This is the response from the Loco:\n\n");
    print (CRLOG substr($response, 0, 512));
    close (CRLOG);
}


sub restart_loco
{
    local($which_loco) = @_;
    system("/home/loco/bin/startloco_N $which_loco");
    $error_msg = "Restarted Loco $which_loco. " . $error_msg;
}
