 -- The contents of this file are subject to the Locomotive Public License 
 -- (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 -- may not use this file except in compliance with the License; You may 
 -- obtain a copy of the License at http://www.locomotive.org/license/.
 -- 
 -- Software distributed under the License is distributed on an "AS IS" basis, 
 -- WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 -- for the specific language governing rights and limitations under the License. 
 --
 -- The initial developers of this code under the LPL is Leverage Information 
 -- Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 -- Information Systems.  All Rights reserved.

--	Primary Schema for User/Session for use with the
--      Leverage Locomotive for Informix On-Line 7.30
--	Copyright 1998 Leverage.	All rights reserved.



-- To Re-install the Locomotive Schema, simply 
-- drop the loco_db database and run this script again.  You can 
-- drop the database by selecting 'Database' from the root menu of 
-- dbaccess and selecting 'Drop'. 




-- To deploy the LOCO application for Informix, the following
-- steps must be taken:
	
--1. At least one  dbspace must be created.  It is recommended that
--	two or more dbspaces should be used, however.  The total size
--	of all the combined dbspaces should be at least 20 mb.
--
--	NOTE: when creating dbspaces or chunks, always use symbolic
--	links to point to the physical spaces the dbspaces occupy.
--	Doing so allows for easy swapping of raw files for cooked files 
--      without taking the database offline.

-- 2. After the dbspaces are created, you probably want to add the loco
--      database user.  To add the loco user, you must first create a 
--      user for your OS.  Once that's done, you can simply grant
--      them the appropriate permissions in Informix.  For instance,
--      after we adduser loco, we can user the following SQL
--      command:

--        grant connect, resource to loco_user;

--      this user can then use his or her unix password to sign on and
--      access the database.

-- 3. Now, a database must be created to hold the tables.  You'll probably 
--      want to do this as the loco user, so you should quit dbaccess, 
--      login as the loco user on your OS, then restart dbaccess. 
--      The database you create should be loaded into one of the dbspaces, 
--      and the log type should be set to unbuffered. 
--	For example, the following SQL code may be used:
		
	create database loco_db in loco_01_dbs with log;

 
 
-- 3. Once the database is created, the tables holding the sequences
--	must be created.  The table names will be loco_sessions_sq, 
--      and loco_BStampID_sq, each with one column called 'sequence'.
 

        create table loco_sessions_sq (
                sequence DECIMAL (20, 0)
        ) in loco_01_dbs;
 
        insert into loco_sessions_sq values (10000);
 

        create table loco_bstampid_sq (
                sequence DECIMAL (20, 0)
        ) in loco_01_dbs;
 
        insert into loco_bstampid_sq values (1000000);


-- Updating the BStampID and creating sessions are done with stored procedures.
--	They can be found in loco_INFORMIX_stored_prodedures.sql
--	select sequence from loco_users_sq.

 
 
-- 4. Next, the two main tables used by the LOCO must be created
--	These two tables must be called 'loco_users' and 
--	'loco_sessions'.  If more than one dbspace is being used, then
--	these tables should be fragmented across the dbspaces. The
--	table sizes should begin at 1 mb and grow in 1 mb.
--	Example SQL code to fragment the tables across two dbspaces
--	is as follows:

	create table loco_users (
		userid SERIAL (10000) PRIMARY KEY,
	        username VARCHAR (100) UNIQUE,
		password VARCHAR (32),
		loco_flags INT,
		added DATETIME YEAR to SECOND,
		modified DATETIME YEAR to SECOND,
		last_login DATETIME YEAR to SECOND
	) fragment by expression
		mod(userid, 2) = 0 in loco_01_dbs,
		mod(userid, 2) = 1 in loco_02_dbs
	extent size 1024 next size 1024;


	create table loco_sessions (
		sid DECIMAL (20, 0) PRIMARY KEY,  
		srid INT,   
		userid INT,
		added DATETIME YEAR to SECOND,
		active DATETIME YEAR to SECOND,
		expire DATETIME YEAR to SECOND
	) fragment by expression
		mod(userid, 2) = 0 in loco_01_dbs,
		mod(userid, 2) = 1 in loco_02_dbs
	extent size 1024 next size 1024;


--	These values should be montored for efficiency using the
--	various tools Informix offers, and can be altered at
--	any time using the ALTER TABLE command.


-- 6. Now, you must add the stored procedures the Locomotive needs.


-- This procedure increments the loco_sessions_sq table and inserts a new
--	session in the loco_sessions table
--





create procedure get_next_BstampID()
        returning decimal (20, 0);

        define value decimal(20, 0);
	define esql integer;

	ON EXCEPTION set esql
		ROLLBACK WORK;
		RAISE EXCEPTION esql;
	END EXCEPTION

	BEGIN WORK;

        let value = (select sequence from loco_bstampid_sq);
        update loco_bstampid_sq set sequence = value + 1;

	COMMIT WORK;
	
        return value;
end procedure;


--	to get the next BStampId, issue the following command:
--
--	execute procedure get_next_BStampID();
--

-- This procedure adds a new session to the sessions table. The
--	 session id is incremented by one for each new session, and 
--	returned as a resultset.
--		
create procedure create_session(srid INT, userid INT,
                added DATETIME YEAR to SECOND, active DATETIME YEAR to SECOND,
                expire DATETIME YEAR to SECOND)
        returning decimal (20, 0);
 
        define value decimal(20, 0);
	define esql integer;

	ON EXCEPTION set esql
		ROLLBACK WORK;
		RAISE EXCEPTION esql;
	END EXCEPTION

	BEGIN WORK;
                                
        let value = (select sequence from loco_sessions_sq);
        insert into loco_sessions 
		(sid, srid, userid, added, active, expire)
		values (value, srid, userid, added, active, expire );
        update loco_sessions_sq set sequence = value + 1;

	COMMIT WORK;

        return value;   

end procedure;


--	Here is a sample use of this procedure: 
--	execute procedure create_session(0, 0, current, current, current);
--


-- This procedure adds a new user tp the user table.  The user id is 
--	incremented by one, and is returned as the resultset
--


create procedure create_user(userid INT, username VARCHAR(32),
                password VARCHAR (32), loco_flags INT)
        returning INT;  

        define value int;

        insert into LOCO_USERS (userid, username, password, 
		loco_flags, added, modified, last_login) values (
		userid, username, password, loco_flags, 
		CURRENT, CURRENT, CURRENT);

        let value = dbinfo('sqlca.sqlerrd1');
        return value;   
end procedure;


-- The example below inserts an administrator into the database

execute procedure create_user (0, 'admin',  'changeme', 7);

 
--drop table loco_session_objects;
create table loco_session_objects (
        sid DECIMAL (20, 0) PRIMARY KEY REFERENCES loco_sessions (sid)
                ON DELETE CASCADE,
        objects BLOB
        ) fragment by expression
                mod(sid, 2) = 0 in loco_01_dbs
                mod(sid, 2) = 1 in loco_02_dbs
        extent size 1024 next size 1024;

