/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

/*
**      Primary Schema for the Discussion  demo for use with
**      Locomotive for Oracle 
*/

-- the community table space
--drop tablespace loco_community_ts;
--create tablespace loco_community_ts
--	datafile '/evol/vol01/oradata/DEVEL/datafiles/loco_community_ts01.dbf'
--	size 10M default storage (initial 1M next 1M pctincrease 10);


-- the community sequences- one for each of the tables
drop sequence community_id_sq;
drop sequence communitytopic_id_sq;
drop sequence communitypost_id_sq;

create sequence community_id_sq start with 1;
create sequence communitytopic_id_sq start with 1;
create sequence communitypost_id_sq start with 1;

-- the community tables
drop table community cascade constraints;
drop table communitytopic cascade constraints;
drop table communitypost;

create table community (
	communityid NUMBER (10) PRIMARY KEY, 
	communityname VARCHAR (255),
	communitydesc LONG,
	communitystatus CHAR (4)
) tablespace loco_community_ts;

create table communitytopic (
	topicid NUMBER (10) PRIMARY KEY,
	topicdesc VARCHAR2 (2000),
	communityid NUMBER (10) REFERENCES community (communityid) 
		ON DELETE CASCADE,
	topicname VARCHAR (255),
	topicstatus CHAR (4)
) tablespace loco_community_ts;

create table communitypost (
	postid NUMBER (10) PRIMARY KEY,
	topicid NUMBER (10) REFERENCES communitytopic (topicid) 
		ON DELETE CASCADE,
	posttitle VARCHAR (255), 
	postbody LONG,
	postauthorfn VARCHAR (100),
	postauthorln VARCHAR (100),
	postemail VARCHAR (100),
	postdate DATE,
	poststatus char (4)
) tablespace loco_community_ts;

-----Permissions for Community System
-----Requires that The Permission Tables have been created
-----See db/perm/loco_permissions.sql

delete from loco_permissions where name = 'COMM_ADMIN';
delete from loco_permissions where name = 'COMM_READ';
delete from loco_permissions where name = 'COMM_ANON';

insert into loco_permissions values (LOCO_PERMISSIONS_SQ.nextval, 'COMM_ADMIN');
insert into loco_permissions values (LOCO_PERMISSIONS_SQ.nextval, 'COMM_READ');
insert into loco_permissions values (LOCO_PERMISSIONS_SQ.nextval, 'COMM_ANON');

----Add permissions to the generic Admin user
delete from loco_user_perms where userid = 1 and perm_id = 
	(select id from loco_permissions where name = 'COMM_ADMIN');

insert into loco_user_perms select 1, id from loco_permissions 
	where name = 'COMM_ADMIN';


insert into community values (0, 'DEFAULT_CATEGORY', 'This category is created as a default to support a single level discussion system', 'LIVE');


