-- The contents of this file are subject to the Locomotive Public License 
-- (LPL), a derivative of the Mozilla Public License, version 1.0. You 
-- may not use this file except in compliance with the License; You may 
-- obtain a copy of the License at http://www.locomotive.org/license/.
-- 
-- Software distributed under the License is distributed on an "AS IS" basis, 
-- WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
-- for the specific language governing rights and limitations under the License. 
--
-- The initial developers of this code under the LPL is Leverage Information 
-- Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
-- Information Systems.  All Rights reserved.

--      Primary Schema for Access Control LIST for use with the
--      Locomotive for SOLID
--
-- This script should be run with solid as the loco user.
-- It's purpose is to set up the tables, sequences, and stored
-- prodedures used by the locomotive for the default user and 
-- session managment facilities.  
-- 
-- create various sequences
drop sequence loco_bstamp_sq;
create sequence loco_bstamp_sq;

drop sequence loco_sessions_sid_sq;
create sequence loco_sessions_sid_sq;

drop sequence loco_users_userid_sq;
create sequence loco_users_userid_sq;

-- create user and session tables

drop table loco_sessions;
create table loco_sessions 
(
	sid DECIMAL (0,20) PRIMARY KEY,
        srid integer,
        userid integer,
        added timestamp,
        active timestamp,
        expire timestamp
);

drop table loco_users;
create table loco_users  
(
	userid integer PRIMARY KEY,
	username varchar (32) UNIQUE,
	password varchar (32),
	loco_flags integer,
	added timestamp,
	modified timestamp,
	loco_login timestamp
);

-- create stored procedures for sequence oriented transactions 
-- proc: loco_get_next_bstamp_id 
drop procedure loco_get_next_bstamp_id;
commit work;
"create procedure loco_get_next_bstamp_id
 returns (stmpid integer)
begin 
  EXEC SEQUENCE loco_bstamp_sq.NEXT INTO stmpid;
end";

-- proc: loco_create_user
drop procedure loco_create_user;
commit work;
"create procedure loco_create_user (
     newuid integer,
     newuname varchar(32),
     newpw varchar(32), 
     newlflgs integer)
 returns (uidout integer)
begin 

 if newuid <= 0 then
  exec sequence loco_users_userid_sq.next into newuid;
 end if 

 exec sql whenever sqlerror rollback, abort;
 exec sql prepare foo 
        insert into loco_users values (
           ?, ?, ?, ?, curtime(), curtime(), curtime());
 exec sql execute foo using (newuid, newuname, newpw, newlflgs);
 exec sql close foo;
 exec sql drop foo;

 set uidout = newuid;
end";

-- create the admin user 
call loco_create_user (1, 'admin', 'changeme', 7);
commit work;

-- proc: loco_create_session
drop procedure loco_create_session;
commit work;
"create procedure loco_create_session (
     srid_in integer,
     userid_in integer, 
     expire_in timestamp)
 returns (sid_out integer)
begin 
 declare new_sid integer; 

 exec sequence loco_users_userid_sq.next into new_sid;

 exec sql whenever sqlerror rollback, abort;
 exec sql prepare foo 
        insert into loco_sessions values (?, ?, ?, curtime(), curtime(), ?);
 exec sql execute foo using (new_sid, srid_in, userid_in, expire_in);
 exec sql close foo;
 exec sql drop foo;

 set sid_out = new_sid;
end";

commit work;

-- These just set the sequence numbers for the userid_sq and the sid_sq
-- to much higher than 1.  I couldn't find any way to do 
-- at creation, which oracle allows you to do.  Bummer.   
-- 
-- create temporary stored proc to set userid sq number to 10000 
drop procedure tmp_set_uid_seq;
commit work;
"create procedure tmp_set_uid_seq
 returns (newval integer)
begin 
	set newval = 10000;
	exec sequence loco_users_userid_sq set value using newval;
end";

call tmp_set_uid_seq;
commit work;
drop procedure tmp_set_uid_seq;
commit work;


-- create temporary stored proc to set session id sequence number to 10000 
drop procedure tmp_set_sid_seq;
commit work;
"create procedure tmp_set_sid_seq
 returns (newval integer)
begin 
	set newval = 10000;
	exec sequence loco_sessions_sid_sq set value using newval;
end";

call tmp_set_sid_seq;
commit work;
drop procedure tmp_set_sid_seq;
commit work;

