/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

/* locolink3.h
   Header file for the ISAPI module which will communicate
   directly with Locomotive
   Author: Steve Doliov
   Copyright 1998, Locomotive Inc.
   Date: 08/21/1998
*/

#ifndef _LAS_ISAPI_H
#define _LAS_ISAPI_H

//#include <afx.h>
//#include <afxwin.h>
//#include <afxmt.h>
//#include <afxext.h>
//#include <afxisapi.h>
//#include <winsock2.h>
#include <httpext.h>

//insert any macros here

//just some details for later debugging info
//note that these macro vars refer to this ISAPI module,
//not to the version number that gets passed on to locomotive
#define MAJOR_VERSION 1
#define MINOR_VERSION 0
#define BUILD_VERSION 0

#define LOCO_PROTOCOL_VERSION 3L

#define CONFIGURATION_FILE "c:\\InetPub\\wwwroot\\locolink_dll.txt"
#define CONFIG_FILE_MAX_LENGTH 4096
#define CONFIG_FILE_MAX_LINES 255
#define CONFIG_FILE_LINE_SEPS " \r\n"
#define CONFIG_FILE_FIELD_SEPS " ,"
//MAX_LINE_LENGTH for conf File (15 bytes ip addr, 2 bytes commas, 5 bytes port,2 bytes weight,2 bytes EOL (\r\n), will give six bytes to whitespace
#define MAX_LINE_LENGTH 32
//MAX_WAIT_TIME is how long (in milliseconds) to wait for MUTEX object before timing out
#define MAX_WAIT_TIME 50

//MAXIMUM NUMBER OF BYTES TO ACCEPT FROM ANY GIVEN ENV VARIABLE
#define ENV_BUFFER_SIZE 4096

#define MAX_SECONDS_WAIT_FOR_INPUT 5
#define MAX_MICRO_SECONDS_WAIT_FOR_INPUT 0

//#define IDS_SERVER        102

//#define _APS_NEXT_RESOURCE_VALUE        103
//#define _APS_NEXT_CONTROL_VALUE         103
//#define _APS_NEXT_SYMED_VALUE           103
//#define _APS_NEXT_COMMAND_VALUE         32768

//insert variables globally available to dll here
  //probably don't want too many of these as ISAPI
  //will likely be multi-threaded and will need to
  //control atomicity of operations on variables
  //think twice before declaring variables here

static int *dllIsInitialized = 0;

static int locoProtocolVersion;

static HANDLE confFileHandle;
static FILETIME *lastWriteTime;
//locThread is the handle of the thread which does the
//configuration file checking
HANDLE locThread;
//locThreadId holds the id of the thread which handles
//the configuration file checking.
//this will be useful for blocking/mutex operations
DWORD locThreadId;

//the handle for the mutex which will govern configuration
//data reading
static HANDLE locThreadMutex;

static LPSTR fileData;

static LPSTR loadBalScheme;

static int *numAvailLocos;

static int linCursor = 0;

static int sumBal = 0;
static int *cumWeight;

static LPSYSTEMTIME sysTime;

#define PREDEFINED_ENV_VARS 28

char *ENVS[PREDEFINED_ENV_VARS] 
               = { "AUTH_TYPE","CONTENT_TYPE","CONTENT_LENGTH",
                   "GATEWAY_INTERFACE","HTTP_ACCEPT",
                   "UNMAPPED_REMOTE_USER",
	               "LOGON_USER","PATH_INFO","PATH_TRANSLATED",
                   "QUERY_STRING",
				   "REMOTE_ADDR","REMOTE_HOST",
                   "REMOTE_USER","REQUEST_METHOD",
				   "SCRIPT_NAME","SERVER_NAME","SERVER_PORT",
				   "SERVER_PORT_SECURE","SERVER_PROTOCOL",
                   "SERVER_SOFTWARE",
                   "HTTPS",
                   "HTTP_COOKIE","HTTP_REFERER","USER_AGENT",
				   "URL","ACCEPT_LANGUAGE","CONNECTION",
                    "ACCEPT_ENCODING"};

/*
                   "HTTPS_SERVER_ISSUER","HTTPS_SERVER_SUBJECT",
                   "APPL_MD_PATH","APPL_PHYSICAL_PATH"   ,
                   "HTTPS_KEYSIZE","HTTPS_SECRETKEYSIZE",
                   "AUTH_PASSWORD","CERT_COOKIE","CERT_FLAGS",
                   "CERT_ISSUER","CERT_KEYSIZE","CERT_SECRETKEYSIZE",
                   "CERT_SERVER_ISSUER","CERT_SERVER_SUBJECT",
                   "INSTANCE_ID","INSTANCE_META_PATH",
                    };

*/
//note, "ALL_HTTP", has been left out of this list because
//other things such as cookies need to be parsed from "ALL_HTTP"
//"ALL_HTTP" "ALL_RAW"

#define IO_BUF_LENGTH 16384
#define ENVVAR_NAME_MAX_LENGTH 255

//struct availableLocos will reside in memory and
//be refreshed every so often
//note for functions which access this data structure:
//when this memory structure is queried, must prevent
//refresher from getting new data

struct availableLocos {
  LPSTR ipaddr;
  USHORT port;
  ULONG weight;
};

//pointer to pointer to memory data with available locos
static availableLocos *al;

//create data structures to hold environment which needs
//to be passed to locomotive or received from locomotive
//
//thinking is this is a construct which is easily changeable
//should protocols change;

struct locoEnvSendStream {
  int version;
  char *buf;
};

struct locoEnvRecvStream {
  //locomotive does not send out version number back to cgi/dll
  //might be stupid now to hold just one member in a data structure
  //but might payoff down the road should protocols change.
  char *buf;
};


//insert function prototypes here

//general funtions
BOOL confFileExists(EXTENSION_CONTROL_BLOCK *pECB);
BOOL readConfFile(LPSTR fileData,EXTENSION_CONTROL_BLOCK *pECB);
BOOL fileDataLoaded(LPSTR fileData,EXTENSION_CONTROL_BLOCK *pECB);
BOOL getEnvVar(char *envName,char *recvBuf,LPDWORD bytesInVar,EXTENSION_CONTROL_BLOCK *pECB);

//check which locomotive to read/write from/to
BOOL needsRefresh(EXTENSION_CONTROL_BLOCK *pECB);
BOOL refreshAvailableLocos(EXTENSION_CONTROL_BLOCK *pECB);
int chooseLocomotive(struct availableLocos *al,LPSTR loadBalScheme,EXTENSION_CONTROL_BLOCK *pECB);

//send data to locomotive
SOCKET getConnection(struct availableLocos *al,int choice,EXTENSION_CONTROL_BLOCK *pECB);
int writeLocoEnvSendStream(int socketConn,EXTENSION_CONTROL_BLOCK *pECB,char *buf);
int writeBuffer(int socketConn,char *buf,int howFar,int flag,struct sockaddr *sa,int x);
int writeLocoClientSTDIN(int socketConn,char *contentLength);

//receive data from locomitive
int fillLocoEnvRecvStream(int socketConn,char *buf,int buflen,char **name,char **vals,
						  EXTENSION_CONTROL_BLOCK *pECB);

//note: ISAPI's readClient() call will be used to fill the
//locomotive env and stdin streams;
//the readLocoEnvRecvStream() and readLocoServerSTDOUT() calls
//will be parsed and sent back to the client via the writeClient()
//ISAPI call

//debugging functions
void messToBrowser(char *s,EXTENSION_CONTROL_BLOCK *pECB);
char *numToString(LONG x);
char *hexString(char *myString);
#endif
