/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */
package org.locomotive.directory;

import java.io.*;
import java.util.*;

/**
 * Interface to an object responsible for parsing a single bean configuration
 *  file and building the appropriate BeanConfiguration structure.
 *
 * @author Gunnar Rnning (gunnar@candleweb.no).
 * @version $Id: BeanParser.java,v 1.4 1999/05/05 10:19:31 gunnar Exp $
 */
public interface BeanParser {
  /**
   * Parse a bean property configuration file and build
   * the approriate BeanConfiguration structure. The method can be called
   * several times with the same bean configuration structure, and will
   * the update the structure to reflect the changes introduced by the
   * new file.
   * @param fileName The fileName of the configuration file.
   * @param beanConfig The beanConfig structure to build/modify.
   */
  public void parseBeanConfiguration(String fileName, 
				     BeanConfiguration beanConfig);
}
