/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */
package org.locomotive.directory;

import java.util.*;

/**
 * Represents a property of a bean. A bean can one single value or a list
 * of values.
 *
 * @author Gunnar Rnning (gunnar@candleweb.no).
 * @version $Id: BeanProperty.java,v 1.2 1999/05/05 10:19:31 gunnar Exp $
 */
public class BeanProperty {
  String name;
  Vector values = new Vector();
  
  /**
   * Set the property name.
   * @param name The property name.
   */
  public void setName(String name){
    this.name = name;
  }

  /**
   * Get the property name.
   * @return A String with the property name.
   */
  public String getName(){
    return name;
  }

  /**
   * Set the property value. This method will throw away existing values.
   * @param value The value for this property. 
   */
  public void setValue(BeanPropertyValue value){
    values.removeAllElements();
    values.addElement(value);
  } 
  
  /**
   * Add a property value. This method will add a property value
   * to the list of values for this property.
   * @param value The value to add for this property. 
   */
  public void addValue(BeanPropertyValue value){
    values.addElement(value);
  }
  
  /**
   * Get the list of property values for this property.
   * @return A vector with BeanPropertyValue objects.
   */
  public Vector getValues(){
    return values;
  }

  public String toString(){
    StringBuffer buf = new StringBuffer("BeanProperty=" + name + 
					"(");
    for(int i = 0; i < values.size(); i++){
      if(i > 0)
	buf.append(",");
      buf.append(values.elementAt(i).toString());
    }
    buf.append(")");
    return buf.toString();
  }
}
