/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */
package org.locomotive.directory;

/**
 * A class representing a bean property value.
 */
public class BeanPropertyValue {
  /**
   * Indicates that value of this property is constant, and 
   * computedValue can be used in future lookups.
   * @see getType().
   */
  public static final int CONSTANT_TYPE = 1;

  /**
   * Indicates that this value references a component, and
   * and that the computed value cannot be stored for future use.
   */
  public static final int COMPONENT_TYPE = 2;

  int type = CONSTANT_TYPE;
  Object computedValue = null;
  String value;

  /**
   * Get the type of this property value declaration. The type defaults
   * to CONSTANT_TYPE.
   */
  public int getType(){
    return type;
  } 

  /**
   * Set the type of this property value declaration. The type defaults
   * to CONSTANT_TYPE.
   * @param type The type of this value declaration.
   */
  public void setType(int type){
    this.type = type;
  }

  /**
   * Get the computed value for this property value. It will
   * return null if no value has been computed for it.
   */
  public Object getComputedValue(){
    return computedValue;
  }

  /**
   * Set the computed value for this property value.
   * It can only be called for the type is CONSTANT_TYPE.
   * @param computedValue Object containing the computed value.
   */
  public void setComputedValue(Object computedValue){
    this.computedValue = computedValue;
  }
  
  /**
   * Get the value as a String. 
   */ 
  public String getValue(){
    return value;
  }

  /**
   * Set the value as a String.
   * @param value The String value for this property. 
   */ 
  public void setValue(String value){
    this.value = value;
  }

  public String toString(){
    String typestr;
    if(type == CONSTANT_TYPE)
      typestr = "CONSTANT_TYPE";
    else if(type == COMPONENT_TYPE)
      typestr = "COMPONENT_TYPE";
    else
      typestr = "UNDEFINED";
    
    return "type=" + typestr + ",value=" + value;
  }
  
}
