/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */
package org.locomotive.directory;

import java.util.*;

/**
 * A directory context is a path in the configuration structure, 
 * e.g. the root of the configuration tree is "/". Directory names
 * for a given directory context is always separated by slash("/").
 *
 * @author Gunnar Rnning (gunnar@candleweb.no).
 * @version $Id: DirectoryContext.java,v 1.2 1999/05/05 10:19:31 gunnar Exp $
 */
public class DirectoryContext {
  static Hashtable contexts = new Hashtable();
  String context;

  
  public static DirectoryContext getDirectoryContext(String context)
  {
    context = canonifyContext(context);

    DirectoryContext c = (DirectoryContext) contexts.get(context);
    if(c == null){
      c = new DirectoryContext(context);
      contexts.put(context, c);
    }
    return c;
  }

  public static DirectoryContext getDirectoryContext(DirectoryContext context, 
						     String relativeContext)
  {
    if(relativeContext == null){
      return getDirectoryContext(context.getContext());
    }
    if(relativeContext.startsWith("/") || context == null)
      return getDirectoryContext(relativeContext);

    return getDirectoryContext(context.getContext() + relativeContext);
  }



  private static String canonifyContext(String context){
    // Test for the trivial cases first. These are probably the ones
    // that will occur most often, and then it makes sense
    // to make these cases as efficient as possible. 

    if(context == null)
      return "/";

    if(context.indexOf("..") == -1){
      if(!context.startsWith("/"))
	context = "/" + context;

      if(!context.endsWith("/"))
	context = context + "/";

      return context;
    }
    
    Vector path = new Vector();

    int currentIndex = 0;

    while(true) {
      int nextIndex = context.indexOf('/', currentIndex);

      if(nextIndex != currentIndex){
	if(context.regionMatches(currentIndex, "..", 0, 2)){
	  if(path.size() > 0)
	    path.removeElementAt(path.size() - 1);
	} else {
	  String name;
	  if(nextIndex != -1)
	    name = context.substring(currentIndex, nextIndex);
	  else {
	    name = context.substring(currentIndex);
	    if(name.equals(""))
	      break;
	  }

	  path.addElement(name);
	}
	
	if(nextIndex == -1) {
	  break;
	}
      }     
      currentIndex = nextIndex + 1;
    }

    StringBuffer buf = new StringBuffer("/");
    for(int i = 0; i < path.size(); i++){
      buf.append(path.elementAt(i));
      buf.append("/");
    }

    return buf.toString();
  }


  private DirectoryContext(String context){
    this.context = context;
  }
  
  public String getContext(){
    return context;
  }

  public static void main(String args[]){
    DirectoryContext context;
    String contexts[] = { "/server", 
			  "/server/httpd/../snmp/",
			  "../server/httpd/../snmp", 
			  "/server/httpd/child/../snmp/", 
			  "/server/httpd/../snmp/.."
    };
    for(int i = 0; i < contexts.length; i++){
      context = getDirectoryContext(contexts[i]);
      System.out.println(context.getContext());
    }
    
  }
}
