/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */
package org.locomotive.directory;

import java.util.*;

/**
 * Data structure for storing and looking up components in a specific scope.
 * 
 * <h3> Open issues </h3>
 * This class should be extended with flexible lookup mechanisms for 
 * directory contexts and components, so it can be used by and administrative
 * interface to servers.
 *
 * @author Gunnar Rnning (gunnar@candleweb.no).
 * @version $Id: ScopeStore.java,v 1.3 1999/05/05 10:19:32 gunnar Exp $
 */
public class ScopeStore {
  Hashtable directoryContexts = new Hashtable();

  /**
   * Get a specific component from this scope store.
   * @param context The directory context for the component.
   * @param componentName The component's name.
   * @return Returns the component if found, or null else.
   */
  public Object getComponent(DirectoryContext context, String componentName)
  {
    Hashtable compTable = (Hashtable) 
      directoryContexts.get(context.getContext());

    if(compTable != null){
      return compTable.get(componentName);
    }
    return null;
  }
  
  /**
   * Put a specific component into the scope store.
   * @param context The directory context for the component.
   * @param componentName The component's name.
   * @param component The component itself.
   */
  public void putComponent(DirectoryContext context, String componentName, 
			   Object component)
  {
    Hashtable compTable = (Hashtable) 
      directoryContexts.get(context.getContext());

    if(compTable == null){
      synchronized(this){
	if(compTable == null){
	  compTable = new Hashtable();
	  directoryContexts.put(context.getContext(), compTable);
	}
      }      
    }
    compTable.put(componentName, component);
    
  }
  
  
}
