package org.locomotive.directory.loco;

/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */

import java.util.*;
import org.locomotive.directory.*; 
import org.locomotive.loco.*;
import org.locomotive.server.*;
import org.locomotive.loco.servlet.*;
import javax.servlet.http.*;
import javax.servlet.*;


/**
 * This is an example application showing how you can use Bean Directory
 * to start up an entire application as defined by a set of configuration
 * files. 
 * <p>
 * The class bootstrap the NameService and starts up the ServiceList, 
 * which in turn will startup all the services in the sequence defined by
 * "/ServiceList.cfg".
 *
 * @author Gunnar Rnning (gunnar@candleweb.no).
 * @version $Id: BeanDirectory.java,v 1.2 1999/05/05 10:19:32 gunnar Exp $
 */
public class BeanDirectory implements ScopeLookup, DirectoryEventListener {
  static BeanDirectory beanDirectory;
  static NameService nameService;

  private static boolean isRunning = false;

  private BeanDirectory(){
  }
  
  public static BeanDirectory getInstance(){
    if(beanDirectory == null){
      beanDirectory = new BeanDirectory();
    }
    return beanDirectory;
  }

  public static Object resolve(HttpServletRequest request, String name){
    ScopeLookup scopeLookup = new ScopeLookupImpl(request, beanDirectory);
    return nameService.resolve(scopeLookup, name);    
  }

  public static Object resolve(String name){
    ScopeLookup scopeLookup = new ScopeLookupImpl(null, beanDirectory);
    return nameService.resolve(scopeLookup, name);
  }
  
  public void log(int level, String message){
    System.err.println(message);
  }

  public void directoryEvent(DirectoryEvent e){
    String fullName = "";
    if(e.getBeanConfig() != null)
      fullName = e.getBeanConfig().getFullName();
    String property = "";
    if(e.getBeanProperty() != null)
      property = "." + e.getBeanProperty().getName();
      
    log(e.getType(), fullName + property + ":" + e.getMessage());
    if(e.getThrowable() != null){
	System.err.println(e.getThrowable().toString());
	e.getThrowable().printStackTrace();
    }
  }

  public Object getScope(String scopeType){
    if(scopeType == null || scopeType.equals("global"))
      return this;
    else
      return null;
  }

  public void startup(String configPath[]){
    nameService = new NameService(configPath);
    nameService.addDirectoryEventListener(this);
    // Use resolve so we can read the real configuration 
    // for the name service from the configuration files.
    nameService = (NameService) nameService.resolve(this, "/NameService");
    if(nameService == null){
      System.err.println("Could not locate /NameService component");
      System.exit(-1);
    }
    nameService.addDirectoryEventListener(this);
    
    // register the name service with itself, so we can lookup the name service
    // component also.
    nameService.registerComponent(this, "global", "/NameService", nameService);
    nameService.registerComponent(this, "global", "/BeanDirectory", this);

    // ServiceList defines a list of services which ultimately defines
    // our entire application.
    nameService.resolve(this, "/ServiceList");
  }

  public static final boolean isRunning(){
    return isRunning;
  }

  public static void main(String args[]){
    if(args.length == 0){
      System.err.println("Configuration path not specified.");
      return;
    }
    String configstr = args[0];

    StringTokenizer st = new StringTokenizer(configstr, ";", false);
    String configPath[] = new String[st.countTokens()];

    int i = 0;
    while(st.hasMoreElements()){
      configPath[i] = st.nextToken();
      i++;
    }

    BeanDirectory beanDirectory = BeanDirectory.getInstance();
    beanDirectory.startup(configPath);

    // The first argument is our config path, so strip that before
    // calling Loco.
    String newArgs[] = new String[args.length - 1];
    for(i = 1; i < args.length; i++){
      newArgs[i - 1] = args[i];
    }

    isRunning = true;

    Loco.main(newArgs);

  }
}

