package org.locomotive.directory.loco;

/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Candleweb AS 
 * Portions created by Candleweb AS are Copyright (C) 1999 Candleweb AS. 
 * All Rights reserved.
 */

import java.util.*;
import org.locomotive.directory.*; 
import org.locomotive.loco.*;
import org.locomotive.server.*;
import org.locomotive.loco.servlet.*;
import javax.servlet.http.*;
import javax.servlet.*;

/**
 * An implementation of the scope lookup interface that implements
 * scope lookup for request, session and global scoped components.
 *
 * @author Gunnar Rnning (gunnar@candleweb.no).
 * @version $Id: ScopeLookupImpl.java,v 1.2 1999/05/05 10:19:32 gunnar Exp $
 */
public class ScopeLookupImpl implements ScopeLookup {
  private HttpServletRequest request;
  private BeanDirectory beanDirectory;

  /**
   * Constructor for the scope lookup class.
   * @param request The request that is resolving the component, or null
   *                if the lookup is not done in the context of a request.
   * @param beanDirectory The BeanDirectory object that acts as the 
   *                      global scope.
   */
  public ScopeLookupImpl(HttpServletRequest request, 
			 BeanDirectory beanDirectory){
    this.request = request;
    this.beanDirectory = beanDirectory;
  }
  
  /**
   * Return the appropriate object for the requested scope.
   * @param scopeType The scope type to lookup.
   * @return Returns the object defining the scope of the scope type.
   */
  public Object getScope(String scopeType){
    if(scopeType == null || scopeType.equals("") || 
       scopeType.equals("global"))
    {  
      return beanDirectory;
    } else if(scopeType.equals("request")) {
      return request;
    } else if(scopeType.equals("session")) {
      if(request != null){
	Object o = request.getSession();
	if(o != null){
	  return o.toString().intern();
	}
	return null;
      }
      else
	return null; 
    } else {
      return null;
    }
  }

} 
