/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1999 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco;

import org.locomotive.server.*;
import org.locomotive.util.*;


/**
 * The config object for the Locomotive.  Each of the Locomotive configs
 * that can be set are listed and documented in the sample loco.conf file
 * included with each distribution.  To see what the current config
 * settings are, log into the Locomotive has an admin user and
 * request the srv URL (i.e. 'http://www.yourdomain.com/cgi-bin/locolink/srv').
 * To change any configs during runtime, use the changeConfig() method.
 * For more information, see the
 * <a href="http://www.leverage.com/docs/install/install_noframe.html#_Toc424012341">Locomotive Installation Guide</a> or the
 * <a href="http://www.leverage.com/docs/admin/admin_noframe.html#_Toc425673336">Locomotive Administrator's Guide</a>.
 */
public class LocoConfig extends org.locomotive.server.Config {
   /** Holds the database type.  Currently Oracle, Informix, Cloudscape,
    * MySQL, Solid, and PostGreSQL are supported.
    */
   public static int DB_TYPE = SQL.DB_TYPE_ORACLE;    // THIS IS THE DEFAULT


   /** This creates a LocoConfig object. It takes the same comman arguments
    * as its super class. */
   public LocoConfig (String args[], String defaultpath)
   throws IllegalArgumentException
   {
       super (args, defaultpath);
   }

    public LocoConfig ()
    {
	super();
    }
    
    public String getSystemTag () 
    { 
	return genericGetString("SYSTEM_TAG");
    }
    
    public void setSystemTag (String val)
    {
	genericSetString("SYSTEM_TAG", val);
    }
    
    public String getSystemInstanceId ()
    {
	return genericGetString("SYSTEM_INSTANCE_ID");
    }
    
    public void setSystemInstanceId (String val)
    {
	genericSetString("SYSTEM_INSTANCE_ID", val);
    }

    public Integer getLocoInstId ()
    {
	return genericGetInteger("LOCO_INST_ID");
    }

    public void setLocoInstId (Integer val)
    {
	genericSetInteger("LOCO_INST_ID", val);
    }

    public String getLocoUrlMain ()
    {
	return genericGetString("LOCO_URL_MAIN");
    }

    public void setLocoUrlMain (String val)
    {
	genericSetString("LOCO_URL_MAIN", val);
    }

    public String getLocoUrlHomepage ()
    {
	return genericGetString("LOCO_URL_HOMEPAGE");
    }

    public void setLocoUrlHomepage (String val)
    {
	genericSetString("LOCO_URL_HOMEPAGE", val);
    }
    
    public String getNsapiTunnelUriTokenPrefix ()
    {
	return genericGetString("NSAPI_TUNNEL_URI_TOKEN_PREFIX");
    }

    public void setNsapiTunnelUriTokenPrefix (String val)
    {
	genericSetString("NSAPI_TUNNEL_URI_TOKEN_PREFIX", val);
    }
    
    public String getSystemServerRootPath ()
    {
	return genericGetString("SYSTEM_SERVER_ROOT_PATH");
    }

    public void setSystemServerRootPath (String val)
    {
	genericSetString("SYSTEM_SERVER_ROOT_PATH", val);
    }
    
    public String getLocoTemplateRootPath ()
    {
	return genericGetString("LOCO_TEMPLATE_ROOT_PATH");
    }

    public void setLocoTemplateRootPath (String val)
    {
	genericSetString("LOCO_TEMPLATE_ROOT_PATH", val);
    }
    
    public String getLocoUploadPath ()
    {
	return genericGetString("LOCO_UPLOAD_PATH");
    }

    public void setLocoUploadPath (String val)
    {
	genericSetString("LOCO_UPLOAD_PATH", val);
    }
    
    public String getServiceClasspath ()
    {
	return genericGetString("SERVICE_CLASSPATH");
    }

    public void setServiceClasspath (String val)
    {
	genericSetString("SERVICE_CLASSPATH", val);
    }
    
    public Integer getLocoPort ()
    {
	return genericGetInteger("LOCO_PORT");
    }

    public void setLocoPort (Integer val)
    {
	genericSetInteger("LOCO_PORT", val);
    }

    public Integer getLocoPortBacklog ()
    {
	return genericGetInteger("LOCO_PORT_BACKLOG");
    }

    public void setLocoPortBacklog (Integer val)
    {
	genericSetInteger("LOCO_PORT_BACKLOG", val);
    }

    
   /** This establishes the default setting for all the Locomotive configs.
    */
   public void setupDefaults()
   {
      // system tag for email and cookies
      addConfig ("SYSTEM_TAG", "DEF_SYS_TAG",
                 forstring, "System Identity Tag");
      addConfig ("SYSTEM_INSTANCE_ID", "sv_one",
                 forstring, "System Instance Identifier");
      addConfig ("SYSTEM_SERVER_ROOT_PATH", "/usr/local/loco",
                 forstring, "Basedir for all file paths - absolute");

      addConfig ("LOCO_PORT", "5421",
                 forint, "TCP port for Request servicing");
      addConfig ("LOCO_PORT_BACKLOG", "5",
                 forint, "TCP Request servicing port backlog (listen)");
      addConfig ("LOCO_INST_ID", "1",
                 forint, "Instance ID for virtual sessions routing");


      // threads
      addConfig ("LOCO_MAX_POOLED_THREADS", "16",
                 forint, "Maximum number Pooled Threads for RequestManagers");
      addConfig ("LOCO_MIN_POOLED_THREADS", "2",
                 forint, "Minimum number Pooled Threads for RequestManagers");

      // session
      addConfig("USE_CACHED_SESSION_OBJECTS", "TRUE", forstring,
                "Whether to cache session objects in the VM, as opposed to " +
                "saving them to the database.");
      addConfig ("LOCO_SESSION_EXPIRE_TIME", "3600",
                 forint, "Dormant seconds allowed before session expires");
      addConfig("LOCO_SESSION_AUTOHANDLE", "YES", forstring,
                "Switch for toggling automatic session expiration routines "+
                "in the RequestManager, defaults to yes");
      addConfig("PERSISTENT_SESSION_COOKIE", "NO", forstring,
                "Switch for permanant session cookies. If yes, visitors " +
                "can return authenticated to the site without logging in" +
                "after a browser (or system) restart");
      addConfig("BSTAMP_COOKIE", "YES", forstring,
                "Yes/no switch for setting of a persistent serialized " +
                "cookie (stamp) for each individual browser");

      // templates
      addConfig ("LOCO_TEMPLATE_ROOT_PATH", "templates",
                 forstring,"Application Server Template Root, relative path");
      addConfig ("LOCO_CHECK_NEW_TEMPLATE_FREQ", "1",
                 forint, "If we're caching templates in memory, we need to" +
                 " check when they've changed on disk.  This is how many" +
                 " times we SKIP checking before checking again -- to save" +
                 " time.  Set it to '1' to check every time.");

      // if the database type is changed here, change the default for
      // DB_TYPE above too
      addConfig("DB_TYPE", "oracle", forstring, "The kind of database. " +
                " Currently Oracle, Informix, and Solid are supported.");

      // For DBConnectionPOOL
      addConfig ("DB_AUTOCONNECT", "YES", forstring, "Whether LOCO should " +
                 "automatically connect to the database on startup.");
      addConfig ("DB_DRIVER_CLASSNAME", "oracle.jdbc.driver.OracleDriver",
                 forstring, "Application Server database driver classname");
      addConfig ("DB_CONNPOOL_URL", "jdbc:oracle:thin:@127.0.0.1:1527:DFLT",
                 forstring, "Application Server database connect url");
      addConfig ("DB_CONNPOOL_USERNAME", "loco",
                 forstring, "Application Server database username");
      addConfig ("DB_CONNPOOL_PASSWORD", "loco_pw",
                 forstring, "Application server database password");
      addConfig ("DB_CONNPOOL_MIN_CONNECTIONS", "4",
                 forint, "Minimum number of pre-opened DB connections");
      addConfig ("DB_CONNPOOL_MAX_CONNECTIONS", "16",
                 forint, "Grow limit for pre-opened DB connections");

      // Logs
      addConfig ("LOG_LEVEL", "9",
                 forint, "Locomotive Server Log log verbosity value");
      addConfig ("LOG_PATH", "logs/Loco-%.log",
                 forstring, "Relative path for Locomotive Server Logs");

      addConfig ("EVENT_LOG_LEVEL", "10",
                 forint, "Locomotive events logging verbosity");
      addConfig ("EVENT_LOG_PATH", "logs/events-%.log",
                 forstring, "path to Locomotive event log");

      addConfig ("NSAPI_TUNNEL_URI_TOKEN_PREFIX", "/ll_",
                 forstring, "URI token prefix used to indicate to the " +
                 "NSAPI that it should call the tunnel");

      // services
      addConfig ("HANDLER_ROUTING_TABLE_PATH", "config/hrt.conf",
                 forstring, "Handler Routing Table Path");
      addConfig("HANDLER_URI_TOKEN_PREFIX", "handler",
                forstring, "All URIs that point to handlers must begin " +
                "with this token");

      addConfig ("SERVLET_ROUTING_TABLE_PATH", "config/servlet.conf",
                 forstring, "Servlet Routing Table Path");
      addConfig("SERVLET_URI_TOKEN_PREFIX", "servlet",
                forstring, "All URIs that point to servlets must begin " +
                "with this token");

      addConfig("SERVICE_DEFAULT", "handler", forstring,
                "the default service.  Must be either HANDLER or SERVLET.");

      addConfig("SERVICE_CLASSPATH", "classes", forstring,
                "Where service classes are located, if outside the " +
                "System CLASSPATH");
      addConfig("AUTO_RELOAD_SERVICES", "NO", forstring,
                "Whether or not to reload services if their class file " +
                "modification dates are more recent than the file the " +
                "current service was loaded from");


      // URL to find myself
      addConfig ("LOCO_URL_MAIN", "http://www.leverage.com/cgi-bin/locolink",
                 forstring, "Full URL to Locomotive default handler");
      addConfig ("LOCO_URL_HOMEPAGE", "default",
                 forstring, "relative URL for Locomotive homepage");
      addConfig ("LOCO_COOKIE_DOMAIN_MAIN", ".leverage.com",
                 forstring, "Domain for cookies ste by the Locomotive");

      // smtp client
      addConfig ("SMTP_SERVER_NAME", "localhost",
                 forstring, "Hostname for outgoing mail SMTP server");
      addConfig ("SMTP_SERVER_PORT", "25",
                 forint, "Port for outgoing mail SMTP server");
      addConfig ("SMTP_SERVER_TIMEOUT", "60",
                 forint, "SMTP client delivery timeout");
      addConfig ("SMTP_HELO_NAMESTRING", "foo.company.com",
                 forstring, "Name string used by SMTP client for SMTP" +
                 " helo handshake");

      // form parsing
      addConfig("LOCO_UPLOAD_PATH", "upload", forstring,
                "The root relative path to the upload directory");
      addConfig("NORMAL_FORM_CONTENT_LENGTH", "131072", forint,
                "The maximum length of normal forms posted to the server");
      addConfig("MULTIPART_FORM_CONTENT_LENGTH", "10485760", forint,
                "The maximum length of multipart forms posted to the server");
      addConfig("LOCO_ACCEPT_IP", "127.0.0.0", forstring,
                "The network ip to allow connections from");
      addConfig("LOCO_ACCEPT_SUBNET", "255.255.255.0", forstring,
                "the size of the subnet to allow");

      // Here are some new config variables for the new GlobalDBCache:

      addConfig ("CACHE_DEFAULT_AUTO_SAVE_INTERVAL", "60", forint,
                 "The default frequency for an object's auto-save " +
                 "interval, which is how often to check whether a " +
                 "particular object has dirty data, expressed in " +
                 "number of seconds");

      addConfig ("CACHE_MAX_AUTO_SAVE_INTERVAL", "3600", forint,
                 "The maximum auto-save interval for any object in " +
                 "this cache, expressed in number of seconds");

      addConfig ("CACHE_AUTO_SAVE_SLEEP_INTERVAL", "20", forint,
                 "How long the AutoSaveThread sleeps before waking " +
                 "up to check dirty data in objects, expressed in " +
                 "number of seconds");

      addConfig ("CACHE_DEFAULT_EXPIRATION_TIME", "3600", forint,
                 "The default frequency for how long an object can " +
                 "remain in the cache (used primarily by the " +
                 "ExpirationThread) expressed in number of seconds");

      addConfig ("CACHE_MAX_EXPIRATION_TIME", "3600", forint,
                 "The maximum time that any object can remain in " +
                 "this cache, expressed in number of seconds");

      addConfig ("CACHE_EXPIRATION_SLEEP_INTERVAL", "10", forint,
                 "How long the ExpirationThread sleeps before " +
                 "waking up to check whether objects have expired, " +
                 "expressed in number of seconds");

      // embedded web servers (default to NONE in this compiled class)
      addConfig("WEB_SERVER_INSTANCE_NAMES", "NONE", forstring,
                "Instance names of any embedded web servers");

      // David Warnock change to support alternative DBConnectionPools
      // Allow a different DBConnectionPool Class. The default is as before ie DBConnectionPool 
      // if left as the default then bhaviour is unchanged. Otherwise define a fully qualified 
      // class name (ie including package) that is a subclass of DBConnectionPool
      addConfig("DB_CONN_POOL_CLASSNAME", "org.locomotive.server.DBConnectionPool", forstring,
                "Class name of DBConnectionPool. Must be a subclass of DBConnectionPool");

   }

   /**
    * a package friendly method that outputs an html table with rows containing
    * the configurations names and current values.
    * Used by the ServerHandler status display
    */
   String toHTMLRows ()
   {
/*	String names[] = {"SYSTEM_TAG", "SYSTEM_INSTANCE_ID",
           "SYSTEM_SERVER_ROOT_PATH",
           "HANDLER_ROUTING_TABLE_PATH", "LOCO_PORT",
           "LOCO_PORT_BACKLOG", "LOCO_MAX_POOLED_THREADS",
           "LOCO_MIN_POOLED_THREADS", "LOCO_SESSION_EXPIRE_TIME",
           "LOCO_SESSION_AUTOHANDLE",
           "LOCO_CHECK_NEW_TEMPLATE_FREQ",
           "DB_AUTOCONNECT", "DB_TYPE",
           "DB_DRIVER_CLASSNAME", "DB_CONNPOOL_URL",
           "DB_CONNPOOL_MIN_CONNECTIONS",
           "DB_CONNPOOL_MAX_CONNECTIONS",
           "LOCO_SESSION_AUTOHANDLE",
           "LOG_LEVEL", "EVENT_LOG_LEVEL",
           "LOCO_URL_MAIN", "LOCO_URL_HOMEPAGE",
           "LOCO_COOKIE_DOMAIN_MAIN"};
           */

      StringBuffer sb = new StringBuffer ();
      // sb.append("<TABLE>\n");
      java.util.Enumeration e = getConfigNamesEnumeration();
      while (e.hasMoreElements()) {
         String name = (String) e.nextElement();
         sb.append ("\t<TR><TD>");
         sb.append (name);
         sb.append ("</TD><TD align=right>");
         sb.append (StringUtil.htmlQuoteString (getString (name)));
         sb.append ("</TD></TR>\n");
      }
//	sb.append("</TABLE>");
      return (sb.toString());
   }


   /** changes the value of a config at runtime.  Checks for change in
    * DB_TYPE, and, if changed, resets the DB_TYPE variable. Also checks to
    * make sure SERVICE_DEFAULT is either HANDLER or SERVLET.
    * @param name: the name of the config
    * @param value the new value for the config.
    * @returns the value of changed config, which may have been altered
    *          to allow it to match the config's constraints.
    * @exception ConfigNotFoundException if the config does not exist.
    * @exception ConfigInitializationException if the config value does not
    *            match the required constraints.  For DB_TYPE, value must be
    *            either 'Oracle', 'Informix', 'Cloudscape', MySQL, or 'Solid'.
    *            For SERVICE_DEFAULT, this must be either 'Handler' or
    *            'Servlet'.
    */
   public String changeConfig (String name, String value)
   throws ConfigNotFoundException, ConfigInitializationException

   {
      if (name.equals("DB_TYPE") ) {
         setDBType(value); // will throw a configInitializationException if
                           // config is not found
      } else if (name.equals("SERVICE_DEFAULT") ) {
         if ( ! (value.equalsIgnoreCase("handler") ||
                 value.equalsIgnoreCase("servlet")) ) {
            throw new ConfigInitializationException(
                                                   "Service type not supported: " + value);
         }
      }

      return super.changeConfig(name, value);
   }

   private void setDBType(String value) throws ConfigInitializationException
   {
      if (value.equalsIgnoreCase("oracle")) {
         DB_TYPE = SQL.DB_TYPE_ORACLE;
      } else if (value.equalsIgnoreCase("informix")) {
         DB_TYPE = SQL.DB_TYPE_INFORMIX;
      } else if (value.equalsIgnoreCase("solid")) {
         DB_TYPE = SQL.DB_TYPE_SOLID;
      } else if (value.equalsIgnoreCase("cloudscape")) {
         DB_TYPE = SQL.DB_TYPE_CLOUDSCAPE;
      } else if (value.equalsIgnoreCase("mysql")) {
         DB_TYPE = SQL.DB_TYPE_MYSQL;
      } else if (value.equalsIgnoreCase("postgresql")) {
         DB_TYPE = SQL.DB_TYPE_POSTGRESQL;
      } else if (value.equalsIgnoreCase("interbase")) {
         DB_TYPE = SQL.DB_TYPE_INTERBASE;
      } else {
         throw new ConfigInitializationException(
                                                "Database type not supported: " + value);
      }
   }
}
