/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.handler;

import org.locomotive.loco.*;
import org.locomotive.server.Log;


/**
 * A sample default handler.  This is the handler that is invoked when 
 * a handler is not specified on the url.  It is autoloaded 
 * at server boot, but can be overridden in the HandlerRoutingTable.
 */
public class DefaultHandler implements Handler
{
    private HandlerData hd;
    
    /**
     * This is the entry point for the server upon receipt of a new 
     * request.  
     * @param hd_in  a HandlerData object, containing request specific 
     *                information
     */
    public void handleRequest (HandlerData hd_in)
    {
	hd = hd_in;

	// call the Loco library call for standard insertions.  
	StandardInsertions.insertAll (hd);

	
	String response = 
	    Loco.evaluateTemplate("default.tmpl", 
				 hd.subs);

	// is the template is missing ?
	if (response == null || response.length() < 1)
	{
	    hd.resp.addString ("<HTML>\n<BODY>\n<H1>Hey</H1>\n");
	    hd.resp.addString ("This is the sample default handler<BR>\n");
	    hd.resp.addString ("Session id is: " + hd.session.getID() + 
			       "<BR>\n");
	    hd.resp.addString ("Have a nice day!\n");
	    hd.resp.addString ("</BODY>\n</HTML>\n");
	    hd.resp.flush();
	}
	else
	{
	    hd.resp.addString (response);
	    hd.resp.flush (hd.server_log);
	}
    }
    
    /**
     * Called during Handler initiation.  This particular method simply 
     * returns true
     * @param initlog - a Log instance used to log details of initiation
     */
    public boolean init (Log initlog)
    {
	return (true);
    }


    /**
     * Called during handler shutdown.  Currently doesn't do anything
     * @param initlog - a Log instance used to log details of initiation
     */
    public boolean shutdown(Log initlog)
    {
	return true;
    }



}

