/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.handler;

import org.locomotive.server.Log;

/**
 * The Locomotive requires a generic way to instantiate a class and
 * have it handle a request; the Handler interface provides it.  Any
 * class that implements the Handler interface can be called by the
 * Locotive to handle a request.  The way the Locomotive knows to
 * call a particular Handler is through a handler token mapping
 * in the hrt.conf file, which matches a url token with Class name.
 * <BR>In the hrt.conf file, you can also specify a path to a properties 
 * file to be loaded for this particular handler.  The properties file
 * can be accessed using the handler_config field in HandlerData or
 * with the Loco.getHandlerProperties(Handler h) call.
 */
public interface Handler
{
    /**
     * this is the entry point for the server for a specific Request.
     * This class is fetched by the HandlerRoutingTable if the 
     * pattern matches (specified in the HandlerRoutingTable config file)
     */
    public void handleRequest (HandlerData hd_in);
    
    /**
     * Called by the HandlerRoutingTable when loaded/linked.
     * Might be useful for loading external data into staticly 
     * available caches.  (Note: ideally, this should be a static method,
     * but Java doesn't allow us to declare static methods in Interfaces)
     */
    public boolean init (Log server_log);

    /**
     * Called by the Locomotive when the server is exiting.
     * You can also call init() and shutdown() yourself to reload this
     * class.
     */
    public boolean shutdown (Log server_log);
}
