/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.handler;

import org.locomotive.loco.*;
import org.locomotive.server.Log;

/** 
 * This handler is invoked when a url cannot be mapped to a handler by
 * the Locomotive.  It returns the template
 * LOCO_TEMPLATE_ROOT_PATH/error/unknown_handler_option.tmpl, or, if
 * that template is missing, a basic warning message 
 */
public class UnknownHandler implements Handler
{
    
    /**
     * this is the entry point for the server.  <BR> Uses the
     * LOCO_TEMPLATE_ROOT_PATH/error/unknown_handler_option.tmpl template.
     * LOCO_TEMPLATE_ROOT_PATH is the template directory, as specified in
     * the Loco configuration file.
     */
    public void handleRequest (HandlerData hd)
    {
	// call the Loco library call for standard insertions.  
	StandardInsertions.insertAll (hd);
	
	if ( hd.url_tokens.length > 0 && hd.url_tokens[0] != null)
	    hd.subs.put("HANDLER_URLTOKEN", hd.url_tokens[0]);

	String response = 
	    Loco.evaluateTemplate("error/unknown_handler_option.tmpl", 
				 hd.subs);

	// is the template is missing ?
	if (response == null || response.length() < 1)
	{
	    response = 
		"<HTML>\n<HEAD><TITLE>Locomotive - Handler Unknown " + 
		"</TITLE></HEAD>\n" + 
		"<BODY>\n\n<H1>Unknown Handler</H1>\n" + 
		"The handler specified could not be found<BR>\n" + 
		"</BODY>\n</HTML>";
	}

	hd.resp.addString (response);
	hd.resp.flush (hd.server_log);
    }
    
    /**
     * we don't load anything, so just return.
     */
    public boolean init (Log initlog)
    {
	return (true);
    }


    /** there's nothing to do here but return happily */
    public boolean shutdown(org.locomotive.server.Log log)
    {
	return true;
    }




}
