/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.perm;

import org.locomotive.server.Log;
import org.locomotive.util.StringUtil;
import org.locomotive.loco.*;
import org.locomotive.loco.handler.*;
import java.sql.*;
import java.util.*;

/**
 * Handler for the optional permissions module of the Locomotive.
 */
public class PermHandler implements Handler
{
    private HandlerData hd;
    
    /**
     * This is the entry point for the server upon receipt of a new 
     * request.  
     * @param hd_in  a HandlerData object, containing request specific 
     *                information
     */
    public void handleRequest (HandlerData hd_in)
    {
	hd = hd_in;
	if(hd.session.isAnonymous())
	{
	    hd.resp.addString ("<HTML>\n<BODY>\n<H1>Permissions</H1>\n");
	    hd.resp.addString ("This is the default permissions " +
			       "handler.<BR><BR>\n");
	    hd.resp.addString ("You have no permissions since you are not " +
			       "logged in.<BR>\n");
	    hd.resp.addString ("</BODY>\n</HTML>\n");
	    hd.resp.flush();
	}
	else
	{
	    PermissionSet pSet = null;
	    
	    try
	    {
		pSet = UserPerm.getPermissions(hd.user.getUserid(), hd.conn);
	    }
	    catch(SQLException se)
	    {
		hd.server_log.log(0, "An error occured accessing permissions" +
				  " for user#" + hd.user.getUserid() + ": " +
				  StringUtil.exceptionStackTrace(se));
		hd.resp.addString ("<HTML>\n<BODY>\n<H1>Error</H1>\n");
		hd.resp.addString ("Failed to retrieve user permissions.\n");
		hd.resp.addString ("</BODY>\n</HTML>\n");
		hd.resp.flush();
		return;
	    }
	    
	    hd.resp.addString ("<HTML>\n<BODY>\n<H1>Permissions</H1>\n");
	    hd.resp.addString ("This is the default permissions " +
			       "handler.<BR><BR>\n");

	    hd.resp.addString ("<H3>Permissions for " +
			       hd.user.getUsername() + ":</H3>\n");
 

	    hd.resp.addString ("\nInteger Representation: {");

	    Vector ints = pSet.toInts();
	    Enumeration e = ints.elements();
	    int i = 0;
	    while(e.hasMoreElements())
	    {
		if (i > 0 ) hd.resp.addString(", ");
		
		hd.resp.addString ( ((Integer)e.nextElement()).toString() );
		i++;
	    }	    
	    hd.resp.addString ("}<BR><BR>\n");

	    Vector perm_names = pSet.getSetPerms();

	    if (perm_names != null && !perm_names.isEmpty())
	    {
		hd.resp.addString ("\nPermission Names:<BR>\n");

		e = perm_names.elements();
		while(e.hasMoreElements())
		{
		    hd.resp.addString ((String)e.nextElement() + "<BR>\n");
		}
	    }
	    
	    hd.resp.addString ("\n<BR>Permission Bits Set: " +pSet.toString() +
				   "<BR>\n");

	    hd.resp.addString ("</BODY>\n</HTML>\n");
	    hd.resp.flush();
	}
    }
    
    /**
     * Called during Handler initiation.  Initializes the PermissionSet class.
     * @param initlog - a Log instance used to log details of initiation
     */
    public boolean init (Log initlog)
    {
	Connection conn = Loco.getDBConnection();

	try
	{
	    Permission.initializeCache(conn);
	}
	catch(SQLException se)
	{
	    initlog.log(0, "An error occurred initializing the permissions  " +
		     "in the database: " + StringUtil.exceptionStackTrace(se));
	}
	finally
	{
	    Loco.putBackDBConnection(conn);
	}

	return (true);
    }


    /**
     * The required shutdown method, as defined in org.locomotive.loco.Handler.  In
     * this example, we don't actually need to do anything.  In the future we'll
     * probably make sure the current state of the cache is saved, though that
     * shouldn't really be necessary.
     * @see org.locomotive.loco.Handler#shutdown
     */
    public boolean shutdown (Log log)
    {
	return (true);
    }


}
