/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.perm;

import org.locomotive.server.Log;
import org.locomotive.util.StringUtil;
import org.locomotive.loco.*;
//import org.locomotive.loco.servlet.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Servlet for the optional permissions module of the Locomotive.
 */
public class PermServlet extends HttpServlet
{
    /**
     * This is the entry point for the server upon receipt of a new 
     * request.  
     * @param hd_in  a HandlerData object, containing request specific 
     *                information
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws IOException, ServletException
    {
	PrintWriter pr = resp.getWriter();
	Session session = (Session) req.getSession();

	if(session.isAnonymous())
	{
	    pr.print ("<HTML>\n<BODY>\n<H1>Permissions</H1>\n");
	    pr.print ("This is the default permissions " +
			       "handler.<BR><BR>\n");
	    pr.print ("You have no permissions since you are not " +
			       "logged in.<BR>\n");
	    pr.print ("</BODY>\n</HTML>\n");
	    pr.flush();
	}
	else
	{
	    PermissionSet pSet = null;
	    Connection conn = 
		(Connection) req.getAttribute("locomotive.connection");
	    User user = (User) session.getValue("locomotive.user");
	    
	    try
	    {
		pSet = UserPerm.getPermissions(user.getUserid(), conn);
	    }
	    catch(SQLException se)
	    {
		Loco.log(0, "An error occured accessing permissions" +
				  " for user#" + user.getUserid() + ": " +
				  StringUtil.exceptionStackTrace(se));
		pr.print ("<HTML>\n<BODY>\n<H1>Error</H1>\n");
		pr.print ("Failed to retrieve user permissions.\n");
		pr.print ("</BODY>\n</HTML>\n");
		pr.flush();
		return;
	    }
	    
	    pr.print ("<HTML>\n<BODY>\n<H1>Permissions</H1>\n");
	    pr.print ("This is the default permissions " +
			       "handler.<BR><BR>\n");

	    pr.print ("<H3>Permissions for " +
		      user.getUsername() + ":</H3>\n");
 

	    pr.print ("\nInteger Representation: {");

	    Vector ints = pSet.toInts();
	    Enumeration e = ints.elements();
	    int i = 0;
	    while(e.hasMoreElements())
	    {
		if (i > 0 ) pr.print(", ");
		
		pr.print ( ((Integer)e.nextElement()).toString() );
		i++;
	    }	    
	    pr.print ("}<BR><BR>\n");

	    Vector perm_names = pSet.getSetPerms();

	    if (perm_names != null && !perm_names.isEmpty())
	    {
		pr.print ("\nPermission Names:<BR>\n");

		e = perm_names.elements();
		while(e.hasMoreElements())
		{
		    pr.print ((String)e.nextElement() + "<BR>\n");
		}
	    }
	    
	    pr.print ("\n<BR>Permission Bits Set: " +pSet.toString() +
				   "<BR>\n");

	    pr.print ("</BODY>\n</HTML>\n");
	    pr.flush();
	}
    }
    
    /**
     * Called during Handler initiation.  Initializes the PermissionSet class.
     * @param initlog - a Log instance used to log details of initiation
     */
    public void init (ServletConfig config) throws ServletException
    {
	Connection conn = Loco.getDBConnection();

	try
	{
	    Permission.initializeCache(conn);
	}
	catch(SQLException se)
	{
	    Loco.log(0, "An error occurred initializing the permissions  " +
		     "in the database: " + StringUtil.exceptionStackTrace(se));
	    throw new ServletException(se);
	}
	finally
	{
	    Loco.putBackDBConnection(conn);
	}
    }


}
