/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.perm;

import java.lang.*;
import java.io.*;
import java.util.*;

/**
 * Provides a data structure for a role (a set of permissions).  Essentially
 * is a subclass of PermissionSet extended with a name and an id.
 */
public class Role extends PermissionSet implements Serializable
{
    // Fields (accessed through the interface)

    private String name;
    private int id = -1;

    // Constructors
   
    /** 
     * Creates a role associated with a name.  This constructor is used
     * before inserting the role into the database, which will assign it
     * a unique id.
     *
     * @param roleName name of role to be created
     */
    public Role(String roleName)
    {
	super();
	name = roleName;
    }

    /** 
     * Creates a role with complete role information.  This constructor
     * is used when the database is loading role information into the
     * Role data structure.
     *
     * @param roleName name of role
     * @param roleId   id of role
     * @param bits     Vector of Integers representing the permission bitfield
     */
    public Role(String roleName, int roleId, Vector ints)
    {
	name = roleName;
	id = roleId;
	setBitfield(ints);
    }

    // Methods

    /** 
     * Returns the name of the role.
     *
     * @return String name of the role
     */
    public String getName() 
    {
	return name;
    }

    /**
     * Returns the id of the role.
     *
     * @return int id of the role
     */
    public int getId() 
    {
	return id;
    }

    /**
     * Sets the id of the role.
     *
     * @param roleId id of role
     */
    public void setId(int roleId)
    {
	id = roleId;
    }

}



