/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.servlet;

import org.locomotive.loco.*;
import org.locomotive.loco.handler.*;
import java.io.*;
import javax.servlet.*;


/** 
 * Performs all the required functions for forwarding or including
 * a request for the present request
 */
public class LocoRequestDispatcher implements RequestDispatcher
{
    
    private String request_uri;

    
    /**
     * Basic constructor should only be called in 
     * ServletRoutingTable.getRequestDispatcher
     */
    LocoRequestDispatcher(String request_uri, LocoServletContext c)
    {
	this.request_uri = request_uri;
    }


    /**
     * This will reset the necessary variables in request, then
     * forward the existing request to another service.
     * @throw IllegalArgumentException if req.getInputStream() or
     *        req.getWriter() has already been called.
     */
    public void forward(ServletRequest req, ServletResponse resp)
	throws IOException, ServletException
    {
	LocoServletRequest lreq = (LocoServletRequest) req;

	HandlerData hd = lreq.generateForward();
	hd.resp = (Response) resp;

	String script_name = lreq.getScriptName();
	if (request_uri.startsWith(script_name) &&
	    request_uri.length() >= script_name.length())
	{
	    request_uri = request_uri.substring(script_name.length());
	}

	Loco.redirect(request_uri, hd);
    }


    /**
     * This will include the contents of the servlet which is invoked to
     * be included in the parent servlet's response.
     */
    public void include(ServletRequest req, ServletResponse resp)
	throws IOException, ServletException
    {
	LocoServletRequest lreq = (LocoServletRequest) req;
	HandlerData hd = lreq.generateForward();

	hd.resp  = ( (LocoServletResponse) resp).generateInclude();

	String script_name = lreq.getScriptName();
	if (request_uri.startsWith(script_name) &&
	    request_uri.length() >= script_name.length())
	{
	    request_uri = request_uri.substring(script_name.length());
	}

	Loco.redirect(request_uri, hd);
    }

}



