/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 2000 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.servlet;

import org.locomotive.loco.*;

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

public class LocoServletConfig implements ServletConfig
{
    private Properties props;
    private ServletRoutingTable srt;

    protected LocoServletConfig (Properties p, ServletRoutingTable srt)
    {
	if (p == null) this.props = new Properties();
	else this.props = p;

	this.srt = srt;
    }

    /** 
     * Returns the value of the init parameter specified by the name.
     * Returns null if a parameter with that name doesn't exist.
     */
    public String getInitParameter(String name)
    {
	//Loco.log(9, "Debug: getting property: " + name + 
	//         " value: " + ((String) props.get(name)));
	return ((String) props.get(name));
    }

    /**
     * Returns an enumeration of the names of all the initialization
     * parameters.  It returns all the names in the main server
     * Config object using Loco.getConfig().getConfigNamesEnumeration().
     */
    public Enumeration getInitParameterNames()
    {
	return(props.keys());
    }

    /**
     * Returns the servlet context object for this servlet.
     */
    public ServletContext getServletContext()
    {
	return new LocoServletContext(srt);
    }
}
