/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.servlet;

import java.io.*;
import org.locomotive.loco.*;
import org.locomotive.server.Log;
import javax.servlet.*;
import javax.servlet.http.*;

/** 
 * This servlet is invoked when a url cannot be mapped to a Servlet by
 * the Locomotive.  It returns the template
 * LOCO_TEMPLATE_ROOT_PATH/error/unknown_service_option.tmpl, or, if
 * that template is missing, a basic warning message 
 */
public class UnknownServlet extends LocoServlet
{
    
    /** Required to use locoServlet fields
    public void service (HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	super.service(req, resp);
    }


    /**
     * this is the entry point for the server.  <BR> Uses the
     * LOCO_TEMPLATE_ROOT_PATH/error/unknown_service_option.tmpl template.
     * LOCO_TEMPLATE_ROOT_PATH is the template directory, as specified in
     * the Loco configuration file.
     */
    public void doGet (HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	if ( url_tokens.length > 0 && url_tokens[0] != null)
	    steam_vars.put("HANDLER_URLTOKEN", url_tokens[0]);

	String response = 
	    Loco.evaluateTemplate("error/unknown_service_option.tmpl", 
				 steam_vars);

	// is the template is missing ?
	if (response == null || response.length() < 1)
	{
	    response = 
		"<HTML>\n<HEAD><TITLE>Locomotive - Servlet Unknown " + 
		"</TITLE></HEAD>\n" + 
		"<BODY>\n\n<H1>Unknown Servlet</H1>\n" + 
		"The service specified could not be found<BR>\n" + 
		"</BODY>\n</HTML>";
	}

	PrintWriter pw = resp.getWriter();
	pw.print(response);
	pw.flush();
    }
    

    public void doPost(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	doGet(req, resp);
    }

}
