/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

//dwq
// should all the newly separated db stuff be in a org.locomotive.loco.db package?
package org.locomotive.loco.store;

import org.locomotive.util.*;
import org.locomotive.loco.store.bstamp.*;
import org.locomotive.loco.store.session.*;
import org.locomotive.loco.store.user.*;
import org.locomotive.loco.Loco;

/**
 * This is a new factory to construct instances
 * of UserStore, SessionStore and BStampStore for the current
 * persistant store

 * Written by David Warnock
 */

public class StoreFactory {

  // SessionStore factory

	//dwq
	// I think it would be more flexible to store the classname for the
	// SessionStore in the configuration. Is that OK?
  public static SessionStore createSessionStore() {
    switch (Loco.getConfig().DB_TYPE) {
    case SQL.DB_TYPE_ORACLE:
      return new SessionStoreJdbcOracle();
    case SQL.DB_TYPE_INFORMIX:
      return new SessionStoreJdbcInformix();
    case SQL.DB_TYPE_SOLID:
      return new SessionStoreJdbcSolid();
    case SQL.DB_TYPE_CLOUDSCAPE:
      return new SessionStoreJdbcCloudscape();
    case SQL.DB_TYPE_MYSQL:
      return new SessionStoreJdbcMysql();
    case SQL.DB_TYPE_POSTGRESQL:
      return new SessionStoreJdbcPostgresql();
    case SQL.DB_TYPE_INTERBASE:
      return new SessionStoreJdbcInterbase();
    default:
      return null;
    }
  }

  public static UserStore createUserStore() {
    switch (Loco.getConfig().DB_TYPE) {
    case SQL.DB_TYPE_ORACLE:
      return new UserStoreJdbcOracle();
    case SQL.DB_TYPE_INFORMIX:
      return new UserStoreJdbcInformix();
    case SQL.DB_TYPE_SOLID:
      return new UserStoreJdbcSolid();
    case SQL.DB_TYPE_CLOUDSCAPE:
      return new UserStoreJdbcCloudscape();
    case SQL.DB_TYPE_MYSQL:
      return new UserStoreJdbcMysql();
    case SQL.DB_TYPE_POSTGRESQL:
      return new UserStoreJdbcPostgresql();
    case SQL.DB_TYPE_INTERBASE:
      return new UserStoreJdbcInterbase();
    default:
      return null;
    }
  }

  public static BStampStore createBStampStore() {
    switch (Loco.getConfig().DB_TYPE) {
    case SQL.DB_TYPE_ORACLE:
      return new BStampStoreJdbcOracle();
    case SQL.DB_TYPE_INFORMIX:
      return new BStampStoreJdbcInformix();
    case SQL.DB_TYPE_SOLID:
      return new BStampStoreJdbcSolid();
    case SQL.DB_TYPE_CLOUDSCAPE:
      return new BStampStoreJdbcCloudscape();
    case SQL.DB_TYPE_MYSQL:
      return new BStampStoreJdbcMysql();
    case SQL.DB_TYPE_POSTGRESQL:
      return new BStampStoreJdbcPostgresql();
    case SQL.DB_TYPE_INTERBASE:
      return new BStampStoreJdbcInterbase();
    default:
      return null;
    }
  }

}



