/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.loco.store.id;


import java.sql.Connection;
import java.math.BigInteger;


public abstract class LocoId {

  public static final int LOCO_SESSIONS = 0;
  public static final int LOCO_USERS    = 1;
  public static final int LOCO_BSTAMP   = 2;


  protected static LocoId instance = null;



  public static final LocoId getInstance() throws IdException {
    if (instance == null) {

      String cName = "org.locomotive.loco.store.id.LocoIdJdbc";

      try {

        synchronized(LocoId.class) {
          if (instance == null) {
            instance = (LocoId) Class.forName(cName).newInstance();
          }
        }

      } catch (ClassNotFoundException e) {
        throw new IdException(""+e);
      } catch (InstantiationException e) {
        throw new IdException(""+e);
      } catch (IllegalAccessException e) {
        throw new IdException(""+e);
      }
    }

    return instance;
  }



  // this is to make the compiler see the jdbc class which it won't
  // do if we only ever refer to it via Class.forName!!!!  :-o
  // this never gets called
  // any other ideas?
  private void keepCompilerHappy() {
    new LocoIdJdbc();
  }





  public final BigInteger getNextId(int type, Connection con) throws IdException {

    // check that type is ok or throw IllegalArgumentException
    // grab a block in future

    return getNextIdFromDatabase(type, 5, con);


  }



  protected abstract BigInteger getNextIdFromDatabase(int type, int increment, Connection con) throws IdException;
  	
	
}


