/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.session;


import org.locomotive.loco.Cloudscape;
import java.util.*;
import java.sql.*;
import org.locomotive.loco.FailedSessionException;
import org.locomotive.loco.Loco;


public class SessionStoreJdbcCloudscape extends SessionStoreJdbc {

  public long createNewSession (Connection conn, int srid, int userid, Timestamp expire) throws FailedSessionException
  {

    // get new sid, create new row in LOCO_SESSIONS
    String statement = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    Statement stmt = null;
    long sid = 0;

    try {
      sid = Cloudscape.createSession(conn, srid, userid, expire);

    } catch (SQLException se) {
      throw new FailedSessionException("CreateNewSession: " +
                                       se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (pstmt!= null) pstmt.close();
      } catch (SQLException se) {

      }
    }
		return sid;
  }

  public Hashtable getSessionObjects(Connection conn, long sid) throws FailedSessionException
  {
		Statement stmt = null;
		ResultSet rs = null;
		Hashtable objects = null;

		try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery("select objects from loco_session_objects " + "where sid = " + sid);
      if (rs.next()) {
        objects = (Hashtable) rs.getObject(1);
      }

    } catch (SQLException se) {
      throw new FailedSessionException("Session " + sid + ": " +
                                       se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (stmt != null) stmt.close();
      } catch (SQLException se) {

      }
    }

    return objects;
  }


  public void storeSessionObjects(Connection conn, Hashtable objects)
  {
    int rows = 0;
    PreparedStatement pstmt1 = null;
    PreparedStatement pstmt2 = null;
    String query1 = null;
    String query2 = null;

    query1 = "delete from loco_session_objects where sid = ?";
    query2 = "insert into loco_session_objects (sid, objects) " +
             " values (?, ?)";

    try {
      pstmt1 = conn.prepareStatement(query1);
      conn.setAutoCommit(false);

      pstmt2 = conn.prepareStatement(query2);

      // for some reason this, combined with the reset() calls, isn't
      // working correctly.  It seems to lose about 2-3 bytes
      // or so every time reset is called.
//	    ByteArrayOutputStream bos = new ByteArrayOutputStream(8192);
//	    ObjectOutputStream oos = new ObjectOutputStream(bos);

      Enumeration e = objects.keys();
      while (e.hasMoreElements()) {
        Long sessid = (Long) e.nextElement();

        try {
          // delete the row from the table
          pstmt1.setLong(1, sessid.longValue());
          pstmt1.executeUpdate();

          // add the new row
          pstmt2.setLong(1, sessid.longValue());
          pstmt2.setObject(2, objects);
//			bos.reset();
//			oos.reset();

          rows += pstmt2.executeUpdate();

          if (! conn.getAutoCommit()) {
            conn.commit();
          }
        } catch (SQLException se) {
          Loco.log(0, "ERROR: session shutdown: could not save " +
                   "session objects for " + sessid + ": " + se);
        }
      }

    } catch (SQLException se) {
      Loco.log(0, "ERROR: session shutdown: could not save session " +
               "objects: " + se);
    } finally {
      try {
        if (pstmt1 != null) pstmt1.close();
        if (pstmt2 != null) pstmt2.close();
        if (! conn.getAutoCommit()) {
          conn.setAutoCommit(true);
        }
      } catch (SQLException se) {

      }
    }
  }
}



