/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.session;

import java.util.*;
import java.sql.*;
import java.math.BigInteger;
import org.locomotive.loco.FailedSessionException;
import org.locomotive.loco.User;
import org.locomotive.loco.store.id.IdException;

public class SessionStoreJdbcInterbase extends SessionStoreJdbc {


  public void associateUser(Connection conn, User user, long sid) throws FailedSessionException {
    String statement = null;
    Statement stmt = null;
    PreparedStatement pstmt = null;


    try {
      statement = "update loco_sessions set userid = ? " +
                  ", " + "active_date = 'NOW' where sid =  ?";

      pstmt = conn.prepareStatement(statement);
      pstmt.setLong(1, user.getUserid());
      pstmt.setLong(2, sid);

      pstmt.executeUpdate ();
    } catch (SQLException se) {
      throw new FailedSessionException(se.toString());
    } finally {
      try {
        if (stmt != null) stmt.close();
      } catch (SQLException se) {

      }
    }
  }

  public void expire (Connection conn, long sid, Timestamp expireTime) throws FailedSessionException
  {
    String statement = null;

    statement = "update loco_sessions set expire = ?, " +
                "active_date = 'NOW' where sid = " + sid;

    try {
      PreparedStatement pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expireTime);
      pstmt.executeUpdate ();
      pstmt.close();
    } catch (SQLException se) {
      throw new FailedSessionException (se.toString());
    }
  }

  public LocoSessionRow getLocoSessionRow(Connection conn, long sid) throws FailedSessionException
  {
    Statement stmt = null;
    ResultSet rs = null;
    LocoSessionRow r = null;

    try {
      String statement = "select srid, userid, expire, added, active_date " +
                         "from loco_sessions where sid = " + sid;

      stmt = conn.createStatement();
      rs = stmt.executeQuery (statement);

      if (! rs.next()) {
        rs.close();
        stmt.close();
        throw new FailedSessionException( "session not found in db: " +
                                          sid);
      }

      r = new LocoSessionRow( rs.getInt(1), rs.getInt(2), rs.getTimestamp(3), rs.getTimestamp(4), rs.getTimestamp(5) );
    } catch (SQLException se) {
      throw new FailedSessionException("Session " + sid + ": " + se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (stmt != null) stmt.close();
      } catch (SQLException se) {

      }
    }
    return r;
  }


  public long createNewSession (Connection conn, int srid, int userid, Timestamp expire) throws FailedSessionException
  {

    // get new sid, create new row in LOCO_SESSIONS
    String statement = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    Statement stmt = null;
    long sid = 0;

    try {

      try {
        BigInteger bigSid = getNextSessionId(conn);
	sid = bigSid.longValue();
      } catch (IdException  e) {
        throw new SQLException(e.getMessage());
      }

//      try {       // get the next session id
//        statement = "select nextval('LOCO_SESSIONS_sq')";
//        stmt = conn.createStatement ();
//        rs = stmt.executeQuery (statement);
//        rs.next();
//        sid = rs.getLong (1);
//      } catch (SQLException se) {
//        throw new FailedSessionException("creating sid: " +
//                                         se.toString());
//      } finally {
//        if (rs != null) rs.close();
//        if (stmt != null) stmt.close();
//      }



      // write everything in, along with current date/time
      statement = "insert into LOCO_SESSIONS " +
                  "(sid, srid, userid, added, active_date, expire) values ( " +
                  sid + ", " + srid + ", " + userid +
                  ", 'NOW', 'NOW', ?)";
      pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp (1, expire); // set in constructor
      pstmt.executeUpdate();
      pstmt.close();

    } catch (SQLException se) {
      throw new FailedSessionException("CreateNewSession: " +
                                       se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (pstmt!= null) pstmt.close();
      } catch (SQLException se) {

      }
    }
    return sid;
  }



  public void resetExpiration (Connection conn, long sid, Timestamp expireTime) throws FailedSessionException
  {
    // write it to the db too
    String statement = null;

    statement = "update loco_sessions set expire = ?, " +
                "active_date = 'NOW' where sid = " + sid;

    try {
      PreparedStatement pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expireTime);
      pstmt.executeUpdate ();
      pstmt.close();
    } catch (SQLException se) {
      throw new FailedSessionException (se + "- sessionId: " + sid);
    }
  }






}


