/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.session;

import java.util.*;
import java.sql.*;
import java.io.*;
import org.locomotive.loco.FailedSessionException;
import org.locomotive.loco.Loco;
import org.locomotive.loco.User;


public class SessionStoreJdbcMysql extends SessionStoreJdbc {

 public void associateUser(Connection conn, User user, long sid) throws FailedSessionException {
    String statement = null;
    Statement stmt = null;
    PreparedStatement pstmt = null;


    try {
      statement = "update loco_sessions set userid = ? " +
                  ", " + "active = {fn NOW()} where sid =  ?";

      pstmt = conn.prepareStatement(statement);
      pstmt.setLong(1, user.getUserid());
      pstmt.setLong(2, sid);

      pstmt.executeUpdate ();
    } catch (SQLException se) {
      throw new FailedSessionException(se.toString());
    } finally {
      try {
        if (stmt != null) stmt.close();
      } catch (SQLException se) {

      }
    }
  }


    public void expire (Connection conn, long sid, Timestamp expireTime)
	throws FailedSessionException
    {
	String statement = null;
	PreparedStatement pstmt = null;
	
	statement = "update loco_sessions set expire = ?, " +
	    "active = SYSDATE() where sid = " + sid;
	
	try
	{
	    pstmt = conn.prepareStatement (statement);
	    pstmt.setTimestamp(1, expireTime);
	    pstmt.executeUpdate ();
	}
	catch (SQLException se)
	{
	    throw new FailedSessionException (se.toString());
	}
	finally
	{
	    try { if (pstmt != null) pstmt.close(); } catch (SQLException e) {}
	}
    }


    public void resetExpiration (Connection conn, long sid,
				 Timestamp expireTime)
	throws FailedSessionException
    {
	// write it to the db too
	String statement = null;
	PreparedStatement pstmt = null;
	
	statement = "update loco_sessions set expire = ?, " +
	    "active = SYSDATE() where sid = " + sid;
	
	try
	{
	    pstmt = conn.prepareStatement (statement);
	    pstmt.setTimestamp(1, expireTime);
	    pstmt.executeUpdate ();
	}
	catch (SQLException se)
	{
	    throw new FailedSessionException (se + "- sessionId: " + sid);
	}
	finally
	{
	    try { if (pstmt != null) pstmt.close(); } catch (SQLException e) {}
	}
    }


    public void storeObjects(Connection conn, long sid, Hashtable objects)
	throws SQLException
    {
	
	PreparedStatement pstmt = null;
	String query = null;
	
	try
	{
	    conn.setAutoCommit(false);
	    query = "replace into loco_session_objects (sid, objects) " +
		"values (?, ?)";
	
	    pstmt = conn.prepareStatement(query);
	    pstmt.setLong(1, sid);
	
	    // is 1 k enough?
	    ByteArrayOutputStream bos = new ByteArrayOutputStream(1024);
	    ObjectOutputStream oos = new ObjectOutputStream(bos);
	    oos.writeObject(objects);
	    oos.flush();
	    ByteArrayInputStream bis =
		new ByteArrayInputStream(bos.toByteArray());
	    pstmt.setBinaryStream(2, bis, bis.available());
	
	    pstmt.executeUpdate();
	
	    if (! conn.getAutoCommit()) {
		conn.commit();
		conn.setAutoCommit(true);
	    }
	}
	catch (SQLException se)
	{
	    if (! conn.getAutoCommit())
	    {
		conn.rollback();
		conn.setAutoCommit(true);
	    }
	    throw se;
	}
	catch (IOException ioe)
	{
	    throw new SQLException ("couldn't write session objects: " +
				    ioe.toString());
	}
	finally
	{
	    if (pstmt != null) pstmt.close();
	}
    }





    public long createNewSession (Connection conn, int srid, int userid,
				  Timestamp expire)
	throws FailedSessionException
    {
	
	// get new sid, create new row in LOCO_SESSIONS
	String statement = null;
	ResultSet rs = null;
	PreparedStatement pstmt = null;
	Statement stmt = null;
	long sid = 0;
	
	try
	{
	    // write everything in, along with current date/time
	    statement = "insert into loco_sessions " +
		"(sid, srid, userid, added, active, expire) values ( " +
		0 + ", " + srid + ", " + userid +
		", SYSDATE(), SYSDATE(), ?)";
	    pstmt = conn.prepareStatement (statement);
	    pstmt.setTimestamp (1, expire); // set in constructor
	    pstmt.executeUpdate();
	    stmt = conn.createStatement();
	    rs = stmt.executeQuery("select LAST_INSERT_ID()");
	    rs.next();
	    sid = rs.getLong(1);
	
	}
	catch (SQLException se)
	{
	    throw new FailedSessionException("CreateNewSession: " +
					     se.toString());
	}
	finally
	{
	    try
	    {
		if (rs != null) rs.close();
		if (pstmt!= null) pstmt.close();
	    }
	    catch (SQLException se) {}
	}
	return sid;
    }

    public void storeSessionObjects(Connection conn, Hashtable objects)
    {
	int rows = 0;
	PreparedStatement pstmt2 = null;
	String query2 = null;
	query2 = "replace into loco_session_objects (sid, objects) " +
	    "values (?, ?)";
	
	try
	{
	    pstmt2 = conn.prepareStatement(query2);
	
	    // for some reason this, combined with the reset() calls, isn't
	    // working correctly.  It seems to lose about 2-3 bytes
	    // or so every time reset is called.
//	    ByteArrayOutputStream bos = new ByteArrayOutputStream(8192);
//	    ObjectOutputStream oos = new ObjectOutputStream(bos);
	
	    Enumeration e = objects.keys();
	    while (e.hasMoreElements())
	    {
		Long sessid = (Long) e.nextElement();
		
		try
		{
		    pstmt2.setLong(1, sessid.longValue());
		
		    ByteArrayOutputStream bos =
			new ByteArrayOutputStream(1024);
		    ObjectOutputStream oos = new ObjectOutputStream(bos);
		    oos.writeObject(objects.get(sessid));
		    oos.flush();
		    ByteArrayInputStream bis =
			new ByteArrayInputStream(bos.toByteArray());
		    pstmt2.setBinaryStream(2, bis, bis.available());
//			bos.reset();
//			oos.reset();
		
		    rows += pstmt2.executeUpdate();
		
		    if (! conn.getAutoCommit())
		    {
			conn.commit();
		    }
		}
		catch (SQLException se)
		{
		    Loco.log(0, "ERROR: session shutdown: could not save " +
			     "session objects for " + sessid + ": " + se);
		}
		catch (IOException ioe)
		{
		    Loco.log(0, "ERROR: session shutdown: could not save " +
			     "session objects for " + sessid + ": " + ioe);
		}
	    }
	
	}
	catch (SQLException se)
	{
	    Loco.log(0, "ERROR: session shutdown: could not save session " +
		     "objects: " + se);
	}
	finally
	{
	    try
	    {
		if (pstmt2 != null) pstmt2.close();
		if (! conn.getAutoCommit()) conn.setAutoCommit(true);
	    }
	    catch (SQLException se) {}
	}
    }

}


