/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.session;

import java.util.*;
import java.sql.*;
import org.locomotive.loco.FailedSessionException;
import org.locomotive.loco.Loco;

public class SessionStoreJdbcOracle extends SessionStoreJdbc {

  public void expire (Connection conn, long sid, Timestamp expireTime) throws FailedSessionException
  {
    String statement = null;

    statement = "update loco_sessions set expire = ?, " +
                "active = SYSDATE where sid = " + sid;

    PreparedStatement pstmt = null;

    try {
      pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expireTime);
      pstmt.executeUpdate ();
      pstmt.close();
    } catch (SQLException se) {
      Loco.log(0, "SQL Exception " + se + "stacktrace ");
      se.printStackTrace();
      throw new FailedSessionException (se.toString());

    }
    finally
    {
	try
	{
	    if (pstmt != null) pstmt.close();
	}
	catch (SQLException se)
	{
	    Loco.log(0, "SQL Exception " + se + "stacktrace ");
	}
    }
  }


  public void resetExpiration (Connection conn, long sid, Timestamp expireTime) throws FailedSessionException
  {
    // write it to the db too
    String statement = null;

    statement = "update loco_sessions set expire = ?, " +
                "active = SYSDATE where sid = " + sid;

    PreparedStatement pstmt = null;

    try {
      pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expireTime);
      pstmt.executeUpdate ();
      pstmt.close();
    } catch (SQLException se) {
      throw new FailedSessionException (se + "- sessionId: " + sid);
    }
    finally
    {
	try
	{
	    if (pstmt != null) pstmt.close();
	}
	catch (SQLException se)
	{
	    se.printStackTrace();
	    Loco.log(0, "SQL Exception " + se + "stacktrace ");
	}
    }
  }


  public long createNewSession (Connection conn, int srid, int userid, Timestamp expire) throws FailedSessionException
  {

    // get new sid, create new row in LOCO_SESSIONS
    String statement = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    Statement stmt = null;

    long sid = 0;


    try {

        try {       // get the next session id
          statement = "select LOCO_SESSIONS_sid_sq.nextval from dual";
          stmt = conn.createStatement ();
          rs = stmt.executeQuery (statement);
          rs.next();
          sid = rs.getLong (1);
        } catch (SQLException se) {
          throw new FailedSessionException("creating sid: " +
                                           se.toString());

        } finally {
	    try
	    {
		if (rs != null) rs.close();
		if (stmt != null) stmt.close();
	    }
	    catch (SQLException se)
	    {

	    }
        }

        // write everything in, along with current date/time
        statement = "insert into LOCO_SESSIONS values " +
                    "(" + sid + ", " + srid + ", " + userid +
                    ", SYSDATE, SYSDATE, ?)";
        pstmt = conn.prepareStatement (statement);
        pstmt.setTimestamp (1, expire); // set in constructor
        pstmt.executeUpdate();

    } catch (SQLException se) {
	Loco.log(0, "SQL Exception " + se + "stacktrace " ); 
	se.printStackTrace();
	throw new FailedSessionException("CreateNewSession: " +
					 se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (pstmt!= null) pstmt.close();
      } catch (SQLException se) {
	  Loco.log(0, "SQL Exception " + se + "stacktrace " ); se.printStackTrace();
      }
    }
    return sid;
  }
}


