/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.session;

import java.util.*;
import java.sql.*;
import org.locomotive.loco.FailedSessionException;
import org.locomotive.loco.User;


public class SessionStoreJdbcPostgresql extends SessionStoreJdbc {


  public void associateUser(Connection conn, User user, long sid) throws FailedSessionException {
    String statement = null;
    Statement stmt = null;
    PreparedStatement pstmt = null;


    try {
      statement = "update loco_sessions set userid = ? " +
                  ", " + "active = current_timestamp where sid =  ?";

      pstmt = conn.prepareStatement(statement);
      pstmt.setLong(1, user.getUserid());
      pstmt.setLong(2, sid);

      pstmt.executeUpdate ();
    } catch (SQLException se) {
      throw new FailedSessionException(se.toString());
    } finally {
      try {
        if (stmt != null) stmt.close();
      } catch (SQLException se) {

      }
    }
  }

  public void expire (Connection conn, long sid, Timestamp expireTime) throws FailedSessionException
  {
    String statement = null;

    statement = "update loco_sessions set expire = ?, " +
                "active = current_timestamp where sid = " + sid;

    try {
      PreparedStatement pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expireTime);
      pstmt.executeUpdate ();
      pstmt.close();
    } catch (SQLException se) {
      throw new FailedSessionException (se.toString());
    }
  }

  public long createNewSession (Connection conn, int srid, int userid, Timestamp expire) throws FailedSessionException
  {

    // get new sid, create new row in LOCO_SESSIONS
    String statement = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    Statement stmt = null;
    long sid = 0;

    try {

      try {       // get the next session id
        statement = "select nextval('LOCO_SESSIONS_sq')";
        stmt = conn.createStatement ();
        rs = stmt.executeQuery (statement);
        rs.next();
        sid = rs.getLong (1);
      } catch (SQLException se) {
        throw new FailedSessionException("creating sid: " +
                                         se.toString());
      } finally {
        if (rs != null) rs.close();
        if (stmt != null) stmt.close();
      }

      // write everything in, along with current date/time
      statement = "insert into LOCO_SESSIONS " +
                  "(sid, srid, userid, added, active, expire) values ( " +
                  sid + ", " + srid + ", " + userid +
                  ", current_timestamp, current_timestamp, ?)";
      pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp (1, expire); // set in constructor
      pstmt.executeUpdate();
      pstmt.close();

    } catch (SQLException se) {
      throw new FailedSessionException("CreateNewSession: " +
                                       se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (pstmt!= null) pstmt.close();
      } catch (SQLException se) {

      }
    }
		return sid;
  }



  public void resetExpiration (Connection conn, long sid, Timestamp expireTime) throws FailedSessionException
  {
    // write it to the db too
    String statement = null;

    statement = "update loco_sessions set expire = ?, " +
                "active = current_timestamp where sid = " + sid;

    try {
      PreparedStatement pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expireTime);
      pstmt.executeUpdate ();
      pstmt.close();
    } catch (SQLException se) {
      throw new FailedSessionException (se + "- sessionId: " + sid);
    }
  }






}


