/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.session;

import java.util.*;
import java.sql.*;
import org.locomotive.loco.FailedSessionException;


public class SessionStoreJdbcSolid extends SessionStoreJdbc {

  public long createNewSession (Connection conn, int srid, int userid, Timestamp expire) throws FailedSessionException
  {

    // get new sid, create new row in LOCO_SESSIONS
    String statement = null;
    ResultSet rs = null;
    PreparedStatement pstmt = null;
    Statement stmt = null;
    long sid = 0;

    try {

      // run procedure, which puts the session in
      // the database and returns its session id
      statement = "call loco_create_session ( " + srid +
                  ", " + userid + ", ?)";
      pstmt = conn.prepareStatement (statement);
      pstmt.setTimestamp(1, expire);   // set in constructor
      rs = pstmt.executeQuery();
      rs.next();
      sid = rs.getLong(1);

    } catch (SQLException se) {
      throw new FailedSessionException("CreateNewSession: " +
                                       se.toString());
    } finally {
      try {
        if (rs != null) rs.close();
        if (pstmt!= null) pstmt.close();
      } catch (SQLException se) {

      }
    }
		return sid;
  }
}


