/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Leverage Information
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage
 * Information Systems. All Rights reserved.
 */

package org.locomotive.loco.store.user;

import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.http.*;
import org.locomotive.util.*;
import org.locomotive.loco.LoadUserException;


public class UserStoreJdbcSolid extends UserStoreJdbc {
  /**
   * Persistantly stores User's data.  Will fail
   * if uid is not unique (uid <= 0 will cause system to pick a userid).
   * Will also fail if un (intended username) is already used.
   *
   * @param conn   a JDBC connection with access to the Loco user database
   * @param uid    the userid, system will pick if <= 0
   * @param un     username of the new user
   * @param pword  password of the new user
   * @param locoflg flags of the new user for Loco-specific functions
   * @exception LoadUserException if there is a database storage error
   */
  public int createUser(Connection conn, int uid, String un, String pword,
              int locoflg) throws LoadUserException
  {
    int userid = uid;
    String username = un;
    String password = pword;
    int loco_flags = locoflg;

    PreparedStatement pstmt = null;
    ResultSet rs = null;
    String statement = null;

    // order in the table is as follows:
    // userid, username, password, loco_flags, added, modified, last_login
    // HEY!  LETS GET THOSE FINALLY CLAUSES IN THERE!
    try {
        // execute the create_user stored procedure, which returns the
        // unique serial number created by the database

        if (userid < 0) userid = 0;   // get the next number in the sq

        statement = "call loco_create_user (?, ?, ?, ?)";
        pstmt = conn.prepareStatement(statement);
        pstmt.setInt(1, userid);
        pstmt.setString (2, username);
        pstmt.setString (3, password);
        pstmt.setInt(4, loco_flags);
        rs = pstmt.executeQuery();
        rs.next();
        userid = rs.getInt(1);
        rs.close();
        pstmt.close();
			return userid;

    } catch (SQLException se) {
      throw new LoadUserException ("storage failure: " + se.toString());
    }
  }

  /**
   * Updates the user.last_login field in the database.
   * silently ignored on anonymous users
   * @param conn an active JDBC connection
   * @exception LoadUserException if there is a database storage error
   */
  public void saveLoginTime (Connection conn, int userid) throws LoadUserException
  {
    if (userid <= 0)
      return;

    String statement = null;

      statement = "update LOCO_USERS set last_login = curtime() " +
                  " where userid = " + userid;

    int result = 0;
    try {
      Statement stmt = conn.createStatement();
      result = stmt.executeUpdate(statement);
      stmt.close();
    } catch (SQLException se) {
      throw new LoadUserException (se.toString());
    }
    if (result == 0) {
      throw new LoadUserException ("User " + userid + " does not exist.");
    }
  }


  /**
   * Updates the User data in the database if changes have been made,
   * silently ignored on anonymous users
   * @param conn an active JDBC connection
   * @exception LoadUserException if there is a database storage error
   */
  public void save (Connection conn, int userid, String username, String password, int loco_flags) throws LoadUserException
  {
    String statement = null;
      statement = "update LOCO_USERS set " +
                  "username = ?,  password = ?,  loco_flags = " + loco_flags +
                  ", modified = curtime() where userid = " + userid;

    try {
      PreparedStatement pstmt = conn.prepareStatement (statement);
      pstmt.setString (1, username);
      pstmt.setString (2, password);
      pstmt.executeUpdate();
      pstmt.close();
    } catch (SQLException se) {
      throw new LoadUserException (se.toString());
    }
  }
}
