/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.module.base;

import org.locomotive.loco.*;
import org.locomotive.loco.handler.*;
import org.locomotive.server.Log;


/**
 * The base handler is an example handler that accompanies the Loco.
 * It provides a simple login/authentication module, a user manager/viewer, 
 * and a sample registration.  It can be used to validate the Loco server
 * privledged user for viewing and manipulating the Loco via the 
 * core Loco ServerHandler.  By default, the base handler is available 
 * using the module name &quot;base&quot; in the HandlerRoutingTable.
 */
public class BaseHandler extends GenericHandler
{
    /**
     * main entry point called by RequestManager for each new request.
     */
    public void handleRequest (HandlerData hd_in)
    {
	super.handleRequest(hd_in);
	
	hd = hd_in;
	
	// call the Loco library call for standard insertions.  
	StandardInsertions.insertAll (hd);

	// put in admin, if the user has privileges
	if (user.canViewServerStatus())
	{
	    steam_vars.put("ADMIN", "true");
	}
	
	// parse url tokens for proper function
	if (url_tokens.length < 2)
	{
	    displayPage ("base/main.tmpl", steam_vars);
	}
	else if ( url_tokens[1].equals("login"))
	{
	    LoginHandler lr = new LoginHandler();
	    lr.handleRequest (hd);
	}
	else if (url_tokens[1].equals ("reg"))
	{
	    RegistrationHandler rr = new RegistrationHandler ();
	    rr.handleRequest (hd);
	}
	else if (url_tokens[1].equals ("logout"))
	{
	    LogoutHandler lr = new LogoutHandler ();
	    lr.handleRequest (hd);
	}
	else if (url_tokens[1].equals ("gen"))
	{
	    displayPage("base/main.tmpl", steam_vars);
	}
	else
	{
	    displayPage("error/unknown_handler_option.tmpl", steam_vars);
	}
    }


    /**
     * sends a generic error message back to the client.  
     * @param hd - the HandlerData as passed into the Handler
     * @param msg - the error message code to be sent 
     * @deprecated in favor of Loco.sendErrorMsg
     */
    public static void sendErrorMsg (HandlerData hd, String msg)
    {
	java.util.Date dt = new java.util.Date();

	// note that if the Response has been flushed at all, the
	// http headers have already been sent.  There is no way to avoid
	// strange behaviours in the output at this point.  

	// clean up whatever else has been placed in the response thus far.
	hd.resp.setContentType("text/html");
	hd.resp.clearHeaders();
	hd.resp.clearBody();

	// do the error message. now uses template, new for 3.3
	hd.subs.put ("ERROR_MSG", msg);
	hd.subs.put ("REQUEST_ID", String.valueOf (hd.getRequestID() ));
	hd.subs.put ("ERROR_DATE", dt.toString());

	// these lines help make sense of things when we're running Locomotives
	// in parallel.
	hd.subs.put ("SYSTEM_TAG", Loco.getConfig().getString ("SYSTEM_TAG"));
	hd.subs.put ("SYSTEM_INSTANCE_ID", 
			Loco.getConfig().getString ("SYSTEM_INSTANCE_ID"));

	// stick things in the template
	String s = Loco.evaluateTemplate ("error/generic_error.tmpl", 
					  hd.subs);

	if (s.equals(""))
	{
	    s = "<HTML>Loco ERROR: missing or empty generic error template. " +
		" Administrator, please configure. </HTML>";
	}

	hd.resp.addString (s);
	hd.resp.flush();
    }

    /**
     * The required init method, as defined in org.locomotive.loco.Handler.  In
     * this example, we don't actually need to do anything.
     * @see org.locomotive.loco.Handler#init
     */
    public boolean init (Log log)
    {
	return (true);
    }

    /**
     * The required shutdown method, as defined in org.locomotive.loco.Handler.  In
     * this example, we don't actually need to do anything.
     * @see org.locomotive.loco.Handler#shutdown
     */
    public boolean shutdown (Log log)
    {
	return (true);
    }

}
