/**
 * BaseServlet.java 
 *
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.module.base;

import org.locomotive.loco.*;
import org.locomotive.loco.servlet.*;
import org.locomotive.server.Log;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The base Servlet is an example Servlet that accompanies the Loco.
 * It provides a simple login/authentication module, a user manager/viewer, 
 * and a sample registration.  It can be used to validate the Loco server
 * privledged user for viewing and manipulating the Loco via the 
 * core Loco ServerServlet.  By default, the base Servlet is available 
 * using the module name &quot;base&quot; in the ServletRoutingTable.
 */
public class BaseServlet extends LocoServlet
{
    private LoginServlet login_servlet;
    private RegistrationServlet reg_servlet;
    private LogoutServlet logout_servlet;


    /**
     * Creates our internal LoginServlet, RegistrationServlet,
     * and LogoutServlet
     */
    public void init(ServletConfig config) throws ServletException
    {
	super.init(config);
	login_servlet = new LoginServlet();
	login_servlet.init(config);
	
	reg_servlet = new RegistrationServlet();
	reg_servlet.init(config);
	
	logout_servlet = new LogoutServlet();
	logout_servlet.init(config);
    }
	



    /** just send off our various requests */
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	StandardInsertions.insertAll (req, steam_vars);
	
	if (user.canViewServerStatus())
	{
	    steam_vars.put("ADMIN", "true");
	}

	// parse url tokens for proper function
	if (url_tokens.length < 2)
	{
	    displayPage ("base/main.tmpl", steam_vars);
	}
	else if ( url_tokens[1].equals("login"))
	{
	    // this will point to login_servlet.doGet()
	    login_servlet.service (req, resp);
	}
	else if (url_tokens[1].equals ("reg"))
	{
	    // this will point to reg_servlet.doGet()
	    reg_servlet.service (req, resp);
 	}
	else if (url_tokens[1].equals ("logout"))
	{
	    // this will point to logout_servlet.doGet()
	    logout_servlet.service (req, resp);
	}
	else if (url_tokens[1].equals ("gen"))
	{
	    displayPage("base/main.tmpl", steam_vars);
	}
	else
	{
	    displayPage("error/unknown_service_option.tmpl", steam_vars);
	}
    }


    /** send off the various posts */
    public void doPost(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	StandardInsertions.insertAll (req, steam_vars);

	if (user.canViewServerStatus())
	{
	    steam_vars.put("ADMIN", "true");
	}

	if (url_tokens.length < 2)
	{
	    displayPage ("base/main.tmpl", steam_vars);
	}	    
	else if ( url_tokens[1].equals("login"))
	{
	    login_servlet.service (req, resp);
	}
	else if (url_tokens[1].equals ("reg"))
	{
	    reg_servlet.service (req, resp);
 	}
	else
	{
	    displayPage("error/unknown_service_option.tmpl", steam_vars);
	}
    }	

}     
