/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.module.base;

import org.locomotive.loco.*;
import org.locomotive.loco.handler.*;

/**
 * A sample Logout handler.  Resets the session cookie and marks the 
 * previous session dead in the session database.  This is effectively 
 * &quot;force expiring&quot; a session.  Sends back a generic
 * &quot;you have logged out&quot; message to the client.  To use this
 * as a separate module add it to your hrt.conf file.  An example would
 * be: <PRE>
 *      logout           org.locomotive.module.base.LogoutHandler
 * </PRE>
 */
public class LogoutHandler extends GenericHandler
{
    public void handleRequest (HandlerData hd)
    {
	super.handleRequest(hd);
	
	try
	{
	    session.expire (conn);
	}
	catch (FailedSessionException fse)
	{
	    handleException(fse, "pm_logout_fse");
	    return;
	}

	// tell the browser.  For right now, this routine sends back 
	// and expired cookie of the same name, clobbering the old one
	// so the browser doesn't go around indentifying itself improperly
	session.updateBrowserCookie (hd.resp);

	// send back the resultant template
	displayPage ("base/notice/now_logged_out.tmpl", steam_vars);
	logEvent("logout", "success", "");
    }
}
