/**
 * LogoutServlet.java
 *
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License. 
 *
 * The initial develop of this code under the LPL Leverage Information Systems.
 * Portions created by Leverage are Copyright (C) 1998 Leverage Information Systems.
 * All Rights reserved.
 */

package org.locomotive.module.base;

import org.locomotive.loco.*;
import org.locomotive.loco.servlet.*;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * A sample Logout servlet.  Resets the session cookie and marks the 
 * previous session dead in the session database.  This is effectively 
 * &quot;force expiring&quot; a session.  Sends back a generic
 * &quot;you have logged out&quot; message to the client.  To use this
 * as a separate module add it to your hrt.conf file.  An example would
 * be: <PRE>
 *      logout           org.locomotive.module.base.LogoutServlet
 * </PRE>
 */
public class LogoutServlet extends LocoServlet
{
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	try
	{
	    session.expire (conn);
	}
	catch (FailedSessionException fse)
	{
	    handleException(fse, "pm_logout_fse");
	    return;
	}

	// tell the browser.  For right now, this routine sends back 
	// and expired cookie of the same name, clobbering the old one
	// so the browser doesn't go around indentifying itself improperly
	session.updateBrowserCookie ((org.locomotive.loco.Response) resp);

	// send back the resultant template
	displayPage ("base/notice/now_logged_out.tmpl", steam_vars);
	logEvent("logout", "success", "");
    }
}
