
package org.locomotive.module.btrack;


import org.locomotive.loco.*;
import org.locomotive.loco.handler.*;
import org.locomotive.server.Log;
import org.locomotive.util.*;

import java.util.Hashtable;
import java.util.Date;
import java.util.Vector;
import java.util.ResourceBundle;
import java.sql.*;


// see the BugTrackerStoreJdbc class for comments
// more comments to follow.

abstract class BugTrackerStore {

  static final int BUG      = 0;
  static final int PROJECT  = 1;
  static final int BUG_TEXT = 2;

  protected static BugTrackerStore instance = null;

  protected static int idIncrement;

  static final BugTrackerStore getInstance() throws BugTrackerException {
    if (instance == null) {

      String cName = "org.locomotive.module.btrack.BugTrackerStoreJdbc";
      try {
        ResourceBundle rb = ResourceBundle.getBundle("config/btrack");
        cName = (String) rb.getObject("store.class");
        idIncrement = Integer.parseInt((String) rb.getObject("store.increment"));
      } catch (Exception e) {
        // use default
        // log the exception here somehow
      }

      try {

        synchronized(BugTrackerStore.class) {
          if (instance == null) {
            instance = (BugTrackerStore) Class.forName(cName).newInstance();
          }
        }

      } catch (ClassNotFoundException e) {
        throw new BugTrackerException(""+e);
      } catch (InstantiationException e) {
        throw new BugTrackerException(""+e);
      } catch (IllegalAccessException e) {
        throw new BugTrackerException(""+e);
      }
    }

    return instance;
  }


  abstract int getNextProjectId(Connection con)
  throws BugTrackerException;

  abstract int getNextBugId(Connection con)
  throws BugTrackerException;

  abstract int getNextBugTextId(Connection con)
  throws BugTrackerException;


  abstract boolean projectExists(int projectId, Connection con)
  throws BugTrackerException;

  abstract boolean projectNameExists(String projectName, Connection con)
  throws BugTrackerException;

  abstract int getProjectIdGivenShortName(String projectName, Connection con)
  throws BugTrackerException;

  abstract int getProjectIdGivenBugId(int bugId, Connection con)
  throws BugTrackerException;

  abstract String getProjectName(int projectId, Connection con)
  throws BugTrackerException;


  abstract int getProjectNamesFromDB(String projectNames[], Connection con)
  throws BugTrackerException;

  abstract void populateProjectIDsAndNames(Vector project_ids,
                                           Vector project_names,
                                           Connection con)
  throws BugTrackerException;


  abstract boolean bugExists(int bugId, Connection con)
  throws BugTrackerException;

  abstract SortableTable getProjects(Connection con)
  throws BugTrackerException;

  abstract SortableTable getBugs(int bugId, int projectId, String status,
                                 int priority, String severity, String version,
                                 int creatorUserId, String creatorUsername, String orderby,
                                 Connection con) throws BugTrackerException;

  abstract boolean getBugData(int bugId, Hashtable steamTable,
                              Connection con) throws BugTrackerException;

  abstract boolean getProjectData(int projectId, Hashtable steamTable,
                                  Connection con) throws BugTrackerException;

  abstract int insertProject(int projectId, String shortTitle,
                             String longTitle, String text, int userId,
                             Connection con) throws BugTrackerException;

  abstract int updateProject(int projectId, String shortTitle,
                             String longTitle, String text, String status,
                             Connection con) throws BugTrackerException;

  abstract int insertBug(int bugId, int projectId, String version,
                         String priority, String severity, String summary,
                         int userId, Connection con)
  throws BugTrackerException;

  abstract int updateBug(int bugId, int projectId, String version,
                         String priority, String status, String severity,
                         String summary, Connection con)
  throws BugTrackerException;

  abstract int insertBugText(int bugTextId, int bugId, int userId,
                             String comments, Connection con)
  throws BugTrackerException;

  abstract String assembleCommentChunk(int bugId, Connection con)
  throws BugTrackerException;

}
