
package org.locomotive.module.btrack;


import org.locomotive.loco.*;
import org.locomotive.loco.handler.*;
import org.locomotive.server.Log;
import org.locomotive.util.*;

import java.util.Hashtable;
import java.util.Date;
import java.util.Vector;
import java.sql.*;


class BugTrackerStoreJdbc extends BugTrackerStore {

  protected int[] nextId = {0,0,0};
  protected int[] maxId  = {0,0,0};


  private int getNextId(int idType, Connection con)
  throws BugTrackerException {

    // have we used up all the ids in the current allocation?
    if (nextId[idType] >= maxId[idType]) {

      try {
        Statement stmt = con.createStatement();

        synchronized(BugTrackerStoreJdbc.class) {
          // get existing highest id
          String statement = "select next_available from btrack_ids "+
            "where id_type = "+idType;
          ResultSet rs = stmt.executeQuery(statement);

          // is the next available id in the database?
          if (rs.next()) {
            nextId[idType] = rs.getInt(1);
            maxId[idType] = nextId[idType] + idIncrement - 1;

            // set to new highest id
            statement = "update btrack_ids set next_available = "+
              (maxId[idType]+1)+" where id_type = "+idType;
            stmt.executeUpdate(statement);

          } else {
            statement = "insert into btrack_ids (id_type, next_available) "+
              "values ("+idType+", "+idIncrement+")";
            stmt.executeUpdate(statement);

            nextId[idType] = 0;
            maxId[idType] = idIncrement - 1;
          }

          rs.close();
          stmt.close();

        }
      } catch (SQLException e) {
        e.printStackTrace();
        throw new BugTrackerException(""+e);
      }

    }
    return nextId[idType]++;
  }



  int getNextProjectId(Connection con) throws BugTrackerException {
    return getNextId(PROJECT, con);
  }


  int getNextBugId(Connection con) throws BugTrackerException {
    return getNextId(BUG, con);
  }


  int getNextBugTextId(Connection con) throws BugTrackerException {
    return getNextId(BUG_TEXT, con);
  }




  /**
   * Tells us if a given project exists in the database - specifically,
   * if there is exactly one project definition by that short_name - anything else
   * is an error.
   */
  boolean projectExists(int project_id, Connection conn)
  throws BugTrackerException
  {
    try {
      // count should be 1 exactly
      String statement = "select count(*) from btrack_projects " +
        "where id = " + project_id;
      Statement stmt = conn.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      int got = 1;

      if (results.next()) {
        got = results.getInt(1);
      }

      results.close();
      stmt.close();

      return(got ==1);

    } catch (SQLException e) {
      throw new BugTrackerException(""+e);
    }
  }



  /**
   * Tells us if a given project exists in the database - specifically,
   * if there is exactly one project definition by that short_name - anything else
   * is an error.
   */
  boolean projectNameExists(String project_name, Connection conn)
  throws BugTrackerException
  {
    try {
      // count should be 1 exactly
      String statement = "select count(*) from btrack_projects " +
        "where short_title = " + SQL.quoteString(project_name);
      Statement stmt = conn.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      int got = 1;

      if (results.next()) {
        got = results.getInt(1);
      }

      results.close();
      stmt.close();

      return(got ==1);

    } catch (SQLException e) {
      throw new BugTrackerException(""+e);
    }
  }




  /**
   *  returns the project id for a project in btrack_projects, given
   *  its short_name value
   */
  int getProjectIdGivenShortName(String proj_name, Connection con)
  throws BugTrackerException
  {
    try {
      //
      int proj_id = 0;
      String statement = "select id from btrack_projects where short_title ="
        + SQL.quoteString(proj_name);

      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      if (results.next()) {
        proj_id = results.getInt(1);
      }

      results.close();
      stmt.close();

      return proj_id;

    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }
  }




  /**
   *  returns the project id for a bug in btrack_bugs
   */
  int getProjectIdGivenBugId(int bugId, Connection con) throws BugTrackerException {
    try {
      int projectId = 0;
      String statement = "select project_id from btrack_bugs where id = "+bugId;

      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      if (results.next()) {
        projectId = results.getInt(1);
      }

      results.close();
      stmt.close();

      return projectId;

    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }
  }





  /**
   * Given a project id, use the also-supplied Connection to return the
   * corersponding project name associated in the database.
   */

  String getProjectName(int projectId, Connection con) throws BugTrackerException {
    try {
      String statement = "select short_title from btrack_projects where " +
        "id = " + projectId;
      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      String result = null;

      // We should get exactly 1 result, so we can use a one-time if:
      if (results.next())
        result = results.getString(1);

      results.close();
      stmt.close();

      if (result == null) result = "";

      return result;

    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }
  }





  /**
   * This method generates a list of all projects in btrack_projects
   * identified by short_name.  It uses an estimated maximum number
   * of existing projects; benefit - can use an array directly rather
   * than a hashtable which would require iterating through an enumeration
   * first.  drawback - maintaining an estimate of the # of projects
   **/
  int  getProjectNamesFromDB(String projects[], Connection conn)
  throws BugTrackerException
  {
    try {
      int project_index = 0;

      String statement = "select short_title from btrack_projects";
      Statement stmt = conn.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      while (results.next()) {
        projects[project_index++] = results.getString(1);
      }

      results.close();
      stmt.close();

      return project_index;

    } catch (SQLException e) {
      throw new BugTrackerException(""+e);
    }
  }




  /**
   * Populate our vectors with all the project ids and names in the database
   * using the supplied Connection. Will order by name. Each index in the
   * project_ids vector passed in will correspond to the name in the
   * project_names vector after this call is through operating on them.
   */
  void populateProjectIDsAndNames(Vector project_ids, Vector project_names,
                                  Connection con) throws BugTrackerException {
    try {
      String statement = "select id, short_title from btrack_projects " +
        "order by short_title";
      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      String temp_string = null;

      while (results.next())
      {
        project_ids.addElement(new Integer(results.getInt(1)));
        temp_string = results.getString(2);
        project_names.addElement(temp_string == null ? "" : temp_string);
      }

      results.close();
      stmt.close();
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }
  }





  /**
   * Tells us if a given bug exists in the database - specifically,
   * if there is exactly one bug definition by that id - anything else
   * is an error.
   */
  boolean bugExists(int bug_id, Connection conn)
  throws BugTrackerException
  {
    try {
      // count should be 1 exactly
      String statement = "select count(*) from btrack_bugs " +
        "where id = " + bug_id;
      Statement stmt = conn.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      int got = 0;

      if (results.next()) {
        got = results.getInt(1);
      }

      results.close();
      stmt.close();

      return(got == 1);

    } catch (SQLException e) {
      throw new BugTrackerException(""+e);
    }
  }






  SortableTable getProjects(Connection con) throws BugTrackerException {
    // We'll use a SortableTable object to make quick work of those
    // pesky database rows:
    SortableTable st = new SortableTable(10, 8, 5, 0);
    try {
      Statement stmt = con.createStatement();
      String statement = "select p.id, short_title, status, text, "+
        "username, p.modified "+
        "from btrack_projects p, loco_users u where p.creator_userid = u.userid";
      ResultSet results = stmt.executeQuery(statement);

      // See above use of sortable table (hanbdleListBugs) for more
      // info on this:
      st = new SortableTable(10, 8, 5, 0);

      st.load(results, 0, 0, true);

      results.close();
      stmt.close();

      // Change the userids in the table over to the translated
      // usename (also change the column name):
      // substituteUsernameForUseridInSortableTable(conn, st, 5);

      st.setColumnName(0, "ID");
      st.setColumnName(1, "SHORT_TITLE");
      st.setColumnName(2, "STATUS");
      st.setColumnName(3, "TEXT");
      st.setColumnName(4, "CREATOR_USERNAME");
      st.setColumnName(5, "MODIFIED");
      return st;

    } catch (SQLException se) {
      throw (BugTrackerException) new BugTrackerException(""+se).fillInStackTrace();
      //handleException(se, generic_error_message);
      //return;
    }
  }






  SortableTable getBugs(int bugId, int projectId, String status, int priority, String severity, String version, int creatorUserId, String creatorUsername, String orderby, Connection con) throws BugTrackerException {

    String statement = "select p.short_title, b.summary, b.status, b.priority, b.severity, b.version, b.modified, b.created, u.username, b.id, b.creator_user_id "+
      "from btrack_bugs b, btrack_projects p, loco_users u "+
      "where b.project_id = p.id and b.creator_user_id = u.userid ";

    // You see, the joiner will be the glue between the parts of the
    // statement. At first it's " where " for the first thing we add,
    // but for subsequent items it will be " and ".

    // We now go through each potential form item. If they are set to
    // meaningful values then we put a kind word in the SQL statement
    // for them. If they are meaningless values (i.e. NULL_ID of some
    // kind) then we ignore them.

    // If they are included in the SQL statement we must remember to
    // changer the joiner variable to " and " for any subsequent
    // conditions that will be added.

    // Because of the join there will always be at least one where
    // clause so the joiner will always be " and " from now on - ee


    if (projectId != BugTrackerHandler.NULL_PROJECT_ID) {
      statement += "and b.project_id = " + projectId;
    }


    if (status != null && !(status.equals(""))) {
      statement += "and b.status = " + SQL.quoteString(status);
    }


    if (creatorUserId != User.NULL_USERID) {
      statement += "and b.creator_user_id = " + creatorUserId;
    }


    if (version != null && !version.equals("")) {
      statement += "and b.version = " + SQL.quoteString(version);
    }


    if (severity != null && !severity.equals("")) {
      statement += "and b.severity = " + SQL.quoteString(severity);
    }


    if (creatorUsername != null && !(creatorUsername.equals(""))) {
      statement += "and (b.creator_user_id in (select userid " +
                   "from loco_users where username = " +
                   SQL.quoteString(creatorUsername) + "))";
    }


    if (priority != BugTrackerHandler.NULL_PRIORITY_VALUE) {
      statement += "and b.priority = " + priority;
    }


    if (bugId != BugTrackerHandler.NULL_BUG_ID) {
      statement += "and b.id = " + bugId;

      // We don't need to set a joiner variable here since this is
      // the last one, but if there are ever any others after this we
      // will want this to be uncommented:

      // joiner = " and ";
    }


    if (orderby != null && !(orderby.equals(""))) {
      if (orderby.equals("Bug Id"))                         statement += " order by b.id";
      else if (orderby.equals("Project, Status, Severity")) statement += " order by p.short_title, b.status, b.severity";
      else if (orderby.equals("Project, Status, Priority")) statement += " order by p.short_title, b.status, b.priority";
      else if (orderby.equals("Project, Status, Age"))      statement += " order by p.short_title, b.status, b.created";
      else if (orderby.equals("Creator, Age"))              statement += " order by u.username, b.created";
    } else {
      statement += " order by b.id";
    }


    // Now our statement has been properly constructed.
    // Let us get thy data.

    // We'll use a SortableTable to get most of the
    // info from the result set. We'll have to change it around a little,
    // but I think you'll find it handy.
    SortableTable st;

    // Initialize the SortableTable... the parameters mean:
    // initial_row_count, initial_column_count, capacity_increment_row,
    // capacity_increment_column
    // initial_row_count is a guess, initial_column_count corresponds to
    // the number of columns in the table, capacity_increment_row is a
    // guess, and capacity_increment_column doesn't really apply, since
    // we're never adding any columns to this SortableTable (for now.)
    st = new SortableTable(10, 13, 5, 0);


    try {
      // Using the oh-so-carefully-constructed statement, do the
      // database footwork:
      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      // Using the SortableTable's handy function, load it up with
      // the result set. The parameters passed in to it are:
      // the result set, start row, start column, and "setNames".
      // "setNames" tells the SortableTable to set the column names
      // to the analagous table names from the database.
      st.load(results, 0, 0, true);

      // Wasn't that easy?

      results.close();
      stmt.close();

    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    // Column names all explicity set because they are not all the same
    // as the database columns
    st.setColumnName(0, "PROJECT_NAME");
    st.setColumnName(1, "SUMMARY");
    st.setColumnName(2, "STATUS");
    st.setColumnName(3, "PRIORITY");
    st.setColumnName(4, "SEVERITY");
    st.setColumnName(5, "VERSION");
    st.setColumnName(6, "MODIFIED");
    st.setColumnName(7, "CREATED");
    st.setColumnName(8, "CREATOR_USERNAME");
    return st;
  }





  boolean getBugData(int bugId, Hashtable steamTable, Connection con) throws BugTrackerException {

    // We want everything from the database about this bug except the
    // id which we already know:
    String statement = "select b.project_id, p.short_title, b.status, "+
      "b.priority, b.creator_user_id, u.username, b.summary, b.severity, "+
      "b.version, b.created, b.modified "+
      "from btrack_bugs b, loco_users u, btrack_projects p "+
      "where b.creator_user_id = u.userid  and b.project_id = p.id "+
      "and b.id = " + bugId;

    int projectId;
    String projectName;
    String status;
    int priority;
    int creatorUserId;
    String creatorUsername;
    String summary;
    String severity;
    String version;
    Date created;
    Date modified;

    try {
      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      // We should get just one row back:
      if (results.next()) {
        projectId = results.getInt(1);
        projectName = results.getString(2);
        status = results.getString(3);
        priority = results.getInt(4);
        creatorUserId = results.getInt(5);
        creatorUsername = results.getString(6);
        summary = results.getString(7);
        severity = results.getString(8);
        version = results.getString(9);
        created = results.getTimestamp(10);
        modified = results.getTimestamp(11);

      } else {
        results.close();
        stmt.close();
        return false;
      }

      results.close();
      stmt.close();

    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }


    // Put everything into the environment for the user:
    steamTable.put("BTRACK_BUG_ID", "" + bugId);
    steamTable.put("BTRACK_PROJECT_ID", "" + projectId);
    steamTable.put("BTRACK_PROJECT_NAME",
                   StringUtil.htmlQuoteString(projectName));
    steamTable.put("BTRACK_CREATOR_USERID", "" + creatorUserId);
    steamTable.put("BTRACK_PRIORITY", "" + priority);

    steamTable.put("BTRACK_SEVERITY", "" + severity);
    steamTable.put("BTRACK_VERSION", "" + version);
    steamTable.put("BTRACK_SUMMARY", "" + summary);

    steamTable.put("BTRACK_CREATOR_USERNAME",
                   StringUtil.htmlQuoteString(creatorUsername));
    steamTable.put("BTRACK_CREATED", "" + created);
    steamTable.put("BTRACK_MODIFIED", "" + modified);
    steamTable.put("BTRACK_STATUS", StringUtil.htmlQuoteString(status));

    return true;
  }






  boolean getProjectData(int projectId, Hashtable steamTable, Connection con) throws BugTrackerException {

    String statement = "select short_title, long_title, text, status, " +
      "creator_userid, created, modified from btrack_projects "+
      "where id = " + projectId;

    String shortTitle;
    String longTitle;
    String text;
    String status;
    int creatorUserId;
    Date created;
    Date modified;
    String creatorUsername;

    // First let's get the data from the database
    try {
      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      // We should get exactly one row back:
      if (results.next()) {
        shortTitle = results.getString(1);
        longTitle = results.getString(2);
        text = results.getString(3);
        status = results.getString(4);
        creatorUserId = results.getInt(5);
        created = results.getTimestamp(6);
        modified = results.getTimestamp(7);
      } else {
        results.close();
        stmt.close();
        return false;
      }

      results.close();
      stmt.close();

      // Now that we have the fundamental data, let's translate the
      // creator's userid into their username (easier to read, you
      // know.)
      creatorUsername = User.getUsername(con, creatorUserId);
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    // Screen out any bad values from the database so we don't cause any
    // exceptions:
    if (shortTitle == null) shortTitle = "";
    if (longTitle == null) longTitle = "";
    if (text == null) text = "";
    if (status == null) status = "";

    // Put everything into the subs hashtable for the user's environment:
    steamTable.put("BTRACK_PROJECT_ID", "" + projectId);
    steamTable.put("BTRACK_CREATOR_USERID", "" + creatorUserId);
    steamTable.put("BTRACK_CREATOR_USERNAME",
                   StringUtil.htmlQuoteString(creatorUsername));
    steamTable.put("BTRACK_CREATED", "" + created);
    steamTable.put("BTRACK_MODIFIED", "" + modified);
    steamTable.put("BTRACK_STATUS", StringUtil.htmlQuoteString(status));
    steamTable.put("BTRACK_TEXT", StringUtil.htmlQuoteString(text));
    steamTable.put("BTRACK_SHORT_TITLE",
                   StringUtil.htmlQuoteString(shortTitle));
    steamTable.put("BTRACK_LONG_TITLE",
                   StringUtil.htmlQuoteString(longTitle));

    return true;

  }





  int insertProject(int projectId, String shortTitle, String longTitle,
                    String text, int userId, Connection con) throws BugTrackerException {
    int result = 0;

    // create string for insert in upcoming synchronized section.
    StringBuffer project_insertion = new StringBuffer(
      "insert into btrack_projects (ID, SHORT_TITLE, LONG_TITLE, TEXT, "+
      "STATUS, CREATOR_USERID, CREATED, MODIFIED) values (");
    project_insertion.append(projectId);
    project_insertion.append(",");
    project_insertion.append(SQL.quoteString(shortTitle));
    project_insertion.append(",");
    project_insertion.append(SQL.quoteString(longTitle));
    project_insertion.append(",");
    project_insertion.append(SQL.quoteString(text));
    project_insertion.append(", 'new',");
    project_insertion.append(userId);
    project_insertion.append(", ?, ?)");

    try {
      PreparedStatement pstmt = con.prepareStatement(project_insertion.toString());
      Timestamp t = new Timestamp(System.currentTimeMillis());
      pstmt.setTimestamp(1, t);
      pstmt.setTimestamp(2, t);
      result = pstmt.executeUpdate();
      pstmt.close();
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    return result;
  }



  int updateProject(int projectId, String shortTitle, String longTitle,
                    String text, String status, Connection con) throws BugTrackerException {
    int result = 0;

    String statement = "update btrack_projects set " +
      "short_title = " + SQL.quoteString(shortTitle) + ", " +
      "long_title = " + SQL.quoteString(longTitle) + ", " +
      "text = " + SQL.quoteString(text) + ", " +
      "status = " + SQL.quoteString(status) + ", " +
      "modified = ?" +
      " where id = " + projectId;

    try {
      PreparedStatement pstmt = con.prepareStatement(statement);
      pstmt.setTimestamp(1, new Timestamp(System.currentTimeMillis()));
      result = pstmt.executeUpdate();
      pstmt.close();
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    return result;
  }




  int insertBug(int bugId, int projectId, String version, String priority,
                String severity, String summary, int userId,
                Connection con) throws BugTrackerException {
    int result = 0;

    // create the query before entering the synchronized area for
    // so others don't have to wait for this to be generated
    StringBuffer bug_insertion = new StringBuffer(
      "insert into btrack_bugs (ID, PROJECT_ID, VERSION, STATUS, PRIORITY, " +
      "SEVERITY, SUMMARY, CREATOR_USER_ID, CREATED, MODIFIED) values (");
    bug_insertion.append(bugId);
    bug_insertion.append(",");
    bug_insertion.append(projectId);
    bug_insertion.append(",");
    bug_insertion.append(SQL.quoteString(version));
    bug_insertion.append(", 'new',");
    bug_insertion.append(SQL.quoteString(priority));
    bug_insertion.append(",");
    bug_insertion.append(SQL.quoteString(severity));
    bug_insertion.append(",");
    bug_insertion.append(SQL.quoteString(summary));
    bug_insertion.append(", ");
    bug_insertion.append(userId);
    bug_insertion.append(", ?, ?)");

    // we have the statement created, insert the new bug into the database:
    try {
      PreparedStatement pstmt = con.prepareStatement(bug_insertion.toString());
      Timestamp t = new Timestamp(System.currentTimeMillis());
      pstmt.setTimestamp(1, t);
      pstmt.setTimestamp(2, t);
      result =  pstmt.executeUpdate();
      pstmt.close();
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    return result;
  }




  int updateBug(int bugId, int projectId, String version, String priority,
                String status, String severity, String summary,
                Connection con) throws BugTrackerException {
    int result = 0;

    String statement = "update btrack_bugs set " +
      "project_id = " + projectId + ", " +
      "priority = " + priority + ", " +
      "status = " + SQL.quoteString(status) + ", " +
      "severity = " + SQL.quoteString(severity) + ", " +
      "version = " + SQL.quoteString(version) + ", " +
      "modified = ?, summary = "+ SQL.quoteString(summary)+
      " where id = " + bugId;

    try {
      PreparedStatement pstmt = con.prepareStatement(statement);
      pstmt.setTimestamp(1, new Timestamp(System.currentTimeMillis()));
      result = pstmt.executeUpdate();
      pstmt.close();
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    return result;
  }







  int insertBugText(int bugTextId, int bugId, int userId, String comments,
                    Connection con) throws BugTrackerException {
    int result = 0;

    // create the query before entering the synchronized area for
    // so others don't have to wait for this to be generated
    StringBuffer bug_comment_insertion = new StringBuffer(
      "insert into btrack_bug_text(bug_text_id, bug_id, creator_user_id, text, created) values(");
    bug_comment_insertion.append(bugTextId);
    bug_comment_insertion.append(", ");
    bug_comment_insertion.append(bugId);
    bug_comment_insertion.append(", ");
    bug_comment_insertion.append(userId);
    bug_comment_insertion.append(", ");
    bug_comment_insertion.append(SQL.quoteString(comments));
    bug_comment_insertion.append(", ?)");

    if (!comments.equals("")) {
      try {
        // insert the descriptive comment for the new bug to the db
        PreparedStatement pstmt = con.prepareStatement(bug_comment_insertion.toString());
        pstmt.setTimestamp(1, new Timestamp(System.currentTimeMillis()));
        result = pstmt.executeUpdate();
        pstmt.close();
      } catch (SQLException se) {
        throw new BugTrackerException(""+se);
      }
    }

    return result;
  }





  /**
   * Given a bug id, use the also-supplied Connection to return the string
   * representing the concatenated list of comments supplied for that bug.
   */

  String assembleCommentChunk(int bugId, Connection con) throws BugTrackerException {

    // We want to extract all the data in a reverse-time order so that the
    // most recent comments come first:
    String statement = "select text, creator_user_id, created from " +
                       "btrack_bug_text where bug_id = " + bugId + " order by " +
                       "created desc";

    String cur_text;
    int cur_userid;
    String cur_username;
    Date cur_created;

    // compiled_string will hold the end-result of what we make here.
    StringBuffer compiled_string = new StringBuffer();

    // No synchronization is necessary since sync's have been locked
    // in all above method calls (including, as of now,
    // displayEditBugAgain, handleListBugs, and handleEditBugFormGen.)

    try
    {
      Statement stmt = con.createStatement();
      ResultSet results = stmt.executeQuery(statement);

      boolean keepgoing = true;

      if (results.next())
      {
        // Now we have retrieved the data from the database.
        // We will loop through each row assembling all the while
        // the string that represents the list of comments
        // (known colloquially as "compiled_string").
        while (keepgoing)
        {
          cur_text = results.getString(1);
          cur_userid = results.getInt(2);
          cur_created = results.getTimestamp(3);

          cur_username = User.getUsername(con, cur_userid);

          if (cur_text == null) cur_text = "";
          if (cur_username == null) cur_username = "";
          if (cur_userid == User.ANONYMOUS_ID) cur_username =
            "anonymous";

          // Now we have retrieved one comment from the database,
          // so let's "compile" it into the string:

          compiled_string.append("By: <b>" +
                                 StringUtil.htmlQuoteString(
                                                           cur_username) +
                                 "</b> - " + cur_created +
                                 "<BR>\n" +
                                 StringUtil.htmlQuoteString(cur_text));

          // We keep looping through the database results until we're
          // out of retrieved rows:
          keepgoing = results.next();

          if (keepgoing) compiled_string.append("<BR>\n");
        }
      }

      results.close();
      stmt.close();
    } catch (SQLException se) {
      throw new BugTrackerException(""+se);
    }

    return(compiled_string.toString());
  }





}
