/**
 * DisplayPageServlet.java
 *
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License. 
 *
 * The initial develop of this code under the LPL Leverage Information Systems.
 * Portions created by Leverage are Copyright (C) 1998 Leverage Information Systems.
 * All Rights reserved.
 */

package org.locomotive.module.display;
import org.locomotive.util.*;
import org.locomotive.server.*;
import org.locomotive.loco.*;
import org.locomotive.loco.servlet.*;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * DisplayPageServlet can be used to map a url into the path of a HTML 
 * template file in the file system.  This gives the Servlet implementer
 * an easy way to evaluate a hierarchy of templates with a given 
 * default or &quot;generic&quot; environment.  
 * For instance, this class would
 * map urls beginning with &quot;disp?base+generic&quot; to the template root
 * sub directory &quot;base/generic&quot;  Therefore, <BR>
 * <PRE> 
 * disp?base+main          maps to $TEMPLATE_ROOT/base/main.tmpl
 * disp?base+generic+page1 maps to $TEMPLATE_ROOT/base/generic/page1.tmpl
 * disp?base+generic+page2 maps to $TEMPLATE_ROOT/base/generic/page2.tmpl
 * </PRE> 
 *
 * DisplayPageServlet will also log the Event of the generic template display, 
 * and neutralize any file system separators/operators from the url_tokens
 * list for security purposes.  In other words, all non-alphanumeric letters
 * in the URL will be converted into a '/', meaning that each alphanumeric
 * word will become a directory name except for the last, which becomes the
 * name of the file. <BR>
 * In order to use this Servlet, you have to have an entry for this class
 * in your servlet.conf config file, ex:
 * <BLOCKQUOTE><PRE>
 * disp          org.locomotive.module.display.DisplayPageServlet
 * </PRE></BLOCKQUOTE>
 * or, alternatively, you can invoke this Servlet from another Servlet:
 * <BLOCKQUOTE><PRE>
 * RequestDispatcher reqd = getServletContext().getRequestDispatcher(
 * handler_uri + "/disp?");
 * reqd.forward(req, resp);
 * </PRE></BLOCKQUOTE>
 */
public class DisplayPageServlet extends LocoServlet
{
    /**
     * main entry point.
     */
    public void service(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	setup(req, resp);
	
        // insert some standard Steam variables, such as date and time.
        StandardInsertions.insertAll (req, steam_vars);
	
        // check that the url_tokens are not null or empty, and have
        // only alphanumeric characters
// XXXXXX CK says: allow some other characters too.
// XXXXXX Ben says: but not shell special chars, like |, <, >, !, $, `, ', \
        if (url_tokens != null && url_tokens.length > 0 &&
	    StringUtil.isAlphanum (
		StringUtil.stringArrayToString(url_tokens, "")))
	{
	    // convert "base?login+form" to "base/login/form.tmpl",
	    // then display that file as a HTML template.
	    String result = displayPage (
		StringUtil.stringArrayToString(url_tokens, "/") + ".tmpl",
		steam_vars);
	    
	    // if there is no file in that file path, then
	    // displayPage() will return an empty string, "", in which
	    // case we should tell the user that the URL was faulty.
	    if (result.equals(""))
		sayBadURL();
	}
	else
	{
	    sayBadURL();
	}
    }

}
