/**
 * GraffitiWallServlet.java
 * (demo application)
 *
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License. 
 *
 * The initial develop of this code under the LPL Leverage Information Systems.
 * Portions created by Leverage are Copyright (C) 1998 Leverage Information Systems.
 * All Rights reserved.
 */

package org.locomotive.module.graffiti;

import java.lang.*;
import java.text.SimpleDateFormat;
import java.sql.SQLException;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.locomotive.loco.*;
import org.locomotive.loco.servlet.*;
import org.locomotive.module.base.*;
import org.locomotive.server.Log;

/** This class implements the GraffitiWall example module, as described in the
 * Locomotive Developers Guide.  Please see 
 * <a href="http://www.org.locomotive.com/docs/loco/dev_frame.html>the guide</a>
 * for more information.  To use this Servlet, insert the following line into
 * the Servlet Routing Table:
 * <BLOCKQUOTE><PRE>
 * graf           org.locomotive.module.graffiti.GraffitiWallServlet
 * </PRE></BLOCKQUOTE>
 */
public class GraffitiWallServlet extends LocoServlet
{
    // Informix has a limit on the length of varchars -- 255 characters
    static int maxTagLength = 255;
    static int maxUsrNameLength = 32; // matches size of varchar in db.

    // Used to remember how long the form_tag submitted by the user was.
    // used only for event-logging.
    int form_tag_length = 0;
    

    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	displayGraffitiPage();
	logEvent ("graf", "pageDisp", "");
    }
    
    
    public void doPost(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	// if form is null, then pretend we had an HTTP 'GET' request.
	if (form_data.isEmpty())
	{
	    displayGraffitiPage();
	    logEvent ("graf", "pageDisp", "");
	}
	else
	{
	    submitFormGraffitiPage();
	    displayGraffitiPage();
	    logEvent("graf", "tagPosted", String.valueOf(form_tag_length));
	}
    }

    /**
     * loads the graffiti page.
     * if isGet is true, an event is logged,
     * otherwise an event is not logged since a
     * a post event was already logged
     */
    private void displayGraffitiPage()
	throws ServletException, IOException
    {
	String tagRows ="";
	try
	{ 
	    tagRows = getTagsAsHTMLRows();
	} 
	catch (SQLException e)
	{
	    //log error and inform user by loading error page
	    handleException(e, "disp_page");
	    return;
	}
       
	steam_vars.put ("TAG_ROWS", tagRows);

	//load page
	displayPage ("graffiti/graffiti.tmpl", steam_vars);
    }

    private void submitFormGraffitiPage()
    {  
	String errors = "";
	String event_code = "";

	// get the new tag
	String form_tag = form_data.getString("NEW_TAG", 
					      "New Graffiti Wall Tag", 
					      0, maxTagLength);

	if (form_data.hasAnyErrors() && !form_data.hasError(FormData.TOO_LONG))
	{
	    // if the error was an error other than that the tag was too long, 
	    // we should give that error message back to the user.
	    
	    steam_vars.put ("GRAF_ERRORS", form_data.allErrorMsgs());
	    return;
	}
	else
	// it's alright if the Tag is too long.  We'll just truncate it.
	{
	    // load new graffiti tag into db
  
	    // get the username
	    String postUsrName;
	    if (user.isAnonymous())
		postUsrName = "anonymous";
	    else 
		postUsrName = user.getUsername(); 
 
	    // if the tag was too long, truncate it
	    if (form_tag.length() >= maxTagLength)
	    {
		String trunc = "... (truncated)";

		form_tag = form_tag.substring
		    (0, maxTagLength - 1 - trunc.length()) + trunc;
	    } 
	    else if (form_tag.equals(""))
	    {
		form_tag = " ";
	    }

	    // if the username was too long, then truncate it
	    if (postUsrName.length() >= maxUsrNameLength)
		postUsrName = postUsrName.substring(0, maxUsrNameLength - 1);
	    
	    // store the tag in the db.  Notice that we use the 
	    // new Tag() constructor to accomplish this.
	    try {
		new Tag(conn, postUsrName, form_tag);
	    } catch (SQLException e) {
		handleException (e, "make_new_tag");
		return;
	    }
	}	
    }


    private String getTagsAsHTMLRows() 
	throws SQLException
    {
	StringBuffer html_rows = new StringBuffer();

//	int index = 0;
	Tag[] tags = Tag.loadAllTags (conn);
 
	// the following will allow us to format a date to look like:
	//     Feb 14  at 8:45:00 PM
	SimpleDateFormat timeFmt = new SimpleDateFormat
	    ("MMM d 'at' h:mm:ss a");

	for (int index = 0; index < tags.length; index++)
	{
	    if (tags[index].getText() != null)
	    {
		html_rows.append("<TR>");  
		html_rows.append("<TD>");
		html_rows.append(timeFmt.format(tags[index].getInsTime()));
		html_rows.append("</TD>");
		   
		html_rows.append("<TD>");  
		html_rows.append(tags[index].getAuthor());
		html_rows.append("</TD>");
		   
		html_rows.append("<TD>");
		html_rows.append(tags[index].getText());
		html_rows.append("</TD></TR> \n");
	    }
	}
	
	return html_rows.toString();
    }

}
