/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */


//kore@telebot.net: added 3 lines to support PostgreSQL (search for "POSTGRESQL" in file);


package org.locomotive.module.userpref;

import java.lang.*;
import java.text.SimpleDateFormat;
import java.sql.*;
import java.util.*;

import org.locomotive.loco.*;
import org.locomotive.loco.perm.*;
import org.locomotive.util.StringConstraint;
import org.locomotive.server.Log;
import org.locomotive.util.SQL;

/**
 *  This class handles db processing and template interaction 
 *  (except for saving error messages to template) for
 *  UserPref and UserMgt handlers.  
 *
 */

public class UserPref
{
    public String userId = "";
    public String userName = "";
    public String fName = "";     
    public String mName = "";
    public String lName = "";
    public String email = "";
    public String hAddr = "";
    public String hCity = "";
    public String hState = "";
    public String hZip = "";
    public String hCountry = "";
    public String wAddr = "";
    public String wCity = "";
    public String wState = "";
    public String wZip = "";
    public String wCountry = "";
    public String hPhone = "";
    public String wPhone = "";
    public String passHint = "";
    public String passWd = "";
    public String lastLogin = "";

    public UserPref() {super();}

    /**
     * Returns an object with userId and userName fields 
     */
    public UserPref(int id, String username)
    {
	this.userId = String.valueOf(id);
	this.userName = username;
    }

    /**
     * Inits object from database with userid = userID.  If a db entry does
     * not exist, an object is returned with userId and userName fields 
     */
    public UserPref (Connection conn, int usrId) throws SQLException
    {                  
	// sets user id
	this.userId = Integer.toString(usrId);

	// looks up username and sets userName
	String statement = "select username from loco_users where "+
	    "userid ="+usrId;
	Statement stmt = null;
	ResultSet rs = null;
            
	try
	{
	    stmt = conn.createStatement();
	    rs = stmt.executeQuery (statement);           
	 
	    if (rs.next())
		this.userName = rs.getString(1);
	}
	catch (SQLException se)
	{
	    Loco.log (0, "ERROR: " + se + "\nquery: " + statement);
	    throw se;
	}
	finally
	    {
		if (rs != null) rs.close();
		if (stmt != null) stmt.close();
	    }

	// checks if user_pref entry exists or userId
	// if not -return an object with username and id initialized
	if(!entryExists(conn, userId))
	{
	    return;
	}

	// gets user prefs information from db

	statement = 
	    "select loco_user_prefs.firstname, middlename, " +
	    "loco_user_prefs.lastname, loco_users.last_login, " +
	    "email_addr, home_addr, home_city, home_state, home_zip, "+
	    "home_country, work_addr, work_city, work_state, work_zip,"+
	    "work_country, home_ph, work_ph, passwd_hint, "+
	    "password from loco_user_prefs, loco_users "+
	    "where user_id_ref = userid and userid = "+ userId;

	try
	{
	    stmt = conn.createStatement();
	    rs = stmt.executeQuery (statement);
	 
	    if (rs.next())
	    {
		fName = checkNull(rs.getString(1)); 
		mName = checkNull(rs.getString(2)); 
		lName = checkNull(rs.getString(3));   	       
		lastLogin = checkNull(rs.getString(4));
		email = checkNull(rs.getString(5));   
		hAddr = checkNull(rs.getString(6)); 
		hCity = checkNull(rs.getString(7)); 
		hState = checkNull(rs.getString(8)); 
		hZip = checkNull(rs.getString(9)); 
		hCountry = checkNull(rs.getString(10)); 
		wAddr = checkNull(rs.getString(11)); 
		wCity = checkNull(rs.getString(12)); 
		wState = checkNull(rs.getString(13)); 
		wZip = checkNull(rs.getString(14)); 
		wCountry = checkNull(rs.getString(15)); 
		hPhone = formatPhoneNumber(checkNull(rs.getString(16)));   
		wPhone = formatPhoneNumber(checkNull(rs.getString(17))); 
		passHint = checkNull(rs.getString(18)); 
		passWd = checkNull(rs.getString(19));
	     
	    }
	 

	}
	catch (SQLException se)
	{
	    Loco.log (0, "ERROR: " + se + "\nquery: " + statement);
	    throw se;
	}
	finally
	    {
		if (rs != null) rs.close();
		if (stmt != null) stmt.close();
	    }

    }

    /**  
     * Searches database with criteria provided in the parameter list and
     * creates a table with search results.  
     * Returns true if successful and the result is stored in parameter 
     * resultTable, otherwise false and an error string
     */


    public static boolean getSearchResultTableRows( Connection conn, 
						    String username,
						    String lastname, 
						    String emailaddr, 
						    String userid,
						    StringBuffer resultTable,
						    StringBuffer errors, 
						    int maxNumRows) 
	throws SQLException
    {
	// NOTE: The implementation of this method could be improved to 
	//       return a SortableTable object instead of a string of
	//       html in parameter resultTable
	
	StringBuffer userCriteria = new StringBuffer();
	StringBuffer userPrefCriteria = new StringBuffer();
	
	// checks other search constraints in case userid not given
	if (userid.equals(""))
	{         
	    if (!username.equals(""))
	    {
		userCriteria.append(
		    " lower(lu.username) LIKE " + 
		    SQL.quoteString("%"+ username.toLowerCase()+"%")+" and ");
	    }

	    if (!lastname.equals(""))
	    {
		userPrefCriteria.append(
		    " lower(lup.lastname) LIKE "+ 
		    SQL.quoteString("%"+lastname.toLowerCase()+"%")+" and ");
	    }

	    if (!emailaddr.equals(""))
	    {
		userPrefCriteria.append(
		    " lower(lup.email_addr) LIKE "+ 
		    SQL.quoteString("%"+emailaddr.toLowerCase()+"%")+" and ");
	    }
	}
	else // in this case, search criteria is just an exact search on user id
	{
	    // ensure userid is a number so sql won't give an error.
	    try 
	    {
		int uid = Integer.parseInt(userid); 
	    }
	    catch (NumberFormatException nfe)
	    {
		errors.append("User Id must be a number");
		return false;
	    }
	    userCriteria.append(" lu.userid = "+userid + " and ");
	}

	// in this case, search fields were left blank
	if (userCriteria.length()==0 && userPrefCriteria.length()==0 )
	{
	    errors.append("Search fields left blank. Please enter data ");
	    errors.append("before selecting \"Search\"");
	    return false;
	}
	
	// let's stip off those trailing 'ands'
	if (userCriteria.length() > 0 && userPrefCriteria.length()==0)
	{
	    // only if we have no following userPrefCriteria
	    userCriteria.setLength(userCriteria.length() - 4);
	}
	else if (userPrefCriteria.length() > 0)
	{
	    userPrefCriteria.setLength(userPrefCriteria.length() - 4);
	}

	// creates search fquery with search criteria
	StringBuffer statement = new StringBuffer();
	statement.append("select lu.username, lup.firstname, lup.lastname, ");
	statement.append("lup.email_addr, lu.userid ");

	switch (LocoConfig.DB_TYPE)
	{
	case SQL.DB_TYPE_MYSQL:
	    statement.append("from loco_users lu ");
	    statement.append("LEFT OUTER JOIN loco_user_prefs lup ON ");
	    statement.append("lu.userid = lup.user_id_ref where ");
	    break;
	case SQL.DB_TYPE_ORACLE:
	    statement.append("from loco_users lu,  loco_user_prefs lup ");
	    statement.append("where lu.userid = lup.user_id_ref(+) and ");
	    break;
	case SQL.DB_TYPE_POSTGRESQL:
	    statement.append("from loco_users lu,  loco_user_prefs lup ");
	    statement.append("where lu.userid = lup.user_id_ref and ");
	    break;	
	}

	statement.append(userCriteria);
	statement.append(userPrefCriteria);
      
	// executes query and substitutes the values into a set
	// of html table 
      
	Statement stmt = null;
	ResultSet rs = null;

	int rowCount = 0;
      
	try
	{
	    stmt = conn.createStatement();
	    rs = stmt.executeQuery (new String(statement));
	     
	    while (rs.next() && (rowCount++ < maxNumRows))
	    { 
		//create html row and append to result string so
		//it can be displayed in results list.
		resultTable.append("<TR><TD>");
		resultTable.append("<A HREF=\"");
		resultTable.append("userpref?manage+userinfo+");
		resultTable.append(rs.getString(5));
		resultTable.append("\">" + rs.getString(1)+"</A> </TD>");
		 
		resultTable.append("<TD>"+checkNull(rs.getString(2))+"</TD>");
		resultTable.append("<TD>"+checkNull(rs.getString(3))+"</TD>");
		resultTable.append("<TD>"+checkNull(rs.getString(4))+"</TD>");
		resultTable.append("<TD>"+rs.getString(5)+"</TD>");
		resultTable.append("</TR>");   
		 
	    }  
	}
	catch (SQLException se)
	{
	    Loco.log(0, "ERROR: " + se + "\n" + statement);
	    throw se;
	}
	finally
	{
	    if (rs != null) rs.close();
	    if (stmt != null) stmt.close();
	}


/*
	// previous query only searched on users who exist in the 
	// loco_user_prefs table.  now check the user table with 
	// search criteria on users who do not exist in the loco_user_prefs
	// table, when the search criteria contained username constraint
	// values
	if (rowCount <  maxNumRows && (!username.equals("")))
	{               
	    String newQuery = "select username, userid from "+
		"loco_users where( not exists (select 'nothing' from "+
		"loco_user_prefs where userid = user_id_ref))";
	    try
	    {
		stmt = conn.createStatement();
		statement = new StringBuffer(newQuery+userCriteria);
		rs = stmt.executeQuery(new String(statement));
		 
		while (rs.next() &&  rowCount++ < maxNumRows )
		{
		    //create html row and append to result string so
		    //it can be displayed in results list.
		    resultTable.append("<TR><TD>");
		    resultTable.append("<A HREF=\"");
		    resultTable.append("userpref?manage+userinfo+");
		    resultTable.append(rs.getString(2)+ 
				       "\">" + rs.getString(1)+"</A> </TD>");
		    resultTable.append("<TD></TD><TD></TD><TD></TD> ");
		    resultTable.append("<TD>"+rs.getString(2)+"</TD></TR>");
		}
	    }
	    catch (SQLException se)
	    {
		Loco.log (0, "ERROR: " + se + "\n" + newQuery);
		throw se;
	    }
	    finally
	    {
		if (rs != null) rs.close();
		if (stmt != null) stmt.close();
	    }
	}           
	*/

	return true;   
    }



    public String printData()
    {
	return new String (userId+" "+ userName + "" + fName + ""+ mName + ""
+ lName + ""+ email + ""+ hAddr + ""+ hCity + "" + hState + ""
+ hZip + ""+ hCountry + "" + wAddr + "" + wCity + "" + wState + ""
+ wZip + ""+ wCountry + "" + hPhone + "" + wPhone + ""
+ passHint + "" + passWd + "" + lastLogin + "");
    }



    public static boolean storeUserPrefsToDB(Connection conn, 
					     UserPref user, 
					     StringBuffer errors) 
	throws LoadUserException, SQLException
    {

	// ensure an entry exists for the usrId in loco_user; 
	// the primary key of loco_user_prefs refers to the user's
	// userid in loco_user, so an entry MUST exist for db consistency
	if (!User.exists(conn, Integer.parseInt(user.userId)))
	{
	    errors.append("Entry does not exist for the ");
	    errors.append("user id "+ user.userId + " in loco_users ");
	    errors.append("- cannot save changes.");
	    return false;
	}
	else  // a loco_users entry exists - save vals to db.
	{
	  
	    String statement = null;
	  
	    if (!entryExists(conn, user.userId))  //do an insert
	    {
		//old!!  just format!!
		statement = 
		    "insert into loco_user_prefs (user_id_ref, "+
		    "firstname, middlename, "+
		    "lastname, email_addr, home_addr, home_city, " + 
		    "home_state, home_zip, home_country, work_addr, "+
		    "work_city, work_state, work_zip, work_country, "+
		    "home_ph, work_ph, passwd_hint) " + 
		    "values(" + user.userId + ",?,?,?,?,?,?,"+
		    "?,?,?,?,?,?,?,?,?,?,?)";
	    }
	    else //do an update
	    {
		statement = 
		    "update loco_user_prefs set "+
		    "firstname = ?, middlename = ?, lastname = ?,"+
		    "email_addr = ?, home_addr = ?, home_city = ?, " + 
		    "home_state = ?, home_zip = ?, home_country = ?,"+
		    "work_addr = ?, work_city = ?, work_state = ?," +
		    "work_zip = ?, work_country = ?, home_ph = ?,"+
		    "work_ph = ?, passwd_hint = ? "+
		    "where user_id_ref=" + user.userId;
	    }

	    PreparedStatement pstmt = null;

	    try
	    {
		pstmt = conn.prepareStatement(statement);

		pstmt.setString (1, user.fName);
		pstmt.setString (2, user.mName);
		pstmt.setString (3, user.lName);
		pstmt.setString (4, user.email);
		pstmt.setString (5, user.hAddr);
		pstmt.setString (6, user.hCity);
		pstmt.setString (7, user.hState);
		pstmt.setString (8, user.hZip);
		pstmt.setString (9, user.hCountry);
		pstmt.setString (10, user.wAddr);
		pstmt.setString (11, user.wCity); 
		pstmt.setString (12, user.wState);
		pstmt.setString (13, user.wZip);
		pstmt.setString (14, user.wCountry);
		pstmt.setString (15, user.hPhone);
		pstmt.setString (16, user.wPhone);
		pstmt.setString (17, user.passHint);
	      
		pstmt.executeUpdate();
	    }
	    catch (SQLException se)
	    {
		Loco.log (0, "ERROR: " + se + "\n" + statement);
		throw se;
	    }
	    finally
	    {
		if (pstmt != null) pstmt.close();
	    }

	    //password fields in a different table. update now.
	  
	    if (!user.passWd.equals(""))
	    {
		statement = "Update loco_users set password=?" + 
		    "where userid=" + user.userId;
		try
		{
		    pstmt = conn.prepareStatement(statement);
		    pstmt.setString(1, user.passWd);
		    pstmt.executeUpdate();
		}
		catch (SQLException se)
		{
		    Loco.log (0, "ERROR: " + se + "\n" + statement);
		    throw se;
		}
		finally
		    {
			if (pstmt != null) pstmt.close();
		    }
	    }
	} 
	  
	return true;
    }


    /**
     *  returns true if an entry already exists in loco_user_prefs
     *  for usrId.  (used by "storeUserPrefsToDB, indicating whether
     *  an insert or an update should be performed.
     */
    public static boolean entryExists(Connection conn, String usrId) 
	throws SQLException
    {
	String statement = "select 'nothing' from loco_user_prefs "+
	    "where user_id_ref=" + usrId;

	Statement stmt = null;
	ResultSet rs = null;

	try
	{
	    stmt = conn.createStatement();
	    rs = stmt.executeQuery (statement);
	   
	    if (rs.next())
		return true;
	    else
		return false;
	}
	catch (SQLException se)
	{
	    Loco.log (0, "ERROR: " + se + "\n" + statement);
	    throw se;
	}
	finally
	{
	    if (rs != null) rs.close();
	    if (stmt != null) stmt.close();
	}
    }

    /** checks if the user with userid has admin priveleges for the UMS */
    public static boolean hasAdminPriveleges(Connection conn, int userid) 
	throws SQLException
    {
	return  UserPerm.getPermissions(userid, conn).isSet("UMS_ADMIN");
    }
 
    /**
     *  if str is null, or str equals "null" it is modified to point to 
     *  an empty string instead
     */
    private static String checkNull(String str) 
    {
	if (str == null ||str.equals("null") )
	    str="";
	return str;
    }

    /**
     *  takes a 10 digit number and inserts characters to format into
     *  a phone number in the form, "(123)456-7890".  if the number is
     *  not 10 characters long, a null string is returned. 
     */
    private String formatPhoneNumber(String number) 
    {
	if (number != null && (number.length()==10))
	{
	    StringBuffer phone = new StringBuffer(13);
	    phone.insert(0, "(");
	    phone.append(number.substring(0, 3));
	    phone.insert(4, ")");
	    phone.append(number.substring(3,6));
	    phone.insert(8, "-");
	    phone.append(number.substring(6, 10));
	    return new String(phone);
        }
        else  //cannot format phone number, return original 
	    return number;
    }
}






