/**
 * UserPrefServlet.java 
 *
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License. 
 *
 * The initial develop of this code under the LPL Leverage Information Systems.
 * Portions created by Leverage are Copyright (C) 1998 Leverage Information Systems.
 * All Rights reserved.
 */

package org.locomotive.module.userpref;

import java.lang.*;
import java.text.SimpleDateFormat;
import java.sql.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.locomotive.loco.*;
import org.locomotive.loco.servlet.*;
import org.locomotive.module.base.*;
import org.locomotive.server.Log;

/**
 * This class must be entered in the servlet routing table in order to
 * access the UserPref sample module.  This can be done by placing
 * the following line in your hrt.conf file: <PRE>
 *     usrpref                    org.locomotive.module.userpref.UserPrefServlet
 * </PRE>
 * @see org.locomotive.module.userpref.PersonalPrefServlet
 */

public class UserPrefServlet extends LocoServlet
{
    private PersonalPrefServlet pps;
    private UserMgtServlet ums;
    
    /**
     * Creates our internal servlets
     */
    public void init(ServletConfig config) throws ServletException
    {
	super.init(config);
	pps = new PersonalPrefServlet();
	pps.init(config);
	
	ums = new UserMgtServlet();
	ums.init(config);
    }
	
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	routeToSubServlet(req, resp);
    }
    
    public void doPost(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException
    {
	routeToSubServlet(req, resp);
    }
    
    /**
     * This will route a request to the PersonalPrefServlet or
     * UserMgtServlet class. 
     */
    public void routeToSubServlet (HttpServletRequest req,
				   HttpServletResponse resp)
	throws ServletException, IOException
    {
	// call the Loco library call for standard insertions.  
	StandardInsertions.insertAll (req, steam_vars);

	if (url_tokens.length < 2 || url_tokens[1].equals("personal"))
	{
	    pps.service (req, resp);
	}
	else if ( url_tokens[1].equals("manage"))
	{
	    ums.service (req, resp);
	}
	else
	{
	    displayPage ("error/unknown_servlet_option.tmpl", steam_vars);
	}
    }
}
