package org.locomotive.obf;

import java.util.*;
import java.lang.*;


public final class Debug {
   public static final int DEBUG_LEVEL_NONE     = 0;
   public static final int DEBUG_LEVEL_ASSERT   = 1;
   public static final int DEBUG_LEVEL_SELECTED = 2;
   public static final int DEBUG_LEVEL_ALL      = 3;

   private static int debugLevel = DEBUG_LEVEL_SELECTED;


   public static final int CLASSNAME_NOT_SHOWN = 0;
   public static final int CLASSNAME_AT_START  = 1;
   public static final int CLASSNAME_AT_END    = 2;

   private static int classNamePos = CLASSNAME_AT_END;


   public static final int SHORT_CLASSNAME = 0;
   public static final int FULL_CLASSNAME  = 1;

   private static int classNameLength = SHORT_CLASSNAME;



   private static Vector debugClasses = null;

   private static int debugDepth = 0;


   private Debug() {
      super();
   }

   public static void msg(Class c, Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (debugLevel == DEBUG_LEVEL_ASSERT) return;
      if (  (debugLevel == DEBUG_LEVEL_SELECTED) && ( !isClassInDebug(c) )  ) return;

      int p = getClassNamePos();

      String indent = "  ";
      while (indent.length() < debugDepth) indent = indent + " ";

      if (p == Debug.CLASSNAME_NOT_SHOWN) System.err.println(indent + "<" + msg + ">");
      else if (p == Debug.CLASSNAME_AT_START)  System.err.println(indent + getClassName(c) + " <" + msg + ">" );
      else if (p == Debug.CLASSNAME_AT_END)    System.err.println(indent + "<" + msg + "> " + getClassName(c));
      else                                     System.err.println(indent + "INVALID ClassNamePos in DebugClasses <" + msg + "> " + getClassName(c));
   }

   public static void msg(Class c, boolean test, Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (debugLevel == DEBUG_LEVEL_ASSERT) return;
      if (  (debugLevel == DEBUG_LEVEL_SELECTED) && ( !isClassInDebug(c) )  ) return;

      if (test) return;

      int p = getClassNamePos();

      String indent = "  ";
      while (indent.length() < debugDepth) indent = indent + " ";

      if (p == Debug.CLASSNAME_NOT_SHOWN) System.err.println(indent + "<" + msg + ">");
      else if (p == Debug.CLASSNAME_AT_START)  System.err.println(indent + getClassName(c) + " <" + msg + ">" );
      else if (p == Debug.CLASSNAME_AT_END)    System.err.println(indent + "<" + msg + "> " + getClassName(c));
      else                                     System.err.println(indent + "INVALID ClassNamePos in DebugClasses <" + msg + "> " + getClassName(c));
   }


   public static void msg(Class ac, Class tc, Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (debugLevel == DEBUG_LEVEL_ASSERT) return;
      if (  (debugLevel == DEBUG_LEVEL_SELECTED) && !( isClassInDebug(ac) || isClassInDebug(tc) )  ) return;

      int p = getClassNamePos();

      String indent = "  ";
      while (indent.length() < debugDepth) indent = indent + " ";

      if (p == Debug.CLASSNAME_NOT_SHOWN) System.err.println(indent + "<" + msg + ">");
      else if (p == Debug.CLASSNAME_AT_START)  System.err.println(indent + getClassName(ac) + " | " + getClassName(ac) + " <" + msg + ">" );
      else if (p == Debug.CLASSNAME_AT_END)    System.err.println(indent + "<" + msg + "> " + getClassName(ac) + " | " + getClassName(ac));
      else                                     System.err.println(indent + "INVALID ClassNamePos in DebugClasses <" + msg + "> " + getClassName(ac) + " | " + getClassName(ac)) ;
   }

   public static void msg(Class ac, Class tc, boolean test, Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (debugLevel == DEBUG_LEVEL_ASSERT) return;
      //if (  (debugLevel == DEBUG_LEVEL_SELECTED) && ( !isClassInDebug(ac) )  ) return;
      if (  (debugLevel == DEBUG_LEVEL_SELECTED) && ( !isClassInDebug(ac) || !isClassInDebug(tc) )  ) return;

      if (test) return;

      int p = getClassNamePos();

      String indent = "  ";
      while (indent.length() < debugDepth) indent = indent + " ";

      if (p == Debug.CLASSNAME_NOT_SHOWN) System.err.println(indent + "<" + msg + ">");
      else if (p == Debug.CLASSNAME_AT_START)  System.err.println(indent + getClassName(ac) + " | " + getClassName(ac) + " <" + msg + ">" );
      else if (p == Debug.CLASSNAME_AT_END)    System.err.println(indent + "<" + msg + "> " + getClassName(ac) + " | " + getClassName(ac));
      else                                     System.err.println(indent + "INVALID ClassNamePos in DebugClasses <" + msg + "> " + getClassName(ac) + " | " + getClassName(ac)) ;
   }


   public static void msgIn(Class ac, Class tc, Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (debugLevel == DEBUG_LEVEL_ASSERT) return;
      if (  (debugLevel == DEBUG_LEVEL_SELECTED) && !( isClassInDebug(ac) || isClassInDebug(tc) )  ) return;

      int p = getClassNamePos();

      String indent = "->";
      while (indent.length() < debugDepth) indent = indent + " ";

      if (p == Debug.CLASSNAME_NOT_SHOWN) System.err.println(indent + "<" + msg + ">");
      else if (p == Debug.CLASSNAME_AT_START)  System.err.println(indent + getClassName(ac) + " | " + getClassName(ac) + " <" + msg + ">" );
      else if (p == Debug.CLASSNAME_AT_END)    System.err.println(indent + "<" + msg + "> " + getClassName(ac) + " | " + getClassName(ac));
      else                                     System.err.println(indent + "INVALID ClassNamePos in DebugClasses <" + msg + "> " + getClassName(ac) + " | " + getClassName(ac)) ;
      debugDepth++;
   }

   public static void msgOut(Class ac, Class tc, Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (debugLevel == DEBUG_LEVEL_ASSERT) return;
      if (  (debugLevel == DEBUG_LEVEL_SELECTED) && !( isClassInDebug(ac) || isClassInDebug(tc) )  ) return;

      int p = getClassNamePos();


      debugDepth--;

      String indent = "<-";
      while (indent.length() < debugDepth) indent = indent + " ";

      if (p == Debug.CLASSNAME_NOT_SHOWN) System.err.println(indent + "<" + msg + ">");
      else if (p == Debug.CLASSNAME_AT_START)  System.err.println(indent + getClassName(ac) + " | " + getClassName(ac) + " <" + msg + ">" );
      else if (p == Debug.CLASSNAME_AT_END)    System.err.println(indent + "<" + msg + "> " + getClassName(ac) + " | " + getClassName(ac));
      else                                     System.err.println(indent + "INVALID ClassNamePos in DebugClasses <" + msg + "> " + getClassName(ac) + " | " + getClassName(ac)) ;
   }

   public static void assert(boolean test, String msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (!test) {
         RuntimeException re = new RuntimeException("Assertion failed: "+msg);
         re.printStackTrace();
         System.exit(1);
      }
   }
   
  public static void abort(Object msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      RuntimeException re = new RuntimeException("Aborted: "+msg);
      re.printStackTrace();
      System.exit(1);
   }

   public static void warning(String msg) {
      warning(false, msg);
   }

   public static void warning(boolean test, String msg) {
      if (debugLevel == DEBUG_LEVEL_NONE) return;
      if (!test) {

        System.err.println("WARNING <" + msg + ">");
      }
   }

  public static void setDebugLevel(int i) {
      switch (i) {
      case DEBUG_LEVEL_NONE    : debugLevel = DEBUG_LEVEL_NONE;     break;
      case DEBUG_LEVEL_ASSERT  : debugLevel = DEBUG_LEVEL_ASSERT;   break;
      case DEBUG_LEVEL_SELECTED: debugLevel = DEBUG_LEVEL_SELECTED; break;
      case DEBUG_LEVEL_ALL     : debugLevel = DEBUG_LEVEL_ALL;      break;
      default: System.err.println("INVALID argument value to Debug.setDebugLevel of: " + i); break;
      }
   }

   public static int getDebugLevel() {
      return debugLevel;
   }

   public synchronized static void addClassToDebug(Class c) {
      if ( !isClassInDebug(c) ) {
         if (debugClasses == null) debugClasses = new Vector(10, 10);
         debugClasses.addElement(c);
      }
   }

   public synchronized static void delClassToDebug(Class c) {
      if ( isClassInDebug(c) ) {
         Class cLookup = null;

         for (int i=0; i < debugClasses.size(); i++) {
            cLookup = (Class)debugClasses.elementAt(i);
            if (cLookup == c) {
               debugClasses.removeElementAt(i);
               if (debugClasses.size() == 0) debugClasses = null;

               return;
            }
         }
      }
   }

   public static boolean isClassInDebug(Class c) {
      if (debugClasses == null) return false;

      Class cLookup = null;

      for (int i=0; i < debugClasses.size(); i++) {
         cLookup = (Class)debugClasses.elementAt(i);
         if (cLookup == c) {
            return true;
         }
      }
      return false;
   }

   public static void setClassNamePos(int i) {
      switch (i) {
      case CLASSNAME_NOT_SHOWN: classNamePos = CLASSNAME_NOT_SHOWN; break;
      case CLASSNAME_AT_START : classNamePos = CLASSNAME_AT_START;  break;
      case CLASSNAME_AT_END   : classNamePos = CLASSNAME_AT_END;    break;
      default: System.err.println("INVALID argument value to Debug.setClassNamePos of: " + i); break;
      }
   }

   public static int getClassNamePos() {
      return classNamePos;
   }


   public static void setClassNameLength(int i) {
     switch (i) {
     case FULL_CLASSNAME:  classNameLength = FULL_CLASSNAME; break;
     case SHORT_CLASSNAME: classNameLength = SHORT_CLASSNAME; break;
     default:              throw new IllegalArgumentException("Illegal class name length: "+i);
     }
   }

   public static int getClassNameLength() {
     return classNameLength;
   }

   public static String getClassName(Object o) {
     return getClassName(o.getClass());
   }

   public static String getClassName(Class c) {
     String s = c.getName();
     return (classNameLength == FULL_CLASSNAME) ? s : s.substring(s.lastIndexOf('.')+1);
   }


   // useful when overriding the toString method of objects
   public static String getStringDescription(Object o, Object details) {
     StringBuffer sb = new StringBuffer();
     sb.append('[');
     sb.append(getClassName(o));
     sb.append('@');
     sb.append(Integer.toHexString(o.hashCode()));
     sb.append(", ");
     sb.append(details);
     sb.append(']');
     return sb.toString();
   }

}
