/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf;

import org.locomotive.obf.data.UniqueId;


/**
 * A process to get the default business object for a given class.
 *
 * @see    GetProcess
 * @see    GetMultipleProcess
 *
 * @author Erik Erskine
 */

public class GetDefaultProcess implements BusinessProcess {

  private Class busClass;


  /**
   * Creates a new process.
   */

  public GetDefaultProcess() {
  }


  /**
   * Creates a new process with the specified business object class
   * and unique identifier.
   *
   * @param      busClass the class of business object to get.  This class
   *             must implement the <code>BusinessObject</code> interface.
   * @param      uniqueId the unique identifier of the object required.
   */

  public GetDefaultProcess(Class busClass) {
    this.busClass = busClass;
  }


  /**
   * Returns the class of business object this process will get.
   *
   * @see        GetDefaultProcess#setBusClass(java.lang.Class)
   */

  public Class getBusClass() {
    return busClass;
  }


  /**
   * Sets the class of business object this process will get.
   *
   * @param      busClass the class of business object to get.
   *             This class must implement must implement the
   *             <code>BusinessObject</code> interface.
   * @see        GetDefaultProcess#getBusClass()
   */

  public void setBusClass(Class c) {
    busClass = c;
  }

}
