/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf;

import org.locomotive.obf.data.UniqueId;


/**
 * A process to get a single business object.
 *
 * @see    GetDefaultProcess
 * @see    GetMultipleProcess
 *
 * @author Erik Erskine
 */

public class GetProcess implements BusinessProcess {

  private Class busClass;
  private UniqueId uniqueId;


  /**
   * Creates a new process.
   */

  public GetProcess() {
  }


  /**
   * Creates a new process with the specified business object class
   * and unique identifier.
   *
   * @param      busClass the class of business object to get.  This class
   *             must implement the <code>UidBusinessObject</code> interface.
   * @param      uniqueId the unique identifier of the object required.
   */

  public GetProcess(Class busClass, UniqueId uniqueId) {
    this.busClass = busClass;
    this.uniqueId = uniqueId;
  }


  /**
   * Returns the class of business object this process will get.
   *
   * @see        GetProcess#setBusClass(java.lang.Class)
   */

  public Class getBusClass() {
    return busClass;
  }


  /**
   * Sets the class of business object this process will get.
   *
   * @param      busClass the class of business object to get.
   *             This class must implement must implement the
   *             <code>UidBusinessObject</code> interface.
   * @see        GetProcess#getBusClass()
   */

  public void setBusClass(Class c) {
    busClass = c;
  }


  /**
   * Returns the unique identifier this process will use when getting
   * the business object.
   *
   * @see        GetProcess#setUniqueId(org.locomotive.obf.data.UniqueId)
   */

  public UniqueId getUniqueId() {
    return uniqueId;
  }


  /**
   * Sets the unique identifier this process uses when getting
   * the business object.
   *
   * @param      busClass the class of business object to get.
   *             This class must implement must implement the
   *             <code>UidBusinessObject</code> interface.
   * @see        GetProcess#getUniqueId()
   */

  public void setUniqueId(UniqueId u) {
    uniqueId = u;
  }

}
