/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf;

/**
 * A process to insert a given business object.
 *
 * @see    UpdateProcess
 * @see    DeleteProcess
 *
 * @author Erik Erskine
 */


public class InsertProcess implements BusinessProcess {

  private UidBusinessObject businessObject;


  /**
   * Creates a new insert process without a business object.
   */

  public InsertProcess() {
  }


  /**
   * Creates a new insert process with the specifies business object
   *
   * @param      businessObject the object to be inserted
   */

  public InsertProcess(UidBusinessObject businessObject) {
    this.businessObject = businessObject;
  }


  /**
   * Returns the business object contained within this process
   *
   * @see        InsertProcess#setBusinessObject(UidBusinessObject)
   */

  public UidBusinessObject getBusinessObject() {
    return businessObject;
  }


  /**
   * Sets the business object contained within this process
   *
   * @param      businessObject the object to be inserted
   * @see        InsertProcess#getBusinessObject()
   */

  public void setBusinessObject(UidBusinessObject b) {
    businessObject = b;
  }

}
