/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf;

/**
 * Interface for business objects used to represent items in a list box.
 * <p>
 * These objects contain a unique identifier and column data.
 *
 * @author David Warnock
 * @see    ObfTableModel
 */

public interface ListBoxBusinessObject extends UidBusinessObject {

  /**
   * Returns the number of columns to be displayed in the table.
   */
  
  public int getColumnCount();
  

  /**
   * Returns an array of column names for the table.
   */
  
  public String[] getColumnNames();
  
  
  /**
   * Returns an array consisting of the default and initial sorting order
   * of table columns.  For example, {3,1,2} would indicate to the table to
   * sort by column 3, then 1, then 2.
   *
   * @see        ListBoxBusinessObject#getDefaultSortDirections
   */
  
  public int[] getDefaultSortColumns();
  

  /**
   * Returns an array of booleans indicating whether the columns sould be
   * sorted in ascending (<code>true</code>) or descending
   * (<code>false</code>) order.  The order of the array elements should
   * match that of <code>getDefaultSortColumns</code>
   *
   * @see        ListBoxBusinessObject#getDefaultSortColumns
   */
  
  public boolean[] getDefaultSortDirections();
  

  /**
   * Returns the object to be displayed in the table.
   *
   * @param      col the table column (before any rearranging)
   */
  
  public Object getColumnObject(int col);

}
