/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf;

import javax.swing.JOptionPane;
import java.awt.Component;


/**
 * General exception to indicate an error has occurred.
 *
 * @author David Warnock
 */

public class ObfException extends Exception {
  
  /**
   * Creates a new exception with no message
   */
  
  public ObfException() {
    super();
  }


  /**
   * Creates a new exception with the specified message.
   *
   * @param      msg a string description of the error
   */

  public ObfException(String msg) {
    super(msg);
  }


  /**
   * Display a swing message dialog containing the description of the error.
   *
   * @see        ObfException#showMessageDialog(java.awt.Component)
   */

  public void showMessageDialog() {
    JOptionPane.showMessageDialog(null, getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
  }


  /**
   * Display a swing message dialog containing the description of the error.
   *
   * @param      parent the component to use as this dialog's parent
   * @see        ObfException#showMessageDialog()
   */

  public void showMessageDialog(Component parent) {
    JOptionPane.showMessageDialog(parent, getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
  }




  public void showException() {
    System.out.println(getMessage());
  }

  public void showException(Component parent) {
    showMessageDialog(parent);
  }


}


