/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf;

import org.locomotive.obf.data.UniqueId;


/**
 * Interface for business objects that contain a unique identifier.
 * <p>
 * Only business objects with a unique identifier can be used for the most
 * common database functions such as insert/update/delete.  If a business
 * object represents some data in a database with a primary key that can
 * identify it, it should implement <code>UidBusinessObject</code> instead
 * of <code>BusinessObject</code>.
 * <p>
 * The identifier can be <code>null</code> if it is not known.
 *
 * @author David Warnock
 * @author Erik Erskine
 */

public interface UidBusinessObject extends BusinessObject {

  /**
   * Returns the unique identifier of this business object.
   * @see UidBusinessObject#setUniqueId(org.locomotive.obf.data.UniqueId)
   */
  
  public UniqueId getUniqueId();


  /**
   * Sets the unique identifier of this business object.
   *
   * @param      u the unique identifier
   * @see        UidBusinessObject#getUniqueId()
   */

  public void setUniqueId(UniqueId u);

}


