/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client;

import org.locomotive.obf.*;
import org.locomotive.obf.event.RefreshListener;
import org.locomotive.obf.event.RefreshManager;
import org.locomotive.obf.data.UniqueId;
import java.util.Vector;


import java.io.*;
import java.net.*;


/**
 * Provides a connection to a server using HTTP.
 * <p>
 * [PENDING: more documentation to follow]
 *
 * @author Erik Erskine
 * @see    ConnectionToLocalServer
 */

public class ConnectionToHttpServer implements ConnectionToServer {

  private RefreshListener clientListener;
  private RefreshManager refreshManager = new RefreshManager();
  private String url, clientUrl;
  private int clientPort;


  /**
   * Create a new connection to the HTTP server at <code>url</code>.
   * <p>
   * Actual HTTP connections are made whenever a process is passed to the
   * server.  The connections are not kept open.
   * <p>
   * [Pending: more documentation to follow]
   * <p>
   * <b>Implementation of notification is not complete yet</b>
   *
   * @param      clientListener the client to receive notification of changes
   *             to business objects
   * @param      url the address of the HTTP server to connect to
   * @deprecated use <code>ConnectionToHttpServer(String)</code> instead
   *             and <code>addRefreshListener</code> to specify the
   *             refresh listener for a given class.
   */

  public ConnectionToHttpServer(RefreshListener clientListener, String url, String clientUrl, int clientPort) {
    this.clientListener = clientListener;
    this.url = url;
    this.clientUrl = clientUrl;
    this.clientPort = clientPort;

    ClientHttpMonitor clientMonitor = new ClientHttpMonitor(clientPort, this);
    clientMonitor.start();

    //ClientHttpMonitor monitorConnection = new ClientHttpMonitor(mgr, url);
    //monitorConnection.start();
  }


  /**
   * Create a new connection to the HTTP server at <code>url</code>.
   * <p>
   * Actual HTTP connections are made whenever a process is passed to the
   * server.  The connections are not kept open.
   * <p>
   * [Pending: more documentation to follow]
   * <p>
   * <b>Implementation of notification is not complete yet</b>
   *
   * @param      url the address of the HTTP server to connect to
   */

  public ConnectionToHttpServer(String url, String clientUrl, int clientPort) {
    this.url = url;
    this.clientUrl = clientUrl;
    this.clientPort = clientPort;

    ClientHttpMonitor clientMonitor = new ClientHttpMonitor(clientPort, this);
    clientMonitor.start();
  }


  
  /**
   * Executes the process on the server and returns the result.
   *
   * @param      process the process to be executed on the server
   * @return     the result as returned from the server
   * @exception  ConnectionToServerException if there was a problem connecting
   *             to the server
   * @exception  ObfException if the server threw an <code>ObfException</code>
   */

  public Object doProcess(BusinessProcess p)
  throws ObfException, ConnectionToServerException {

    ObjectOutputStream out;
    ObjectInputStream in;
    Object reply;

    // establish connection
    try {
      URLConnection uc = new URL(url).openConnection();
      uc.setUseCaches(false);
      uc.setDoOutput(true);
      uc.setDoInput(true);
      uc.setRequestProperty("Content-type", "application/octet-stream");

      // put the process object on the stream
      out = new ObjectOutputStream(uc.getOutputStream());
      out.writeObject(p);

      // close stream + send request
      out.close();
      uc.connect();
      in = new ObjectInputStream(uc.getInputStream());

    } catch (MalformedURLException e) {
      throw new ConnectionToServerException(""+e);
    } catch (IOException e) {
      e.printStackTrace();
      throw new ConnectionToServerException(""+e);
    }


    // receive response from the input stream
    try {
      reply = in.readObject();

      if (reply != null) {
        //System.out.println("* received "+reply.getClass().getName()+
        //                   " from server");
      } else {
        //System.out.println("* received null from server");
      }

    } catch (ClassNotFoundException e) {
      throw new ConnectionToServerException("(client side) "+e);
    } catch (IOException e) {
      throw new ConnectionToServerException("(client side) "+e);
    }



    if (reply instanceof ObfException) {
      throw new ObfException(""+reply);
    }


    // tidy up
    try { out.close(); } catch (IOException e) {}
    try { in.close();  } catch (IOException e) {}

    return reply;

  }




  // --- Old refreshing methods ---


  /**
   * Informs the server that this connection should receive notification
   * of changes to business objects.
   * <p>
   * <b>Not implemented yet</b>
   *
   * @param      busClass the business object class to listen to.
   * @exception  ObfException if the server threw an <code>ObfException</code>
   * @deprecated Replaced by <code>addRefreshListener</code>
   */

  public void addClientRefreshListener(Class busClass)
  throws ObfException {

    AddClientListenerProcess process = new AddClientListenerProcess(clientUrl, clientPort, busClass);
    doProcess(process);

  }


  /**
   * Informs the server that this connection should no longer receive
   * notification of changes to business objects.
   * <p>
   * <b>Not implemented yet</b>
   *
   * @param      busClass the business object class to listen to.
   * @exception  ObfException if the server threw an <code>ObfException</code>
   * @deprecated Replaced by <code>removeRefreshListener</code>
   */

  public void removeClientRefreshListener(Class busClass)
  throws ObfException {

    RemoveClientListenerProcess process = new RemoveClientListenerProcess(clientUrl, clientPort, busClass);
    doProcess(process);

  }



  /**
   * Sets the client listener that will receive refresh notifications.
   * @deprecated Replaced by <code>addRefreshListener</code>
   */

  public void setClientListener(RefreshListener listener) {
    clientListener = listener;
  }





  // --- Refresher methods ---


  /**
   * Adds the specified refresh listener to be notified of
   * refreshes from this object.
   *
   * @param      listener the listener to be added
   * @param      busClass the business object class to listen to
   */

  public void addRefreshListener(RefreshListener listener, Class busClass) {
    System.out.println("client side connection: addRefreshListener(listener, "+busClass+")");
    try {

      // if this connection is not already listening to the class,
      // we nned to communicate and say this connection is listening
      if (refreshManager.isMonitoringClass(busClass) == false) {
        AddClientListenerProcess process =
          new AddClientListenerProcess(clientUrl, clientPort, busClass);
        doProcess(process);
      }

    } catch (ObfException oe) {
      // what now?
    }

    refreshManager.addRefreshListener(listener, busClass);
  }



  /**
   * Removes the specified refresh listener so that it no longer receives
   * notification of refreshes from this object.
   *
   * @param      listener the listener to be removed
   * @param      busClass the business object class to listen to
   */

  public void removeRefreshListener(RefreshListener listener, Class busClass) {
    System.out.println("client side connection: removeRefreshListener(listener, "+busClass+")");
    refreshManager.removeRefreshListener(listener, busClass);
    try {

      // if no listeners are left that are interested in this class
      // communicate and say we should stop listening
      if (refreshManager.isMonitoringClass(busClass) == false) {
        RemoveClientListenerProcess process = new RemoveClientListenerProcess(clientUrl, clientPort, busClass);
        doProcess(process);
      }

    } catch (ObfException oe) {
      // what now?
    }

  }




  // --- Handle things sent FROM the server (these are refreshes) ---

  protected void receiveProcess(BusinessProcess p) throws ObfException {

     System.out.println("Client connection has received: "+p.getClass().getName());

  }





  /**
   * Called by the local server to notify the client of an
   * updated business object
   * @depreciated Now done by <code>receiveProcess</code>
   */

  public void notifyClientOfUpdate(UidBusinessObject busObject) {
    if (clientListener != null)
      clientListener.updatedObject(busObject);
  }


  /**
   * Called by the local server to notify the client of an
   * inserted business object
   * @depreciated Now done by <code>receiveProcess</code>
   */

  public void notifyClientOfInsert(UidBusinessObject busObject) {
    if (clientListener != null)
      clientListener.insertedObject(busObject);
  }


  /**
   * Called by the local server to notify the client of an
   * deleted business object
   * @depreciated Now done by <code>receiveProcess</code>
   */

  public void notifyClientOfDelete(UidBusinessObject busObject) {
    if (clientListener != null)
      clientListener.deletedObject(busObject);
  }


  /**
   * Called by the local server to notify the client of a
   * business object class that needs refreshing
   * @depreciated Now done by <code>receiveProcess</code>
   */

  public void notifyClientOfRefresh(Class busClass) {
    if (clientListener != null)
      clientListener.refreshObjects(busClass);
  }

}
