/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client;

import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import org.locomotive.obf.event.Refresher;
import java.util.Vector;


/**
 * Interface for classes providing a client-side connection to one or
 * more servers.
 *
 * @author Erik Erskine
 * @see    ConnectionToLocalServer
 * @see    ConnectionToHttpServer
 */

public interface ConnectionToServer extends Refresher {

  /**
   * Pass the process to the server, and returns the result.
   *
   * @param      process the process to be executed on the server
   * @return     the result as returned from the server
   * @exception  ConnectionToServerException if there was a problem connecting
   *             to the server
   * @exception  ObfException if the server threw an <code>ObfException</code>
   */

  public Object doProcess(BusinessProcess process)
  throws ObfException, ConnectionToServerException;


  /**
   * Informs the server that this connection should receive notification
   * of changes to business objects.
   *
   * @param      busClass the business object class to listen to.
   * @exception  ConnectionToServerException if there was a problem
   *             connecting to the server
   * @exception  ObfException if the server threw an <code>ObfException</code>
   * @deprecated Replaced by addRefreshListener
   */
/*
  public void addClientRefreshListener(Class busClass)
  throws ObfException, ConnectionToServerException;
*/

  /**
   * Informs the server that this connection should no longer receive
   * notification of changes to business objects.
   *
   * @param      busClass the business object class to listen to.
   * @exception  ConnectionToServerException if there was a problem
   *             connecting to the server
   * @exception  ObfException if the server threw an <code>ObfException</code>
   * @deprecated Replaced by removeRefreshListener
   */
/*
  public void removeClientRefreshListener(Class busClass)
  throws ObfException, ConnectionToServerException;
*/
}
