/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client;

import org.locomotive.obf.*;
import org.locomotive.obf.event.*;
import org.locomotive.obf.client.*;
import org.locomotive.obf.client.event.*;


/**
 * DataEntryController
 * <p>
 * [pending: more documentation to follow]
 *
 * @author David Warnock
 * @author Erik Erskine
 * @see    
 */



public interface DataEntryController extends DataEntryListener,
  RefreshListener, Validation, ValidityListener, ModificationListener {


  /**
   * Return the current business object of type <code>c</code> that this
   * controller has.  This method is typically called by panels.
   * <p>
   * <b>Check:</b> what if we want two different object of the same class
   *
   * @param      c the class of business object to get
   * @exception  ClassCastException if the controller couldn't handle
   *             objects of class <code>c</code>
   */

  public BusinessObject getBusinessObject(Class c);

  public BusinessObject getBusinessObject(Class c, Object where);


  /**
   * Execute a business process and return the result.
   *
   * @param      process       the process to execute.
   * @exception  ObfException  if the process failed to execute successfully.
   */

  public Object doProcess(BusinessProcess process) throws ObfException;


  /**
   * Adds the specified listener to receive data entry mode change events
   * from this controller.
   *
   * @param      l the listener to add
   */

  public void addDataEntryModeChangeListener(DataEntryModeChangeListener l);


  /**
   * Removes the specified listener so that it no longer receives
   * data entry mode change events from this controller.
   *
   * @param      l the listener to remove
   */

  public void removeDataEntryModeChangeListener(
    DataEntryModeChangeListener l);


  /**
   * Adds the specified listener to receive data change events
   * from this controller.
   *
   * @param      l the listener to add
   */

  public void addDataChangeListener(DataChangeListener l);
  
  
  /**
   * Removes the specified listener so that it no longer receives
   * data change events from this controller.
   *
   * @param      l the listener to remove
   */

  public void removeDataChangeListener(DataChangeListener l);

}
