/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client;

import org.locomotive.obf.*;
import org.locomotive.obf.event.*;
import org.locomotive.obf.data.UniqueId;
import java.util.Vector;


/**
 * Interface for managers.  The manager is required to implement the
 * <code>doProcess()</code> method, and act as a <code>Refresher</code> to
 * notify listeners of changes to business objects.<br>
 * A manager may send processes to a server (either local or remote), but may
 * also be single-tier in which case it must deal with the processes itself.
 *
 * @author Erik Erskine
 */

public interface Manager extends Refresher {

  /**
   * Execute a business process and return the result.
   *
   * @param      process       the process to execute.
   * @exception  ObfException  if the process failed to execute successfully.
   */

  public Object doProcess(BusinessProcess process) throws ObfException;



  // added by ee - I think these will be removed from here because managers may not support
  // all of these things.  Implementations can then add whatever methods they wish

  //public Vector getBusinessObjects(Class busClass) throws ObfException;
  //public Vector getBusinessObjects(Class busClass, int iMaxRows) throws ObfException;
  //public Vector getBusinessObjects(Class busClass, Object whereObj) throws ObfException;
  //public Vector getBusinessObjects(Class busClass, Object whereObj, int iMaxRows) throws ObfException;
  //public Vector getBusinessObjects(Class busClass, Object whereObj, Object orderByObj) throws ObfException;
  //public Vector getBusinessObjects(Class busClass, Object whereObj, Object orderByObj, int iMaxRows) throws ObfException;
  //public UidBusinessObject getBusinessObject(Class busClass, UniqueId whereObj) throws ObfException;
  //public BusinessObject getDefaultBusinessObject(Class busClass) throws ObfException;
  //public UniqueId insertBusinessObject(UidBusinessObject busObj) throws ObfException;
  //public void updateBusinessObject(UidBusinessObject busObj) throws ObfException;
  //public void deleteBusinessObject(UidBusinessObject busObj) throws ObfException;


}
