/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.event;


import java.util.Vector;


public class DataEntryEventManager {

   private Vector listeners = new Vector();

   public DataEntryEventManager() {
      super();
   }

   public DataEntryEventManager(DataEntryListener listener) {
      super();
      listeners.addElement(listener);
   }


   public synchronized void addListener(DataEntryListener listener) {
     listeners.addElement(listener);
   }
 
   public synchronized void removeListener(DataEntryListener listener) {
     listeners.removeElement(listener);
   }
 
   public void newData(Object source) {
     DataEntryNewEvent e = new DataEntryNewEvent(source);
     Vector localList = new Vector();
      synchronized (this) { 
        localList = (Vector) listeners.clone();
     }
 
     for (int value = 0; value < localList.size(); value++) {
       ((DataEntryListener)localList.elementAt(value)).newData(e);
     }
   }

   public void editData(Object source) {
     DataEntryEditEvent e = new DataEntryEditEvent(source);
     Vector localList = new Vector();
      synchronized (this) { 
        localList = (Vector) listeners.clone();
     }
 
     for (int value = 0; value < localList.size(); value++) {
       ((DataEntryListener)localList.elementAt(value)).editData(e);
     }
   }

   public void deleteData(Object source) {
     DataEntryDeleteEvent e = new DataEntryDeleteEvent(source);
     Vector localList = new Vector();
      synchronized (this) { 
        localList = (Vector) listeners.clone();
     }
 
     for (int value = 0; value < localList.size(); value++) {
       ((DataEntryListener)localList.elementAt(value)).deleteData(e);
     }
   }

   public void saveData(Object source) {
     DataEntrySaveEvent e = new DataEntrySaveEvent(source);
     Vector localList = new Vector();
      synchronized (this) { 
        localList = (Vector) listeners.clone();
     }
 
     for (int value = 0; value < localList.size(); value++) {
       ((DataEntryListener)localList.elementAt(value)).saveData(e);
     }
   }

   public void cancelData(Object source) {
     DataEntryCancelEvent e = new DataEntryCancelEvent(source);
     Vector localList = new Vector();
      synchronized (this) { 
        localList = (Vector) listeners.clone();
     }
 
     for (int value = 0; value < localList.size(); value++) {
       ((DataEntryListener)localList.elementAt(value)).cancelData(e);
     }
   }
}
