/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;

import org.locomotive.obf.data.*;
import org.locomotive.obf.event.*;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;



public abstract class AbstractObfTextField extends javax.swing.JTextField implements EditableField, FocusListener, ValidityListener {

  private Vector ValidityChangedListeners = new Vector();

  private Border normalBorder;
  private boolean valid = true;


  public AbstractObfTextField(int columns) {
    super(columns);
    getKeymap().removeKeyStrokeBinding(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
    normalBorder = getBorder();
    addFocusListener(this);
  }




  public EditableDataObject getDataObject() {
    return (EditableDataObject) getDocument();
  }


  public void setDataObject(EditableDataObject data) {
    if (data == null) {
      // do nothing!

    } else if (data.getClass() == getDataObjectClass()) {
      setDocument((ObfDocument) data);
      doBorder(data.isValid());
      data.addValidityListener(this);

    } else if (data != null)
      throw new IllegalArgumentException("setData method in "+getClass().getName()+" was passed "+data.getClass().getName()+", expected "+getDataObjectClass().getName());
  }





  public void validityChanged(ValidityEvent ve) {
    if (ve.getSource() == getDataObject())
      doBorder(ve.getValidity());
  }


  public void setEnabled(boolean b) {
    // don't draw a red border if the field is disabled, even if it is not valid
    super.setEnabled(b);
    EditableDataObject doc = (EditableDataObject) getDocument();
    if (b && doc != null)
      doBorder(doc.isValid());
    else
      doBorder(true);
  }

  public void doBorder(boolean good) {
    if (good || !isEnabled())
      setBorder(normalBorder);
    else
      setBorder(new LineBorder(Color.red, 1));
  }




  public void focusGained(FocusEvent fe) {
    selectAll();
  }

  public void focusLost(FocusEvent fe) {
    ObfDocument doc = (ObfDocument) getDocument();
    if (doc != null)
      doc.updateText();
  }

}
