/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;

import org.locomotive.obf.data.*;

/**
 * Label to display <code>BooleanModel</code> objects
 */


public class BooleanLabel extends AbstractObfLabel {

  private static String defaultTrueCaption  = "Yes";
  private static String defaultFalseCaption = "No";

  protected String trueCaption, falseCaption;


  public BooleanLabel() {
    this(defaultTrueCaption, defaultFalseCaption, null);
  }

  public BooleanLabel(BooleanModel model) {
    this(defaultTrueCaption, defaultFalseCaption, model);
  }

  public BooleanLabel(String tc, String fc) {
    this(tc, fc, null);
  }

  public BooleanLabel(String tc, String fc, BooleanModel model) {
    super(Math.max(tc.length(), fc.length()), LEFT);
    trueCaption = tc;
    falseCaption = fc;
    setDataObject(model);
  }


  public Class getDataObjectClass() {
    return BooleanModel.class;
  }

  protected void displayData() {
    if (data == null || data.getData() == null) {
      setText(" ");
    } else {
      boolean b = ((Boolean) data.getData()).booleanValue();
      setText(b ? trueCaption : falseCaption);
    }
  }

}
