/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;


import org.locomotive.obf.data.*;
import org.locomotive.obf.event.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;



/**
 * Provides two mutually exculsive radio buttons to select a boolean value.
 *
 * @see org.locomotive.data.BooleanModel 
 * @see org.locomotive.client.swing.BooleanLabel 
 *
 */


public class BooleanRadioField extends JPanel implements EditableField, ModificationListener, ActionListener {


  private static String defaultTrueCaption  = "Yes";
  private static String defaultFalseCaption = "No";

  private String trueCaption, falseCaption;
  private JRadioButton trueRadio, falseRadio;
  private ButtonGroup group = new ButtonGroup();
  private BooleanModel model = null;


  public BooleanRadioField() {
    this(defaultTrueCaption, defaultFalseCaption, false, false);
  }

  public BooleanRadioField(boolean vertical) {
    this(defaultTrueCaption, defaultFalseCaption, vertical, false);
  }

  public BooleanRadioField(boolean vertical, boolean falseFirst) {
    this(defaultTrueCaption, defaultFalseCaption, vertical, falseFirst);
  }

  public BooleanRadioField(String tc, String fc) {
    this(tc, fc, false, false);
  }

  public BooleanRadioField(String tc, String fc, boolean vertical) {
    this(tc, fc, vertical, false);
  }

  public BooleanRadioField(String tc, String fc, boolean vertical, boolean falseFirst) {
    trueCaption = tc;
    falseCaption = fc;
    group.add(trueRadio = createRadioButton(trueCaption));
    group.add(falseRadio = createRadioButton(falseCaption));
    trueRadio.addActionListener(this);
    falseRadio.addActionListener(this);

    if (vertical)
      setLayout(new GridLayout(0,1));

    // add the radio buttons to the panel
    if (falseFirst) {
      add(falseRadio);
      add(trueRadio);
    } else {
      add(trueRadio);
      add(falseRadio);
    }
    
  }




  /**
   * Wrapper to simply create and return a new <code>JRadioButton</code>.
   * <br>Override this if you want to use a subclass of <code>JRadioButton</code> instead
   */

  protected JRadioButton createRadioButton(String caption) {
    return new JRadioButton(caption);
  }


  public Class getDataObjectClass() {
    return BooleanModel.class;
  }


  public EditableDataObject getDataObject() {
    return model;
  }
  

  public void setDataObject(EditableDataObject o) {
    if (o == null || o instanceof BooleanModel) {
      
      if (model != null)
        model.removeModificationListener(this);
      model = (BooleanModel) o;
      if (model != null)
        model.addModificationListener(this);

    } else
      throw new IllegalArgumentException("setDataObject method in EnumRadioField was passed a "+o.getClass().getName()+", expected an BooleanModel");

    updateRadios();
  }



  public void modified(ModificationEvent me) {
    updateRadios();
  }

  
  public void updateRadios() {
    if (model != null) {
      boolean v = ((Boolean) model.getData()).booleanValue();

      if (v)
        trueRadio.setSelected(true);
      else
        falseRadio.setSelected(true);
    }
  }


  public void actionPerformed(ActionEvent ae) {
    if (model != null)
      model.setData(new Boolean(ae.getSource() == trueRadio));
  }

  

}
