/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;


import org.locomotive.obf.client.*;
import org.locomotive.obf.client.event.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;



public class DefaultButtonBar extends JPanel implements ActionListener, DataEntryModeChangeListener {
  
  private DataEntryController controller = null;
  //private DataEntryEventManager evtMgr = new DataEntryEventManager();


  private JButton newButton    = new JButton("New");
  private JButton editButton   = new JButton("Edit");
  private JButton deleteButton = new JButton("Delete");
  private JButton saveButton   = new JButton("Save");
  private JButton cancelButton = new JButton("Cancel");


  
  public DefaultButtonBar(DataEntryController c) {
    controller = c;
    controller.addDataEntryModeChangeListener(this);
    //evtMgr.addListener(controller);

    newButton.addActionListener(this);
    editButton.addActionListener(this);
    deleteButton.addActionListener(this);
    saveButton.addActionListener(this);
    cancelButton.addActionListener(this);

    setLayout(new GridLayout(0,1));
    add(newButton);
    add(editButton);
    add(deleteButton);
    add(saveButton);
    add(cancelButton);
  }



  public void actionPerformed(ActionEvent ae) {
    JButton b = (JButton) ae.getSource();

    if      (b == newButton)    controller.newData(new DataEntryNewEvent(this));
    else if (b == editButton)   controller.editData(new DataEntryEditEvent(this));
    else if (b == deleteButton) deleteClick();
    else if (b == saveButton)   controller.saveData(new DataEntrySaveEvent(this));
    else if (b == cancelButton) controller.cancelData(new DataEntryCancelEvent(this));
  }

  private void deleteClick() {
    if (JOptionPane.showConfirmDialog(this, "Are you sure?", "Confirm Delete", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
      controller.deleteData(new DataEntryDeleteEvent(this));
  }



  // button bar listens to controller for mode changes, in order to enable/disable buttons
  public void dataEntryModeChange(DataEntryModeChangeEvent e, byte oldMode, byte newMode) {
    if (e.getSource() == controller) {
      boolean browsing = (newMode == DataEntryModeChangeEvent.BROWSE);
      newButton.setEnabled(browsing);
      editButton.setEnabled(browsing);
      deleteButton.setEnabled(browsing);
      saveButton.setEnabled(!browsing);
      cancelButton.setEnabled(!browsing);
    }
  }

}

