/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;


import org.locomotive.obf.data.*;
import org.locomotive.obf.event.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;



/**
 * Provides a series of radio buttons to select values in an enumeration.
 * <br>The radio buttons are mutually exclusive
 *
 * @see org.locomotive.data.EnumModel 
 * @see org.locomotive.client.swing.EnumLabel 
 *
 */


public class EnumRadioField extends JPanel implements EditableField, ModificationListener, ActionListener {

  private String[] captions = null;
  private JRadioButton[] radio;
  private ButtonGroup group = new ButtonGroup();
  private EnumModel model = null;



  public EnumRadioField(String[] c) {
    this(c, false, false);
  }

  public EnumRadioField(String[] c, boolean vertical) {
    this(c, vertical, false);
  }

  public EnumRadioField(String[] c, boolean vertical, boolean reversed) {
    if (c.length < 1)
      throw new IllegalArgumentException("EnumRadioField must be created with at least one caption");

    if (vertical)
      setLayout(new GridLayout(0,1));

    captions = c;
    radio = new JRadioButton[captions.length];

    // add the radio buttons to the panel
    if (reversed) {
      for (int x=captions.length-1; x>=0; x--) {
        radio[x] = createRadioButton(captions[x]);
        add(radio[x]);
        radio[x].addActionListener(this);
      }
    } else {
      for (int x=0; x<captions.length; x++) {
        radio[x] = createRadioButton(captions[x]);
        add(radio[x]);
        radio[x].addActionListener(this);
      }
    }

    // put all the radio buttons in a button group so they are mutually exclusive
    for (int x=0; x<captions.length; x++)
      group.add(radio[x]);
    
  }



  /**
   * Wrapper to simply create and return a new <code>JRadioButton</code>.
   * <br>Override this if you want to use a subclass of <code>JRadioButton</code> instead
   */

  protected JRadioButton createRadioButton(String caption) {
    return new JRadioButton(caption);
  }


  public Class getDataObjectClass() {
    return EnumModel.class;
  }


  public EditableDataObject getDataObject() {
    return model;
  }
  

  public void setDataObject(EditableDataObject o) {
    if (o == null || o instanceof EnumModel) {
      
      if (model != null)
        model.removeModificationListener(this);
      model = (EnumModel) o;
      if (model != null)
        model.addModificationListener(this);

    } else
      throw new IllegalArgumentException("setDataObject method in EnumRadioField was passed a "+o.getClass().getName()+", expected an EnumModel");

    updateRadios();
  }



  public void modified(ModificationEvent me) {
    updateRadios();
  }

  protected void updateRadios() {
    if (model != null) {
      int index = ((Integer) model.getData()).intValue();

      radio[index].setSelected(true);   // setSelected does't cause an actionPerformed event so we don't loop  :-)
    }
  }


  public void actionPerformed(ActionEvent ae) {
    if (model != null) {
      for (int x=0; x<radio.length; x++)
        if (ae.getSource() == radio[x])
          model.setData(new Integer(x));
    }
  }



}
