/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;

import org.locomotive.obf.data.*;
import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.util.Locale;

/**
 * Label to display <code>IntegerDocument</code> objects
 */


public class IntegerLabel extends AbstractObfLabel {

  protected NumberFormat numberFormat;

  public IntegerLabel() {
    this(null, 10);
  }

  public IntegerLabel(int width) {
    this(null, width);
  }

  public IntegerLabel(IntegerDocument doc) {
    this(doc, 10);
  }
    
  public IntegerLabel(IntegerDocument doc, int width) {
    super(width, LEFT);
    numberFormat = DecimalFormat.getInstance(Locale.getDefault());
    setDataObject(doc);
  }


  public Class getDataObjectClass() {
    return IntegerDocument.class;
  }

  protected void displayData() {
    if (data == null || data.getData() == null)
      setText("");
    else
      setText(numberFormat.format(((Integer) data.getData()).intValue()));
  }

}
