/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.client.swing;


import org.locomotive.obf.*;
import org.locomotive.obf.client.*;
import org.locomotive.obf.event.*;
import org.locomotive.obf.data.UniqueId;


import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;



public class ObfTableModel extends AbstractTableModel implements RefreshListener {


  public void printModel() {
    System.out.println("---------------------------------------");
    for (int x=0; x<getRowCount(); x++)
      System.out.println(x+" "+getBusinessObjectAt(x));
    System.out.println("---------------------------------------");
  }


  protected Vector data;  // vector of business objects
  protected Manager mgr;
  protected Class busClass;
  protected int columnCount = 0;
  protected int limit = -1;
  protected Object whereObject = null;

  protected int[]     defaultSortColumns;
  protected boolean[] defaultSortDirections;
  protected String[]  columnNames;



  public ObfTableModel(ListBoxBusinessObject o) {
    this(o, new Vector());
  }

  public ObfTableModel(ListBoxBusinessObject o, Vector v) {
    super();
    init(o);
    setData(v);
  }



  public ObfTableModel(Manager m, ListBoxBusinessObject o) {
    this(m, o, true, true);
  }

  public ObfTableModel(Manager m, ListBoxBusinessObject o, boolean autoLoad) {
    this(m, o, autoLoad, true);
  }

  public ObfTableModel(Manager m, ListBoxBusinessObject o, boolean autoLoad, boolean autoUpdate) {
    super();
    init(o);

    mgr = m;
    if (autoLoad)
      setData(loadData());

    if (autoUpdate)
      mgr.addRefreshListener(this, busClass);
  }


  private void init(ListBoxBusinessObject o) {
    busClass              = o.getClass();
    columnCount           = o.getColumnCount();
    columnNames           = o.getColumnNames();
    defaultSortColumns    = o.getDefaultSortColumns();
    defaultSortDirections = o.getDefaultSortDirections();
  }


  private Vector loadData() {
    Vector v;
    try {

      GetMultipleProcess p = new GetMultipleProcess(busClass, whereObject, null, limit);

      v = (Vector) mgr.doProcess(p);

      /*
      if (whereObject == null)
        v = mgr.getBusinessObjects(busClass);
      else
        v = mgr.getBusinessObjects(busClass, whereObject);
      */
      return v;
      
    } catch (ObfException e) {
      return null;
    }
  }


  public void setData(Vector v) {
    data = (v != null) ? v : new Vector();
    fireTableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
    //printModel();
  }




  public void setWhereObject(Object w) {
    whereObject = w;
  }

  public Object getWhereObject() {
    return whereObject;
  }


  public int getLimit() {
    return limit;
  }

  public void setLimit(int l) {
    limit = l;
  }



  public String getColumnName(int col) {
    return columnNames[col];
  }

  public int getColumnCount() {
    return columnCount;
  }

  public int getRowCount() {
    if (data == null) {
      return 0;
    } else {
      return data.size();
    }
  }

  public Object getValueAt(int row, int col) {
    ListBoxBusinessObject o = getBusinessObjectAt(row);
    return o.getColumnObject(col);
  }

  public ListBoxBusinessObject getBusinessObject(UniqueId uid) {
    int row = findUniqueId(uid);

    if (row > -1)
      return getBusinessObjectAt(row);
    else
      return null;
  }


  public ListBoxBusinessObject getBusinessObjectAt(int row) {
    return (ListBoxBusinessObject) data.elementAt(row);
  }

  public UniqueId getUniqueIdAt(int row) {
    ListBoxBusinessObject o = getBusinessObjectAt(row);
    return o.getUniqueId();
  }

  public Class getColumnClass(int c) {
    if (getValueAt(0, c) == null)
      return String.class;
    else
      return getValueAt(0, c).getClass();
  }

  public int[] getDefaultSortColumns() {
    return defaultSortColumns;
  }

  public boolean[] getDefaultSortDirections() {
    return defaultSortDirections;
  }



  public int findObject(ListBoxBusinessObject findObj) {
    if (findObj == null)
      return -1;

    UniqueId findId = findObj.getUniqueId();
    if (findId == null)
      return -1;

    ListBoxBusinessObject checkObj;
    UniqueId checkId;

    int i = 0;
    int row = -1;
    while (i < getRowCount() && row == -1) {
      checkObj = (ListBoxBusinessObject) data.elementAt(i);
      checkId  = checkObj.getUniqueId();
      if (checkId.equals(findId))
        row = i;
      i++;
    }
    return row;
  }



  public int findUniqueId(UniqueId findId) {
    if (findId == null)
      return -1;

    ListBoxBusinessObject checkObj;
    UniqueId checkId;

    int i = 0;
    int row = -1;
    while (i < getRowCount() && row == -1) {
      checkObj = (ListBoxBusinessObject) data.elementAt(i);
      checkId  = checkObj.getUniqueId();
      if (checkId.equals(findId))
        row = i;
      i++;
    }
    return row;
  }





  // --- changes to the data

  public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
    Debug.abort("setValueAt() called, not allowed");
  }



  public void insertedObject(UidBusinessObject busObject) {
    if (busObject.getClass() == busClass) {

      if (data != null && findObject((ListBoxBusinessObject) busObject) == -1) {
        Debug.msg(getClass(), "inserting "+busObject);
        data.addElement(busObject);
        fireTableRowsInserted(data.size()-1, data.size()-1);
      } else
        Debug.msg(getClass(), "not inserting "+busObject+" - already in model");

      //printModel();
    }
  }


  public void updatedObject(UidBusinessObject busObject) {
    if (busObject.getClass() == busClass) {

      int row = findObject((ListBoxBusinessObject) busObject);
      if (row > -1) {
        Debug.msg(getClass(), "updating "+busObject+" - row "+row);
        data.setElementAt(busObject, row);
        fireTableRowsUpdated(row, row);
      } else
        Debug.msg(getClass(), "not updating "+busObject+" - not in model");

      //printModel();
    }
  }


  public void deletedObject(UidBusinessObject busObject) {
    if (busObject.getClass() == busClass) {

      int row = findObject((ListBoxBusinessObject) busObject);
      if (row > -1) {
        Debug.msg(getClass(), "deleting "+busObject+" - row "+row);
        data.removeElementAt(row);
        fireTableRowsDeleted(row, row);
      } else
        Debug.msg(getClass(), "not deleting "+busObject+" - not in model");

      //printModel();
    }
  }


  public void refreshObjects(Class objClass) {
    if (objClass == busClass) {
      Debug.msg(getClass(), "refreshObjects "+objClass.getName());
      setData(loadData());
    }
  }


  public void reload() {
    setData(loadData());
  }

}

