/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.data;


/**
 * An implementation of <code>EditableDataObject</code> for booleans.
 * <p>
 * Note that the <code>Boolean</code> wrapper object is used, rather than
 * the primitive type <code>boolean</code> - this is because the interface
 * requires that the data contained must be an object.
 *
 * @author Erik Erskine
 */

public class BooleanModel extends AbstractEditableDataObject {

  /**
   * Creates a new model with both original and current values <code>b</code>.
   *
   * @param      b the original value, to be converted to a
   *             <code>Boolean</code> object
   */

  public BooleanModel(boolean b) {
    this(new Boolean(b));
  }


  /**
   * Creates a new model with both original and current values <code>b</code>.
   */

  public BooleanModel(Boolean b) {
    super(b);
    setNullAllowed(false);
  }


  /**
   * Always returns <code>Boolean.class</code>.
   */

  public Class getDataClass() {
    return Boolean.class;
  }
  

  public boolean getBoolean() {
    return ((Boolean) getData()).booleanValue();
  }

  public boolean getOriginalBoolean() {
    return ((Boolean) getOriginalData()).booleanValue();
  }

  public void setBoolean(boolean b) {
    setData(b ? Boolean.TRUE : Boolean.FALSE);
  }


  // --- Validation ---


  /**
   * Always returns <code>true</code>.
   */

  public boolean isValid() {
    return true;
  }



  // --- Modification ---

  /**
   * Returns <code>true</code> if the original and current values
   * are the same.
   */

  public boolean isModified() {
    return ((Boolean) value).booleanValue() !=
      ((Boolean) originalValue).booleanValue();
  }


}

