/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.data;


/**
 * An interface for classes that are used to store data that is contained
 * within a business object.  <code>DataObject</code> classes act as a wrapper
 * around a base object, such as a Date.  The base object can be
 * <code>null</code>.
 * <p>
 * This interface is useful for defining objects that are used in conjunction
 * with the GUI classes in the <code>org.locomotive.obf.client.swing</code>
 * package.  It is not necessary for business objects to only contain objects
 * that implement this interface if you do not use these GUI objects.
 * <p>
 * The <code>EditableDataObject</code> interface extends this
 * and is used to contain data that can change.
 *
 * @author Erik Erskine
 * @see    EditableDataObject
 */

public interface DataObject extends java.io.Serializable {

  /**
   * Returns the class of the contained base object.
   */
  
  public Class getDataClass();


  /**
   * Returns the contained base object.
   *
   * @see       DataObject#setData(java.lang.Object);
   */

  public Object getData();


  /**
   * Sets the base object.
   *
   * @exception ClassCastException if the data did not match the allowed type
   *            as returned by <code>getDataClass()</code>
   * @see       DataObject#getData();
   * @see       DataObject#getDataClass();
   */

  public void setData(Object data) throws ClassCastException;

}


