/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.data;


/**
 * An implementation of <code>EditableDataObject</code> for integers
 * used within the context of an enumeration.
 * <p>
 * Note that the <code>Integer</code> wrapper object is used, rather than
 * the primitive type <code>int</code> - this is because the interface
 * requires that the data contained must be an object.
 *
 * @author Erik Erskine
 */


public class EnumModel extends AbstractEditableDataObject {


  /**
   * Creates a new model with both original and current values <code>i</code>.
   *
   * @param      i the original value, to be converted to an
   *             <code>Integer</code> object
   */

  public EnumModel(int i) {
    this(new Integer(i));
  }

  
  /**
   * Creates a new model with both original and current values <code>i</code>.
   */
  
  public EnumModel(Integer i) {
    super(i);
    setNullAllowed(false);
  }


  /**
   * Always returns <code>Integer.class</code>.
   */
  
  public Class getDataClass() {
    return Integer.class;
  }
  

  public int getInt() {
    return ((Integer) getData()).intValue();
  }

  public int getOriginalInt() {
    return ((Integer) getOriginalData()).intValue();
  }

  public void setBigDecimal(int i) {
    setData(new Integer(i));
  }


  // --- Validation ---

  /**
   * Always returns <code>true</code>.
   */

  public boolean isValid() {
    return true;
  }



  // --- Modification ---


  /**
   * Returns <code>true</code> if the original and current values
   * are the same.
   */

  public boolean isModified() {
    return ((Integer) value).intValue() !=
      ((Integer) originalValue).intValue();
  }


}


