/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.data;


/**
 * Interface for unique ids used within a business object.<br>
 *
 * Provides functionality to compare two ids for order using the
 * <code>CompareTo</code> method.  Note: Future versions will implement
 * the <code>Comparable</code> interface in Java 1.2.<br>
 *
 * Note: subclasses must override the <code>equals()</code> method so that
 * for two distinct objects <code>x</code> and <code>y</code> representing
 * the same identifier, <code>x.equals(y) == true</code> holds.
 *
 * @author Erik Erskine
 * @author David Warnock
 * @see org.locomotive.obf.UniqueStringId
 */

public interface UniqueId extends java.io.Serializable {

  /**
   * Compares this object with the specified object for order.<br>
   *
   * @return    <code>0</code> if this object represents the same unique
   *            identifier as the argument; negative if this object is ordered
   *            before the argument; positive if this object is ordered after
   *            the argument.
   * @param     o the object to compare with this one
   * @exception ClassCastException if the argument is of a different type to
   *            this object.
   */

  public int compareTo(Object o);

}


