/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.data;


/**
 * An implementation of <code>EditableDataObject</code> containing a unique
 * identifier.  The identifier can be accessed using the
 * <code>getDataObject</code> and <code>setDataObject</code> methods.
 *
 * @see    AbstractEditableDataObject#getDataObject()
 * @see    AbstractEditableDataObject#setDataObject(java.lang.Object)
 * @author Erik Erskine
 */


public class UniqueIdModel extends AbstractEditableDataObject {

  /**
   * Creates a new model with the original value <code>null</code>.
   */

  public UniqueIdModel() {
    super();
  }


  /**
   * Creates a new model with the original value <code>u</code>.
   */

  public UniqueIdModel(UniqueId u) {
    super(u);
  }


  /**
   * Returns <code>UniqueId.class</code> - the type of any object contained
   * within this <code>UniqueIdModel</code>.
   */

  public Class getDataClass() {
    return UniqueId.class;
  }
  

  public UniqueId getUniqueId() {
    return (UniqueId) getData();
  }

  public UniqueId getOriginalUniqueId() {
    return (UniqueId) getOriginalData();
  }

  public void setUniqueId(UniqueId u) {
    setData(u);
  }


  // --- Validation ---
 
  
  /**
   * Tests whether the unique identifier contained is valid.
   * <p>
   * This only extends to checking whether a uid is <code>null</code> if
   * <code>null</code> is not allowed.  Any other value is treated as valid.
   * It does not (and is not able to) check that the uid is valid in any
   * other context, eg as a foreign key in the database.
   */

  public boolean isValid() {
    return value != null || allowNull;
  }


  // --- Modification ---


  /**
   * Returns <code>true</code> if the original and current values are equal.
   * Note that the UniqueId class used must correctly implement the
   * <code>equals()</code> method.
   *
   * @see   UniqueId#equals(java.lang.Object)
   */

  public boolean isModified() {
    if (value == null)
      return originalValue != null;
    else
      return (originalValue == null ||
              ((UniqueId)value).equals((UniqueId) originalValue) == false);
  }


}


