/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.event;


/**
 * An interface for classes that support modification.<p>
 * Classes implementing <code>Modifiable</code> can contain a
 * <code>ModificationManager</code> object, which provides
 * implementation for adding and removing listeners.
 *
 * @author Erik Erskine
 * @see    ModificationEvent 
 * @see    ModificationManager 
 * @see    ModificationListener 
 *
 */


public interface Modifiable {

  /**
   * Tests if this object has been modified.
   */
  
  public boolean isModified();


  /**
   * Adds the specified modification listener to receive modification events
   * from this object.
   *
   * @param      ml the listener to be added
   * @see        #removeModificationListener
   */
  
  public void addModificationListener(ModificationListener ml);


  /**
   * Removes the specified modification listener so that it no longer receives
   * modification events from this object.
   *
   * @param      ml the listener to be removed
   * @see        #addModificationListener
   */

  public void removeModificationListener(ModificationListener ml);
  
}


