/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.event;

import org.locomotive.obf.UidBusinessObject;


/**
 * The listener interface for receiving notification of refreshes.
 *
 * @see    Refresher 
 * @see    RefreshManager 
 *
 * @author Erik Erskine
 */

public interface RefreshListener {

  /**
   * Invoked whenever this object receives notification of an
   * updated business object.
   *
   * @param      busObject the updated business object
   */

  public void updatedObject(UidBusinessObject busObject);


  /**
   * Invoked whenever this object receives notification of an
   * inserted business object.
   *
   * @param      busObject the inserted business object
   */

  public void insertedObject(UidBusinessObject busObject);


  /**
   * Invoked whenever this object receives notification of an
   * deleted business object.
   *
   * @param      busObject the deleted business object
   */

  public void deletedObject(UidBusinessObject busObject);


  /**
   * Invoked whenever this object receives notification of a business object
   * class that needs to be refreshed.
   *
   * @param      busClass the business object class that needs refreshing
   */

  public void refreshObjects(Class busClass);

}
