/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.event;


/**
 * An interface for classes that can support refreshing.  Objects
 * implementing <code>Refresher</code> must be able to notify listeners of
 * inserted, updated and deleted objects, as well as changes to a business
 * object class that require complete reloading of all the data.
 * <p>
 * [PENDING: mode javadoc to follow]
 *
 * @see org.locomotive.obf.event.RefreshListener 
 * @see org.locomotive.obf.event.RefreshManager 
 *
 * @author Erik Erskine
 */

public interface Refresher {

  /**
   * Adds the specified refresh listener to be notified of
   * refreshes from this object.
   *
   * @param      listener the listener to be added
   * @param      busClass the business object class to listen to
   */

  public void addRefreshListener(RefreshListener listener, Class busClass);


  /**
   * Removes the specified refresh listener so that it no longer receives
   * notification of refreshes from this object.
   *
   * @param      listener the listener to be removed
   * @param      busClass the business object class to listen to
   */

  public void removeRefreshListener(RefreshListener listener, Class busClass);

}


