/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.event;


/**
 * An interface for classes that support validation,
 * ie. have a valid/invalid state.<p>
 * Classes implementing <code>Validation</code> can contain a
 * <code>ValidityManager</code> object, which provides
 * implementation for adding and removing listeners.
 *
 * @author Erik Erskine
 * @see    ValidityEvent 
 * @see    ValidityManager 
 * @see    ValidityListener 
 *
 */

public interface Validation {


  /**
   * Tests if this object is valid.
   */
  
  public boolean isValid();
  
  
  /**
   * Adds the specified validity listener to receive validity events
   * from this object.
   *
   * @param      vl the listener to be added
   * @see        #removeValidityListener
   */

  public void addValidityListener(ValidityListener vl);


  /**
   * Removes the specified validity listener so that it no longer receives
   * validity events from this object.
   *
   * @param      vl the listener to be removed
   * @see        #addValidityListener
   */

  public void removeValidityListener(ValidityListener vl);
  
}


