/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.event;


/**
 * Event to indicate that the validity of an object has changed
 *
 * @see org.locomotive.obf.event.Validation 
 * @see org.locomotive.obf.event.ValidityManager 
 * @see org.locomotive.obf.event.ValidityListener 
 *
 * @author Erik Erskine
 */

public class ValidityEvent extends java.util.EventObject {

  private boolean validity;


  /**
   * Creates a new <code>ValidityEvent</code> with the specified
   * source and validity.
   * 
   * @param      source the source object
   * @param      validity the validity of the source object
   */

  public ValidityEvent(Validation source, boolean validity) {
    super(source);
    this.validity = validity;
  }


  /**
   * Returns the validity of the source object of this event.
   */

  public boolean getValidity() {
    return validity;
  }


  /**
   * Returns a string representation of this object.
   */

  public String toString() {
    return getClass().getName() + "[source="+source+",validity="+validity+"]";
  }
  
}


