/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.example;


import org.locomotive.obf.*;
import org.locomotive.obf.data.*;
import org.locomotive.obf.client.*;
import org.locomotive.obf.client.event.*;
import org.locomotive.obf.client.swing.*;



public class ExampleController extends AbstractDataEntryController {

  private ExampleRwBusinessObject current = null;
  private ExampleForm form;


  public ExampleController(ObfManager mgr) {
    setManager(mgr);
  }


  public ExampleForm getForm() {
    if (form == null) {
      form = new ExampleForm(this);
      fireDataEntryModeChangeEvent(DataEntryModeChangeEvent.NEW, DataEntryModeChangeEvent.BROWSE);
    }
    return form;
  }



  public BusinessObject getBusinessObject(Class c) {
    
    UniqueId uid = (UniqueId) getUidModel().getData();

    try {

      if (c == ExampleRwBusinessObject.class) {


        if (uid == null) {
          // do nothing if there is no uid in the model
        
        } else if (current == null || !uid.equals(current.getUniqueId())) {
          // if the current business object is null or has a different uid to the model, load the new one

          //---
          GetProcess p = new GetProcess(c, uid);
          current = (ExampleRwBusinessObject) getManager().doProcess(p);
          //---
          //current = (ExampleRwBusinessObject) getManager().getBusinessObject(c, uid);
          //---

        }


        if (current == null) {
          // if current is null at this point load the default (new clicked)
          //---
          GetDefaultProcess p = new GetDefaultProcess(c);
          current = (ExampleRwBusinessObject) getManager().doProcess(p);
          //---
          //current = (ExampleRwBusinessObject) getManager().getDefaultBusinessObject(c);
          //---
        }

        return current;

      }

    } catch (ObfException e) {
      e.showMessageDialog(form);
    }

    return null;
  }


  
  protected void setBusinessObjectsToNull() {
    current = null;
  }


  protected void insert() throws ObfException {
    try {
      InsertProcess p = new InsertProcess(current);
      getManager().doProcess(p);
    } catch (ObfException e) {
      e.showMessageDialog(form);
      throw e;
    }
  }

  protected void update() throws ObfException {
    try {
      UpdateProcess p = new UpdateProcess(current);
      getManager().doProcess(p);
    } catch (ObfException e) {
      e.showMessageDialog(form);
      throw e;
    }
  }

  protected void delete() throws ObfException {
    try {
      DeleteProcess p = new DeleteProcess(current);
      getManager().doProcess(p);
    } catch (ObfException e) {
      e.showMessageDialog(form);
      throw e;
    }
  }

  /*
  protected void beforeNew() {
    //list.selectNone();     // what here?
  }


  protected void afterInsertTransaction()  {
    Debug.msg(getClass(), "afterInsertTransaction");
    updateTableAfterInsert();
  }
  */




  public boolean isValid() {
    return current != null && current.isValid();
  }


}