/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.example;


import org.locomotive.obf.*;
import org.locomotive.obf.client.*;
import org.locomotive.obf.client.swing.*;

import javax.swing.*;
import java.awt.*;


public class ExampleForm extends DataEntryForm { 

  private JPanel card;
  private ExampleController controller;

  private ObfTable            table;
  private ExampleBrowsePanel  browsePanel;
  private ExampleEditPanel    editPanel;
  private DefaultButtonBar buttons;
  


  public ExampleForm(ExampleController c) {
    super(c);

    controller = c;
    browsePanel = new ExampleBrowsePanel(controller);
    editPanel = new ExampleEditPanel(controller);
    buttons = new DefaultButtonBar(controller);

    table = new ObfTable(new ObfTableModel(controller.getManager(), new ExampleLbBusinessObject(), true, true));
    table.setDataObject(controller.getUidModel());
    table.selectFirst();

    card = new JPanel(new CardLayout());
    card.add("browse", browsePanel);
    card.add("edit", editPanel);
    
    setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints();

    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0.0;
    gbc.weighty = 1.0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.insets = new Insets(10,10,10,10);
    add(table.createScrollingTable(), gbc);

    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.insets = new Insets(10,0,10,0);
    add(card, gbc);

    gbc.gridx = 2;
    gbc.gridy = 0;
    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.weightx = 0.0;
    gbc.weighty = 1.0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.insets = new Insets(10,10,10,10);
    add(buttons, gbc);
  }


  public void browse() {
    ((CardLayout) card.getLayout()).show(card, "browse");
    browsePanel.setActive(true);
    editPanel.setActive(false);
  }

  public void edit() {
    ((CardLayout) card.getLayout()).show(card, "edit");
    browsePanel.setActive(false);
    editPanel.setActive(true);
  }

}


