/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.example;


import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import org.locomotive.obf.server.*;
import java.sql.*;



public class ExampleLbHandler extends DefaultBusinessObjectHandler implements ReloadListener {

  static {
    // register this handler the first time it is loaded
    ReloadQueue.getInstance().addReloadListener(ExampleLbHandler.class, ExampleRwBusinessObject.class);
  }


  public ExampleLbHandler() {
    super();
    setBusinessObjectClass(ExampleLbBusinessObject.class);
    setMainTable("obf_example");
    setPrimaryKey("id");
    setOrderBy("name");
    setSelectSql("SELECT id, name FROM obf_example");
  }


  protected UidBusinessObject getObjectFromResultSetRow(ResultSet rs, Connection con) throws ObfException, SQLException {
    String code = rs.getString(1);
    String name = rs.getString(2);
    return new ExampleLbBusinessObject(code, name);
  }

  protected UniqueId getPrimaryKey(BusinessObject busObj) throws ObfException {
    return ((UidBusinessObject) busObj).getUniqueId();
  }


  public void reload(ReloadNotice notice, Connection con) {
    if (notice.getBusClass() == ExampleRwBusinessObject.class) {

      try {
        ExampleLbBusinessObject o = null;
        if (notice.getUniqueId() != null)
          o = (ExampleLbBusinessObject) getBusinessObject(ExampleLbBusinessObject.class, notice.getUniqueId(), con);

        switch (notice.getType()) {
        case ReloadNotice.INSERT:
          ClientNotificationQueue.getInstance().insertedObject(o);
          break;
        case ReloadNotice.UPDATE:
          ClientNotificationQueue.getInstance().updatedObject(o);
          break;
        case ReloadNotice.DELETE:
          ClientNotificationQueue.getInstance().deletedObject(o);
          break;
        case ReloadNotice.REFRESH:
          ClientNotificationQueue.getInstance().refreshObjects(ExampleLbBusinessObject.class);
          break;
        }

      } catch (Exception e) {
        //?
      }
    }
  }


}

