/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.example;


import org.locomotive.obf.*;
import org.locomotive.obf.data.*;
import org.locomotive.obf.event.*;
import java.util.Date;
import java.io.*;


public class ExampleRwBusinessObject extends AbstractRwBusinessObject {

  public static final String[] STATUS = {"High", "Normal", "Low"};

  private UniqueStringId  code;
  private StringDocument  name;
  private DateDocument    date;
  private IntegerDocument value;
  private EnumModel       status;
  private BooleanModel    checked;




  public ExampleRwBusinessObject(UniqueStringId c, StringDocument n, DateDocument d, IntegerDocument v, EnumModel s, BooleanModel b) {
    code = c;
    name = n;
    date = d;
    value = v;
    status = s;
    checked = b;

    name.setLength(1, 10);
    name.addValidityListener(this);
    date.addValidityListener(this);
    value.addValidityListener(this);
    
    wasValid = isValid();
  }


  public ExampleRwBusinessObject(String c, String n, Date d, int v, int s, boolean b) {
    this(UniqueStringId.create(c), new StringDocument(n), new DateDocument(d), new IntegerDocument(v), new EnumModel(s), new BooleanModel(b));
  }


  public ExampleRwBusinessObject() {
    this(null, new StringDocument(), new DateDocument(), new IntegerDocument(0), new EnumModel(1), new BooleanModel(false));
  }



  public UniqueId getUniqueId() {
    return code;
  }
  
  public void setUniqueId(UniqueId uid) {
    if (uid == null || uid instanceof UniqueStringId)
      code = (UniqueStringId) uid;
    else
      throw new IllegalArgumentException("setUniqueId must be passed UniqueStringId");
  }




  public UniqueStringId  getCode()    { return code; }
  public StringDocument  getName()    { return name; }
  public DateDocument    getDate()    { return date; }
  public IntegerDocument getValue()   { return value; }
  public EnumModel       getStatus()  { return status; }
  public BooleanModel    getChecked() { return checked; }




  public boolean isValid() {
    return name.isValid() && date.isValid() && value.isValid();
  }
  


  public String toString() {
    return getClass().getName() + "[code="+code+",name="+name+"]";
  }


}

