/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.example;


import org.locomotive.obf.*;
import org.locomotive.obf.server.*;
import org.locomotive.obf.data.*;
import java.sql.ResultSet;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Vector;
import java.util.Date;



public class ExampleRwHandler extends DefaultBusinessObjectHandler implements ReloadListener {

  static {
    ReloadQueue.getInstance().addReloadListener(ExampleRwHandler.class, ExampleRwBusinessObject.class);
  }


  public ExampleRwHandler() {
    super();
    setBusinessObjectClass(ExampleRwBusinessObject.class);
    setMainTable("obf_example");
    setPrimaryKey("id");
    setSelectSql("SELECT id, name, day, value, status, checked FROM obf_example");
    setAllowInsert(true);
    setAllowUpdate(true);
    setAllowDelete(true);
  }

  protected UidBusinessObject getObjectFromResultSetRow(ResultSet rs, Connection con) throws SQLException {
    String  code    = rs.getString(1);
    String  name    = rs.getString(2);
    Date    date    = rs.getDate(3);
    int     value   = rs.getInt(4);
    int     status  = rs.getInt(5);
    boolean checked = rs.getBoolean(6);
    return new ExampleRwBusinessObject(code, name, date, value, status, checked);
  }

  protected UniqueId getPrimaryKey(BusinessObject busObj) throws ObfException {
    return ((UidBusinessObject) busObj).getUniqueId();
  }

  protected UidBusinessObject getPreparedInsertObject(UidBusinessObject busObj, Connection con) throws ObfException {
    ExampleRwBusinessObject o = (ExampleRwBusinessObject) busObj;
    o.setUniqueId(ExampleUidGenerator.getInstance().getNextUid(con));
    return o;
  }

  protected void prepareInsertSql(UidBusinessObject busObj, Connection con) throws ObfException {
    ExampleRwBusinessObject o = (ExampleRwBusinessObject) busObj;
    addToInsert("id",      o.getCode());
    addToInsert("name",    o.getName());
    addToInsert("day",     o.getDate());
    addToInsert("value",   o.getValue());
    addToInsert("status",  o.getStatus());
    addToInsert("checked", o.getChecked());
  }

  protected void prepareUpdateSql(UidBusinessObject busObj, Connection con) throws ObfException {
    ExampleRwBusinessObject o = (ExampleRwBusinessObject) busObj;
    addToUpdate("name",    o.getName());
    addToUpdate("day",     o.getDate());
    addToUpdate("value",   o.getValue());
    addToUpdate("status",  o.getStatus());
    addToUpdate("checked", o.getChecked());
  }


  public void reload(ReloadNotice notice, Connection con) {
    if (notice.getBusClass() == ExampleRwBusinessObject.class) {

      try {
        ExampleRwBusinessObject o = null;
        if (notice.getUniqueId() != null)
          o = (ExampleRwBusinessObject) getBusinessObject(ExampleRwBusinessObject.class, notice.getUniqueId(), con);

        switch (notice.getType()) {
        case ReloadNotice.INSERT:
          ClientNotificationQueue.getInstance().insertedObject(o);
          break;
        case ReloadNotice.UPDATE:
          ClientNotificationQueue.getInstance().updatedObject(o);
          break;
        case ReloadNotice.DELETE:
          ClientNotificationQueue.getInstance().deletedObject(o);
          break;
        case ReloadNotice.REFRESH:
          ClientNotificationQueue.getInstance().refreshObjects(ExampleRwBusinessObject.class);
          break;
        }

      } catch (Exception e) {
        //?
      }
    }
  }



}
