/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.example;

import org.locomotive.obf.*;
import org.locomotive.obf.data.*;
import org.locomotive.obf.server.UniqueIdGenerator;
import java.sql.*;


public class ExampleUidGenerator implements UniqueIdGenerator{

  private static ExampleUidGenerator instance = null;

  private int next = 0, max = 0, increment = 10;


  private ExampleUidGenerator() {
  }


  public static ExampleUidGenerator getInstance() {
    if (instance == null) {
      synchronized(ExampleUidGenerator.class) {
        if (instance == null)
          instance = new ExampleUidGenerator();
      }
    }
    return instance;
  }



  public UniqueId getNextUid(Connection con) throws ObfException {

    // have we used up all the ids in the current allocation?
    if (next >= max) {

      try {
        Statement stmt = con.createStatement();
        
        synchronized(ExampleUidGenerator.class) {

          // get existing highest id
          String statement = "select next_available from obf_ids";
          ResultSet rs = stmt.executeQuery(statement);
          
          // is the next available id in the database?
          if (rs.next()) {
            next = rs.getInt(1);
            max = next + increment - 1;

            // if so, set to new highest id
            statement = "update obf_ids set next_available = "+(max+1);
            stmt.executeUpdate(statement);

          } else {
            // if not, add it to the database
            statement = "insert into obf_ids (next_available) values ("+increment+")";
            stmt.executeUpdate(statement);

            next = 0;
            max = increment - 1;
          }

          rs.close();
          stmt.close();

        } // end synchronized

      } catch (SQLException e) {
        throw new ObfException(""+e);
      }

    } else {
      // use from existing allocation
      next++;
    }

    return UniqueStringId.create(""+next);

  }



}


