/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import org.locomotive.obf.event.RefreshListener;
import java.util.Vector;
import java.io.*;
import java.net.*;


// NOTE: This might need to implement equals()

public class ConnectionToHttpClient implements ConnectionToClient {

  private String url;
  private int port;


  public ConnectionToHttpClient(String url, int port) {
    this.url = url;
    this.port = port;
  }


  public Object doProcess(BusinessProcess process)
  throws ObfException, IOException {

    System.out.println("sending "+process.getClass().getName()+
                       " to client at "+url+" on port "+port);

    ObjectOutputStream out = null;

    // establish connection
    try {

      Socket sock = new Socket(url, port);
      out = new ObjectOutputStream(sock.getOutputStream());

      out.writeObject(process);

      // close stream + send request
      out.close();
      sock.close();

    } catch (MalformedURLException e) {
      //throw new ConnectionToServerException(""+e);
    }


    try { out.close(); } catch (Exception e) {}

    return null;
  }


  // don't implement receiveProcess()
  // because it comes to the ObfServlet instead!


  public String getUrl() {
    return url;
  }

  public void setUrl(String u) {
    url = u;
  }

  public int getPort() {
    return port;
  }

  public void setPort(int p) {
    port = p;
  }

  
  // RefreshListener

  /**
   * Invoked whenever this object receives notification of an
   * inserted business object.
   * It then notifies the client of this change.
   *
   * @param      busObject the inserted business object
   */

  public void insertedObject(UidBusinessObject busObject) {
    try {
      doProcess(new RefreshProcess(RefreshProcess.INSERT, busObject, null));
    } catch (ObfException e) {
      e.printStackTrace();
    } catch (IOException e) {
      System.out.println("failed to notify "+this+" of insert, don't notify them of anything again");
      ClientNotificationQueue.getInstance().removeRefreshListeners(this);
    }

  }


  /**
   * Invoked whenever this object receives notification of an
   * updated business object.
   * It then notifies the client of this change.
   *
   * @param      busObject the updated business object
   */

  public void updatedObject(UidBusinessObject busObject) {

    try {
      doProcess(new RefreshProcess(RefreshProcess.UPDATE, busObject, null));
    } catch (ObfException e) {
      e.printStackTrace();
    } catch (IOException e) {
      System.out.println("failed to notify "+this+" of update, don't notify them of anything again");
      ClientNotificationQueue.getInstance().removeRefreshListeners(this);
    }

  }


  /**
   * Invoked whenever this object receives notification of an
   * deleted business object.
   * It then notifies the client of this change.
   *
   * @param      busObject the deleted business object
   */

  public void deletedObject(UidBusinessObject busObject) {

    try {
      doProcess(new RefreshProcess(RefreshProcess.DELETE, busObject, null));
    } catch (ObfException e) {
      e.printStackTrace();
    } catch (IOException e) {
      System.out.println("failed to notify "+this+" of delete, don't notify them of anything again");
      ClientNotificationQueue.getInstance().removeRefreshListeners(this);
    }

  }


  /**
   * Invoked whenever this object receives notification of a business object
   * class that needs to be refreshed.
   * It then notifies the client of this change.
   *
   * @param      busClass the business object class that needs refreshing
   */

  public void refreshObjects(Class busClass) {

    try {
      doProcess(new RefreshProcess(RefreshProcess.REFRESH, null, busClass));
    } catch (ObfException e) {
      e.printStackTrace();
    } catch (IOException e) {
      System.out.println("failed to notify "+this+" of refresh, don't notify them of anything again");
      ClientNotificationQueue.getInstance().removeRefreshListeners(this);
    }

  }


  public String toString() {
    return "["+getClass().getName()+":"+url+":"+port+"]";

  }


  public boolean equals(Object o) {
    if (o != null && o instanceof ConnectionToHttpClient) {
      ConnectionToHttpClient c = (ConnectionToHttpClient) o;
      return url.equals(c.url) && port == c.port;
    } else {
      return false;
    }
  }


  public int hashCode() {
    return (url+port).hashCode();
  }


}
