/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;


import org.locomotive.obf.*;
import org.locomotive.obf.data.*;
import java.sql.*;
import java.util.*;
import java.text.*;
import java.math.*;


/**
 * Default handler for business objects.
 * <p>
 * [Pending: more javadoc to follow]
 * <p>
 * <b>Note: this class may change</b>
 *
 * @author David Warnock
 */

public abstract class DefaultBusinessObjectHandler implements BusinessObjectHandler {

   protected static char QUOTE='\'';
   protected static char COMMA=',';
   protected static char NEWLINE='\n';

   private Class dac = Class.class;
   private Class dtc = Class.class;

   protected UniqueIdGenerator uidGenerator;
   protected SqlRenderer renderer = SqlRendererJdbc.getInstance();

   private String updateSets  = "";
   private String updateWhere = "";

   private String insertCols   = "";
   private String insertValues = "";

   private boolean autoInsertSql       = true;
   private boolean autoUpdateSql       = true;
   private boolean allowInsert         = false;
   private boolean allowUpdate         = false;
   private boolean allowDelete         = false;
   private boolean allowProcess        = false;
   private boolean deleteByDepreciate  = true;
   private boolean ignoreDepreciated   = true;

   private Class   busClass = null;

   private String depreciateColName   = "depreciated";
   private String depreciatedValue    = "1";
   private String notDepreciatedValue = "0";
   private String mainTable         = null;
   private String primaryKey        = "";
   private String selectSql         = "";
   private String orderBy           = "";
   private String selectWhere       = "";

   protected abstract UidBusinessObject getObjectFromResultSetRow(ResultSet rs, Connection con) throws ObfException, SQLException;
   protected abstract UniqueId getPrimaryKey(BusinessObject busObj) throws ObfException;

   protected String getSelectSql(Class busClass, Object whereObj, Object orderBy, Connection con) throws ObfException {
      Debug.assert(false, "ObfHandlerSql.getSelectSql must be overridden to support this type of get request");
      throw new ObfException("ObfHandlerSql.getSelectSql must be overridden to support this type of get request");
   }
   protected String getDeleteSql(UidBusinessObject busObj, Connection con) throws ObfException {
      Debug.assert(false, "ObfHandlerSql.getDeleteSql must be overridden to support this type of delete request");
      throw new ObfException("ObfHandlerSql.getDeleteSql must be overridden to support this type of delete request");
   }
   protected String getInsertSql(UidBusinessObject busObj, Connection con) throws ObfException {
      Debug.assert(false, "ObfHandlerSql.getInsertSql must be overridden as autoInsertSql is false");
      throw new ObfException("ObfHandlerSql.getInsertSql must be overridden as autoInsertSql is false");
   }
   protected String getUpdateSql(UidBusinessObject busObj, Connection con) throws ObfException {
      Debug.assert(false, "ObfHandlerSql.getUpdateSql must be overridden as autoUpdateSql is false");
      throw new ObfException("ObfHandlerSql.getUpdateSql must be overridden as autoUpdateSql is false");
   }

   protected UidBusinessObject getPreparedInsertObject(UidBusinessObject busObj, Connection con) throws ObfException { return busObj;}

   protected void  prepareInsertSql(UidBusinessObject busObj, Connection con) throws ObfException {
      Debug.assert(false, "ObfHandlerSql.prepareInsertSql must be overridden to support inserting rows");
      throw new ObfException("ObfHandlerSql.prepareInsertSql must be overridden to support inserting rows");
   }
   protected void  prepareUpdateSql(UidBusinessObject busObj, Connection con) throws ObfException {
      Debug.assert(false, "ObfHandlerSql.prepareUpdateSql must be overridden to support updating rows");
      throw new ObfException("ObfHandlerSql.prepareUpdateSql must be overridden to support updating rows");
   }

   protected String getWhereFromPrimaryKey(UniqueId pK) throws ObfException {
      if (pK instanceof UniqueStringId) {
         return renderer.getSqlCompare(primaryKey, (UniqueStringId) pK);
         //return primaryKey + " = '" + ((UniqueStringId)pK).getUniqueId() + "'";
      } else if (pK instanceof UniqueIntegerId) {
         return renderer.getSqlCompare(primaryKey, (UniqueIntegerId) pK);
      }

      if (pK == null ) {
         return "";
      }
      Debug.assert(false, "ObfHandlerSql.getWhereFromPrimaryKey cannot generate Primary Key automatically from: " + primaryKey.getClass() );
      throw new ObfException("ObfHandlerSql.getWhereFromPrimaryKey cannot generate Primary Key automatically from: " + primaryKey.getClass());
   }


   // these allow custom processing
   protected void afterInsert(BusinessObject busObj, UniqueId id, Connection con) throws ObfException {};
   protected void afterUpdate(BusinessObject busObj, Connection con) throws ObfException {};
   protected void afterDelete(BusinessObject busObj, Connection con) throws ObfException {};

   protected void beforeInsert(BusinessObject busObj, UniqueId id, Connection con) throws ObfException {};
   protected void beforeUpdate(BusinessObject busObj, Connection con) throws ObfException {};
   protected void beforeDelete(BusinessObject busObj, Connection con) throws ObfException {};





   public void setUidGenerator(UniqueIdGenerator g) {
      uidGenerator = g;
   }

   public UniqueIdGenerator getUidGenerator() {
      return uidGenerator;
   }

   public void setRenderer(SqlRenderer r) {
      renderer = r;
   }

   public SqlRenderer getRenderer() {
      return renderer;
   }




   protected void setAutoInsertSql(boolean b)      { autoInsertSql       = b;}
   protected void setAutoUpdateSql(boolean b)      { autoUpdateSql       = b;}
   protected void setAllowInsert(boolean b)        { allowInsert         = b;}
   protected void setAllowUpdate(boolean b)        { allowUpdate         = b;}
   protected void setAllowDelete(boolean b)        { allowDelete         = b;}
   protected void setAllowProcess(boolean b)       { allowProcess        = b;}
   protected void setDeleteByDepreciate(boolean b) { deleteByDepreciate  = b;}
   protected void setIgnoreDepreciated(boolean b)  { ignoreDepreciated   = b;}

   protected void setBusinessObjectClass(Class c)  { busClass = c;}

   protected void setDepreciateColName(String s)   { depreciateColName   = s;}
   protected void setDepreciatedValue(String s)    { depreciatedValue    = s;}
   protected void setNotDepreciatedValue(String s) { notDepreciatedValue = s;}

   protected void setMainTable(String s)           { mainTable           = s;}
   protected void setPrimaryKey(String s)          { primaryKey          = s;}
   protected void setSelectSql(String s)           { selectSql           = s;}
   protected void setOrderBy(String s)             { orderBy             = s;}
   protected void setSelectWhere(String s)         { selectWhere         = s;}

   protected String getSelectSqlStr()              { return selectSql;}
   protected String getSelectWhereStr()              { return selectWhere;}

   protected void checkBusClass(Class busClass) throws ObfException {
      if (busClass == null) {
         throw new ObfException(this.getClass() +  " no Business Class Provided for checking");
      }
      if (busClass != busClass) {
         throw new ObfException(this.getClass() +  " wrong business class provided: " + busClass);
      }
   }

   protected void clearUpdate() {
      updateSets  = "";
      updateWhere = "";
   }

   private String getUpdateSets() {
      return updateSets;
   }

   private String getUpdateWhere() {
      return updateWhere;
   }






   // --- Update

   private void addUpdateDelimiters() {
      if (!updateSets.equals(""))  updateSets += ", ";
      if (!updateWhere.equals("")) updateWhere += " AND ";
   }

   protected void addToUpdate(String sCol, EditableDataObject o) {
      if (o != null && o.isModified()) {
         addUpdateDelimiters();
         updateSets  += renderer.getSqlAssignment(sCol, o);
         updateWhere += renderer.getSqlCompareOriginal(sCol, o);
      }
   }

   protected void addToUpperUpdate(String sCol, EditableDataObject o) {
      if (o != null && o.isModified()) {
         addUpdateDelimiters();
         updateSets  += renderer.getSqlAssignment(sCol, o, true);
         updateWhere += renderer.getSqlCompareOriginal(sCol, o, true);
      }
   }

   protected void addToUpdate(String sCol, UniqueStringId o, UniqueStringId n) {
      addUpdateDelimiters();
      updateSets  += renderer.getSqlAssignment(sCol, n);
      updateWhere += renderer.getSqlCompare(sCol, o);
   }



   // --- check these

   protected void addToUpdate(String sCol, String sOld, String sNew) {
      if ( sOld == null ) sOld = "";
      if ( sNew == null ) sNew = "";

      if ( !sOld.equals(sNew) ) {
         if (!updateSets.equals("")) updateSets = updateSets + ", ";

         if (!updateWhere.equals("")) updateWhere = updateWhere + " AND ";

         updateSets  = updateSets  + sCol + " = '"  + sNew + "'";
         updateWhere = updateWhere + sCol + " = '"  + sOld + "'";
      }
   }




   protected void clearInsert() {
      insertCols   = "";
      insertValues = "";
   }

   private String getInsertCols() {
      return insertCols;
   }

   private String getInsertValues() {
      return insertValues;
   }





   // --- Insert

   private void addInsertDelimiters() {
      if (!insertCols.equals(""))   insertCols += ", ";
      if (!insertValues.equals("")) insertValues += ", ";
   }

   protected void addToInsert(String sCol, EditableDataObject o) {
      addInsertDelimiters();
      insertCols   += sCol;
      insertValues += renderer.getSql(o);
   }

   protected void addToUpperInsert(String sCol, EditableDataObject o) {
      addInsertDelimiters();
      insertCols   += sCol;
      insertValues += renderer.getSql(o, true);
   }

   protected void addToInsert(String sCol, UniqueStringId u) {
      addInsertDelimiters();
      insertCols   += sCol;
      insertValues += renderer.getSql(u);
   }

   protected void addToInsert(String sCol, String sValue) {
      addInsertDelimiters();
      insertCols   += sCol;
      insertValues = insertValues + "'" + sValue + "'";
   }

   protected void addToInsert(String sCol, int iValue) {
      addInsertDelimiters();
      insertCols   += sCol;
      insertValues += iValue;
   }

   protected void addNullToInsert(String sCol) {
      addInsertDelimiters();
      insertCols   += sCol;
      insertValues += "null";
   }




   // IobfRetriever
   private synchronized Vector getBusinessObjects(Class busClass, Connection con) throws ObfException, SQLException {
      Vector result = new Vector(100, 10);
      checkBusClass(busClass);
      Statement stmt = null;
      ResultSet rs = null;
      try {
      String sWhere = "";
      String sOrderBy = "";

      if ( ignoreDepreciated ) {
         if ( selectWhere.equals("") ) {
            sWhere = " WHERE " + depreciateColName + " = " + notDepreciatedValue ;
         } else {
            sWhere = " WHERE " + depreciateColName + " = " + notDepreciatedValue + " AND " + selectWhere;
         }
      } else if ( !selectWhere.equals("") ) {
         sWhere = " WHERE " + selectWhere;
      }

      if ( !orderBy.equals("") ) {
         sOrderBy = " ORDER BY " + orderBy;
      }

      String sql = selectSql + sWhere + sOrderBy;

      Debug.msg(dac, dtc, "getBusinessObjects(class) sql = " + sql);
      stmt = con.createStatement();
      rs   = stmt.executeQuery(sql);

      while (rs.next() ) {
         result.addElement( getObjectFromResultSetRow(rs, con) );
      }

      } finally {
        if (rs != null) rs.close();
        if (stmt != null) stmt.close();
      }

      return result;
      //} catch (SQLException e) {
      //  throw new ObfException(dtc + ".getBusinessObjects SQLException: " + e.getMessage() );
      //}
   }



   private synchronized Vector getBusinessObjects(Class busClass, int iMaxRows, Connection con) throws ObfException, SQLException {
      Vector result = new Vector(100, 10);
      checkBusClass(busClass);
      //try {
      String sWhere = "";
      String sOrderBy = "";

      if ( ignoreDepreciated ) {
         if ( selectWhere.equals("") ) {
            sWhere = " WHERE " + depreciateColName + " = " + notDepreciatedValue ;
         } else {
            sWhere = " WHERE " + depreciateColName + " = " + notDepreciatedValue + " AND " + selectWhere;
         }
      } else if ( !selectWhere.equals("") ) {
         sWhere = " WHERE " + selectWhere;
      }

      if ( !orderBy.equals("") ) {
         sOrderBy = " ORDER BY " + orderBy;
      }

      String sql = selectSql + sWhere + sOrderBy;

      Debug.msg(dac, dtc, "getBusinessObjects(class, count) sql = " + sql);

      Statement stmt = null;
      try {
         stmt = con.createStatement();
         ResultSet rs = null;
         try {
            rs   = stmt.executeQuery(sql);

            int i = 1;
            while (rs.next() && i <= iMaxRows) {
               result.addElement( getObjectFromResultSetRow(rs, con) );
               i++;
            }

         } finally {
            if (rs != null) rs.close();
         }
      } finally {
         if (stmt != null) stmt.close();
      }

      return result;
      //} catch (SQLException e) {
      //  throw new ObfException(dtc + ".getBusinessObjects with max rows SQLException: " + e.getMessage() );
      //}
   }

   private synchronized Vector getBusinessObjects(Class busClass, Object whereObj, Connection con) throws ObfException, SQLException {
      return getBusinessObjects(busClass, whereObj, null, con);
   }

   private synchronized Vector getBusinessObjects(Class busClass, Object whereObj, int iMaxRows, Connection con) throws ObfException, SQLException  {
      return getBusinessObjects(busClass, whereObj, null, iMaxRows, con);
   }

   private synchronized Vector getBusinessObjects(Class busClass, Object whereObj, Object orderByObj, Connection con) throws ObfException, SQLException  {
      Vector result = new Vector(100, 10);
      checkBusClass(busClass);
      //try {
      String sql     = getSelectSql(busClass, whereObj, orderByObj, con);
      Debug.msg(dac, dtc, "getBusinessObjects(all param) sql = " + sql);
      Statement stmt = null;
      try {
         stmt = con.createStatement();
         ResultSet rs = null;
         try {
            rs   = stmt.executeQuery(sql);

            while (rs.next() ) {
               result.addElement( getObjectFromResultSetRow(rs, con) );
            }

         } finally {
            if (rs != null) rs.close();
         }
      } finally {
         if (stmt != null) stmt.close();
      }
      return result;
      //} catch (SQLException e) {
      //  throw new ObfException(dtc + ".getBusinessObjects with max rows SQLException: " + e.getMessage() );
      //}
   }

   public synchronized Vector getBusinessObjects(Class busClass, Object whereObj, Object orderByObj, int iMaxRows, Connection con) throws ObfException, SQLException  {

      // this is a bit of hack for now to have 1 getBusinessObjects() method and still avoid implementing getSelectSql for everything - ee
      if (whereObj == null && orderByObj == null) {
         if (iMaxRows < 0)
            return getBusinessObjects(busClass, con);
         else
            return getBusinessObjects(busClass, iMaxRows, con);
      }
      // end of hack


      Vector result = new Vector(100, 10);
      checkBusClass(busClass);
      //try {
      String sql     = getSelectSql(busClass, whereObj, orderByObj, con);
      Debug.msg(dac, dtc, "getBusinessObjects(all param inc count) sql = " + sql);
      Statement stmt = null;
      try {
         stmt = con.createStatement();
         ResultSet rs = null;
         try {
            rs   = stmt.executeQuery(sql);

            int i = 1;
            boolean dontUseLimit = iMaxRows < 0;
            while (rs.next() && (dontUseLimit || i <= iMaxRows)) {
               result.addElement( getObjectFromResultSetRow(rs, con) );
               i++;
            }

         } finally {
            if (rs != null) rs.close();
         }
      } finally {
         if (stmt != null) stmt.close();
      }

      return result;

      //} catch (SQLException e) {
      //  throw new ObfException(dtc + ".getBusinessObjects SQLException: " + e.getMessage() );
      //}
   }


   public synchronized UidBusinessObject getBusinessObject(Class busClass, UniqueId whereObj, Connection con) throws ObfException, SQLException  {
      UidBusinessObject result = null;
      checkBusClass(busClass);
      //try {
      String sP = getWhereFromPrimaryKey(whereObj);
      String sql;
      if (sP == null || sP.equals("") ) {
         if ( selectWhere.equals("") ) {
            sql = selectSql;
         } else {
            sql = selectSql + " WHERE " + selectWhere;
         }
      } else {
         if ( selectWhere.equals("") ) {
            sql = selectSql + " WHERE " + sP;
         } else {
            sql = selectSql + " WHERE " + sP + " AND " + selectWhere;
         }
      }
      Debug.msg(dac, dtc, "getBusinessObject sql = " + sql);
      Statement stmt = null;
      try {
         stmt = con.createStatement();
         ResultSet rs = null;
         try {
            rs   = stmt.executeQuery(sql);

            if (rs.next()) {
               result = getObjectFromResultSetRow(rs, con);
            }
         } finally {
            if (rs != null) rs.close();
         }
      } finally {
         if (stmt != null) stmt.close();
      }

      return result;
      //} catch (SQLException e) {
      //  throw new ObfException(dtc + ".getBusinessObject: " + e.getMessage() );
      //}
   }


   public BusinessObject getDefaultBusinessObject(Class busClass, Connection con) throws ObfException {
      checkBusClass(busClass);
      try {
         return (BusinessObject) busClass.newInstance();
      } catch (InstantiationException e) {
         throw new ObfException("ObfHandlerSql.getDefaultBusinessObject InstantiationException " + e.getMessage() );
      } catch (IllegalAccessException e) {
         throw new ObfException("ObfHandlerSql.getDefaultBusinessObject IllegalAccessException " + e.getMessage() );
      }
   }



   public UniqueId insertBusinessObject(UidBusinessObject busObj, Connection con) throws ObfException, SQLException {
      if (!allowInsert) {
         throw new ObfException("ObfHandlerSql.insertBusinessObject Insert not allowed");
      }
      checkBusClass(busObj.getClass());
      //try {
      String sql;
      //Connection con = svr.getConnection();
      Debug.msg(dac, dtc, "InsertBusinessObject 1");

      busObj = getPreparedInsertObject(busObj, con);
      UniqueId id = getPrimaryKey(busObj);

      clearInsert();
      if (autoInsertSql) {
         Debug.msg(dac, dtc, "InsertBusinessObject 2");
         prepareInsertSql(busObj, con);
         if ( ignoreDepreciated ) {
            sql = "INSERT INTO " + mainTable + " (" + getInsertCols() + ", " + depreciateColName + ") VALUES (" + getInsertValues() + ", " + notDepreciatedValue + ")";
         } else {
            sql = "INSERT INTO " + mainTable + " (" + getInsertCols() + ") VALUES (" + getInsertValues() + ")";
         }
      } else {
         Debug.msg(dac, dtc, "InsertBusinessObject 2");
         sql = getInsertSql(busObj, con);
      }
      clearInsert();

      Statement stmt = null;
      try {
         stmt  = con.createStatement();

         Debug.msg(dac, dtc, "InsertBusinessObject sql = " + sql);
         beforeInsert(busObj, id, con);
         stmt.executeUpdate(sql);
      } finally {
         if (stmt != null) stmt.close();
      }
      afterInsert(busObj, id, con);

      //this is now done by the InsertProcessHandler
      //svr.addToReloadQueue( new ObfReloadObject( ObfReloadObject.INSERT_RELOAD, getPrimaryKey(busObj), busObj.getClass() ));
      return id;
      //} catch (SQLException e) {
      //  throw new ObfException("ObfHandlerSql.insertBusinessObject SQLException: " + e.getMessage() );
      //}
   }



   public void updateBusinessObject(UidBusinessObject busObj, Connection con) throws ObfException, SQLException {
      if (!allowUpdate) {
         throw new ObfException("ObfHandlerSql.updateBusinessObject Update not allowed");
      }
      checkBusClass(busObj.getClass());
      //try {
      String sql;
      //Connection con = svr.getConnection();
      clearUpdate();
      if (autoUpdateSql) {
         prepareUpdateSql(busObj, con);
         String sW = getWhereFromPrimaryKey( getPrimaryKey(busObj) );
         if ( getUpdateSets().equals("") ) {
            sql = null;
         } else {
            if (sW == null || sW.equals("")) {
               sql = "UPDATE " + mainTable + " SET " + getUpdateSets() + " WHERE " + getUpdateWhere();
            } else {
               sql = "UPDATE " + mainTable + " SET " + getUpdateSets() + " WHERE " + sW + " AND " + getUpdateWhere();
            }
         }
      } else {
         sql = getUpdateSql(busObj, con);
      }
      clearUpdate();
      if (sql != null) {
        Statement stmt = null;
         try {
            stmt  = con.createStatement();

            Debug.msg(dac, dtc, "updateBusinessObject sql = " + sql);
            beforeUpdate(busObj, con);
            stmt.executeUpdate(sql);
         } finally {
            if (stmt != null) stmt.close();
         }
         afterUpdate(busObj, con);

         //this is now done by the UpdateProcessHandler
         //svr.addToReloadQueue( new ObfReloadObject( ObfReloadObject.UPDATE_RELOAD, getPrimaryKey(busObj), busObj.getClass() ));
      }
      //} catch (SQLException e) {
      //  throw new ObfException("ObfHandlerSql.updateBusinessObject SQLException: " + e.getMessage() );
      //}
   }



   public void deleteBusinessObject(UidBusinessObject busObj, Connection con) throws ObfException, SQLException {
      if (!allowDelete) {
         throw new ObfException("ObfHandlerSql.deleteBusinessObject Delete not allowed");
      }
      checkBusClass(busObj.getClass());
      //try {
      String sql;
      //Connection con = svr.getConnection();
      if (mainTable != null) {
         String sW = getWhereFromPrimaryKey( getPrimaryKey(busObj) );

         if (sW == null || sW.equals("")) {
            throw new ObfException("ObfHandlerSql.DeleteBusinessObject SQLException no primary key");
         }
         if (deleteByDepreciate) {
            sql = "UPDATE " + mainTable + " SET " + depreciateColName + " = 1 WHERE " + sW;
         } else {
            sql = "DELETE FROM " + mainTable + " WHERE " + sW;
         }
      } else {
         sql = getDeleteSql(busObj, con);
      }


      Debug.msg(dac, dtc, "deleteBusinessObject sql = " + sql);
      Statement stmt = null;
      try {
         stmt  = con.createStatement();

         beforeDelete(busObj, con);
         stmt.executeUpdate(sql);
      } finally {
         if (stmt != null) stmt.close();
      }
      afterDelete(busObj, con);

      //this is now done by the DeleteProcessHandler
      //svr.addToReloadQueue( new ObfReloadObject( ObfReloadObject.DELETE_RELOAD, getPrimaryKey(busObj), busObj.getClass() ));
      //} catch (SQLException e) {
      //  throw new ObfException("ObfHandlerSql.DeleteBusinessObject SQLException: " + e.getMessage() );
      //}
   }



   // --- Lookup foreign keys

   public String getNameFromUid(String nameColumn, String uidColumn, String table, String uid, Connection con) throws SQLException {
      String result = null;

      if (uid != null) {
        Statement stmt = null;
         try {
            stmt = con.createStatement();
            ResultSet rs = null;
            try {
               rs = stmt.executeQuery("SELECT " + nameColumn + " FROM " + table + " WHERE " + uidColumn + " = '" + uid + "'");
               if (rs.next())
                  result = rs.getString(1);
            } finally {
               rs.close();
            }
         } finally {
            stmt.close();
         }
      }
      return result;
   }


   public String getNameFromUid(String nameColumn, String uidColumn, String table, DataObject data, Connection con) throws SQLException {
      String result = null;

      if (data != null) {
        Statement stmt = null;
         try {
            stmt = con.createStatement();
            ResultSet rs = null;
            try {
               rs = stmt.executeQuery("SELECT " + nameColumn + " FROM " + table + " WHERE " + renderer.getSqlCompare(uidColumn, data));
               if (rs.next())
                  result = rs.getString(1);
            } finally {
               if (rs != null) rs.close();
            }
         } finally {
            if (stmt != null) stmt.close();
         }
      }
      return result;
   }




/*
  // BusinessProcess
  public Object getProcessResult(Object oProcess)   throws ObfException { Debug.assert(false, "DO NOT USE! Make sure Connection is passed and used"); return null;}
  public Object getProcessId(Object oProcess)       throws ObfException { Debug.assert(false, "DO NOT USE! Make sure Connection is passed and used"); return null;}
  public Object getProcessProgress(Object oProcess) throws ObfException { Debug.assert(false, "DO NOT USE! Make sure Connection is passed and used"); return null;}

  public Object getProcessResult(Object oProcess, ObfServerSql svr) throws ObfException {
    if (!allowProcess) {
      throw new ObfException("ObfHandlerSql.getProcessResult Process not allowed");
    }
    Debug.assert(false, "Not implemented yet"); return null;
  }

  public Object getProcessId(Object oProcess, ObfServerSql svr) throws ObfException {
    if (!allowProcess) {
      throw new ObfException("ObfHandlerSql.getProcessId Process not allowed");
    }
    Debug.assert(false, "Not implemented yet"); return null;
  }

  public Object getProcessProgress(Object oProcess, ObfServerSql svr) throws ObfException {
    if (!allowProcess) {
      throw new ObfException("ObfHandlerSql.getProcessProgress  Process not allowed");
    }
    Debug.assert(false, "Not implemented yet"); return null;
  }
  */

}
