/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import java.util.Vector;
import java.util.Enumeration;
import java.sql.Connection;
import java.sql.SQLException;


/**
 * Handles the <code>GetDefaultProcess</code> to get the default business
 * for a given class.
 *
 * @author Erik Erskine
 * @see    GetProcessHandler
 * @see    GetMultipleProcessHandler
 */

public class GetDefaultProcessHandler implements ProcessHandler {


  /**
   * Gets the default business object for a given class.
   *
   * @param      process  the <code>GetDefaultProcess</code> object describing
   *                      the business object to get.
   * @param      con      a database connection to use
   * @return     the default business object
   * @exception  ClassCastException if <code>process</code> is not of type
   *             <code>GetDefaultProcess</code>
   *             ObfException if another error occurs
   * @see        org.locomotive.obf.GetDefaultProcess
   */

  public Object doProcess(BusinessProcess process, Connection con)
  throws ObfException {

    GetDefaultProcess gp = (GetDefaultProcess) process;
    ServerSupporter supporter = ServerSupporter.getInstance();

    BusinessObjectHandler h =
      supporter.getBusinessObjectHandler(gp.getBusClass());

    BusinessObject result = null;
    
    try {
      result = h.getDefaultBusinessObject(gp.getBusClass(), con);

    } catch (SQLException e) {
//      supporter.dropConnection(con);
      throw new ObfException(""+e);
    }

    return result;
  }


}
