/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import java.util.Vector;
import java.util.Enumeration;
import java.sql.Connection;
import java.sql.SQLException;


/**
 * Handles the <code>GetMultipleProcess</code> to get business objects
 * from the database.
 *
 * @author Erik Erskine
 * @see    GetProcessHandler
 * @see    GetDefaultProcessHandler
 */

public class GetMultipleProcessHandler implements ProcessHandler {


  /**
   * Gets business objects from the database.
   *
   * @param      process  the <code>GetMultipleProcess</code> object
   *                      describing the business objects to get.
   * @param      con      a database connection to use
   * @return     a vector of business objects that were found
   * @exception  ClassCastException if the handler could not deal with
   *             processes of the type of <code>p</code>
   * @exception  ClassCastException if <code>process</code> is not of type
   *             <code>GetMultipleProcess</code>
   * @see        org.locomotive.obf.GetMultipleProcess
   */

  public Object doProcess(BusinessProcess process, Connection con)
  throws ObfException {

    GetMultipleProcess gp = (GetMultipleProcess) process;
    ServerSupporter supporter = ServerSupporter.getInstance();

    BusinessObjectHandler h =
      supporter.getBusinessObjectHandler(gp.getBusClass());

    Vector result = null;
    
    try {

      result = h.getBusinessObjects(gp.getBusClass(), gp.getWhere(),
                                    gp.getOrder(), gp.getLimit(), con);

    } catch (Exception e) {
      e.printStackTrace();
//      supporter.dropConnection(con);
      throw new ObfException(""+e);
    }

    return result;
  }


}
