/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import java.util.Vector;
import java.util.Enumeration;
import java.sql.Connection;
import java.sql.SQLException;


/**
 * Handles the <code>GetProcess</code> to get a single business object
 * from the database.
 *
 * @author Erik Erskine
 * @see    GetMultipleProcessHandler
 * @see    GetDefaultProcessHandler
 */

public class GetProcessHandler implements ProcessHandler {


  /**
   * Gets a business object from the database.
   *
   * @param      process  the <code>GetProcess</code> object describing
   *                      the business object to get.
   * @param      con      a database connection to use
   * @return     the business object if it was found, <code>null</code>
   *             if it could not be found
   * @exception  ClassCastException if <code>process</code> is not of type
   *             <code>GetProcess</code>
   *             ObfException if another error occurs
   * @see        org.locomotive.obf.GetProcess
   */

  public Object doProcess(BusinessProcess process, Connection con)
  throws ObfException {

    GetProcess gp = (GetProcess) process;
    ServerSupporter supporter = ServerSupporter.getInstance();

    BusinessObjectHandler h =
      supporter.getBusinessObjectHandler(gp.getBusClass());

    UidBusinessObject result = null;
    
    try {
      result = h.getBusinessObject(gp.getBusClass(), gp.getUniqueId(), con);
//      supporter.dropConnection(con);

    } catch (SQLException e) {
      throw new ObfException(""+e);
    }

    return result;
  }


}
