/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.*;
import org.locomotive.obf.data.UniqueId;
import java.sql.*;


/**
 * Handles the <code>InsertProcess</code> to insert a business object
 * into the database.
 *
 * @author Erik Erskine
 * @see    UpdateProcessHandler
 * @see    DeleteProcessHandler
 */

public class InsertProcessHandler extends TransactionProcessHandler {


  /**
   * Inserts a business object into the database.
   *
   * @param      process  the <code>InsertProcess</code> object containing
   *                      the business object to insert.
   * @param      con      a database connection to use
   * @return     the unique identifier this business object was allocated
   * @exception  ClassCastException if <code>process</code> is not of type
   *             <code>InsertProcess</code>
   *             ObfException if another error occurs
   * @see        org.locomotive.obf.InsertProcess
   */

  public Object doTransaction(BusinessProcess p, Connection con)
  throws ObfException, SQLException {

    InsertProcess ip = (InsertProcess) p;
    UidBusinessObject busObject = ip.getBusinessObject();
    UniqueId result = null;

    ServerSupporter supporter = ServerSupporter.getInstance();
    BusinessObjectHandler h = supporter.getBusinessObjectHandler(busObject.getClass());
    
    try {
      result = h.insertBusinessObject(busObject, con);
      addReloadNotice(ReloadNotice.INSERT, busObject.getClass(), result);

    } catch (SQLException e) {
      e.printStackTrace();
      throw (SQLException) e.fillInStackTrace();
    }

    return result;
  }


}
