/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */


package org.locomotive.obf.server;

import org.locomotive.util.*;
import org.locomotive.server.*;
import org.locomotive.loco.*;
import org.locomotive.loco.servlet.*;

import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.Connection;

import org.locomotive.obf.*;
import org.locomotive.obf.data.*;
import org.locomotive.obf.example.*;
import org.locomotive.obf.client.*;
import org.locomotive.obf.server.*;



/**
 * A locomotive servlet that is an OBF server.
 * <p>
 * [Pending: more documentation]
 * <p>
 * <b>Does not yet support notification of clients</b>
 *
 * @author Erik Erskine
 */


public class ObfServlet extends LocoServlet {



   /**
    * Sets up the various fields the class requires.
    * As <code>LocoServlet.setup()</code>, except that it does <i>not</i>
    * try and parse the post data because this is binary (a serialized object).
    *
    * @param      req               ???
    * @param      resp              ???
    * @exception  ServletException  ???
    * @exception  IOException       ???
    */

   public void setup(HttpServletRequest req, HttpServletResponse resp)
   throws IOException, ServletException {
      this.req = (LocoServletRequest) req;
      this.resp = resp;

      url_tokens = (String []) req.getAttribute("locomotive.url_tokens");
      steam_vars = (Hashtable) req.getAttribute("locomotive.steam_vars");
      conn = (Connection) req.getAttribute("locomotive.connection");
      //form_data = (FormData) req.getAttribute("locomotive.form_data");
      session = (Session) req.getSession(true);
      user = (User) session.getValue("locomotive.user");

      resp.setContentType("text/html");
      StandardInsertions.insertAll(req, steam_vars);
   }



   /**
    * Main entry point for handling HTTP POST requests.
    * All OBF processes are transmitted using the POST method.
    *
    * @param      req               ???
    * @param      resp              ???
    * @exception  ServletException  ???
    * @exception  IOException       ???
    */

   public void doPost(HttpServletRequest req, HttpServletResponse resp)
   throws ServletException, IOException {
      BusinessProcess process = null;

      try {
         // receive process object from the stream
         ObjectInputStream in = new ObjectInputStream(req.getInputStream());
         process = (BusinessProcess) in.readObject();
         System.out.println("* received "+process.getClass()+" from client");

      } catch (StreamCorruptedException e) {
         //throw new ConnectionToClientException(""+e);
         
      } catch (OptionalDataException e) {
         //throw new ConnectionToClientException(""+e);
         
      } catch (ClassNotFoundException e) {
         //throw new ConnectionToClientException(""+e);
         
      } catch (IOException e) {
         //throw new ConnectionToClientException(""+e);
         
      } catch (ClassCastException e) {
         //throw new ConnectionToClientException(""+e);
         
      }


      // now decide what to do with it
      Object result;
      try {

         // USE LOCOMOTIVE CONNECTION HERE!!!
         // check if each call to the servlet gets it's own connection
         // or if can they be shared
         // if they are shared we have a problem with transactions

         Connection con = ServerSupporter.getInstance().getConnection();

         try {
            ProcessHandler h = ServerSupporter.getInstance().getProcessHandler(process.getClass());
            result = h.doProcess(process, con);
            //result = h.doProcess(process, conn);

         } finally {
            ServerSupporter.getInstance().dropConnection(con);
         }


      } catch (ObfException e) {
         System.out.println("server caught "+e+" - sending it to client");
         e.printStackTrace();
         result = e;

      } catch (Exception e) {
        // if anything else goes wrong, ie some dodgy code in the handler,
        // grab it here and send a nice ObfException back to the client
        System.out.println("server caught "+e+" - sending an ObfException to client");
        e.printStackTrace();
        result = new ObfException("server caught "+e+" somewhat unexpectedly");

      }


      // now send the result back
      try {
         ObjectOutputStream out = new ObjectOutputStream(resp.getOutputStream());

         if (result != null) {
            //System.out.println("* sending "+result.getClass());
            
         } else {
            //System.out.println("* sending null");
            
         }

         out.writeObject(result);
         out.flush();
         out.close();
      } catch (Exception e) {
         e.printStackTrace();
      }

   }


}
