/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.BusinessProcess;
import org.locomotive.obf.ObfException;
import java.sql.Connection;


/**
 * Interface for process handler classes.  The process handlers are
 * responsible for putting notifications of changes on the reload queue.
 * They can call business object handlers to get, insert, update and delete
 * business objects done as part of the process.
 * <p>
 * [Pending: more documentation to follow]
 *
 * @author Erik Erskine
 * @see    TransactionProcessHandler
 */

public interface ProcessHandler {

  /**
   * Execute a specified process and return the result.
   *
   * @param      p     a process to execute
   * @param      con   a database connection to use
   * @exception  ClassCastException if the handler could not deal with
   *             processes of the type of <code>p</code>
   *             ObfException if another error occurs
   */

  public Object doProcess(BusinessProcess p, Connection con)
  throws ObfException;

}
