/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;



import org.locomotive.obf.data.*;


/**
 * Interface for sql renderers that simplify the generation of SQL statements.
 * <p>
 * <b>Note: these classes may change to make use of the
 * <code>org.locmotive.util.SQL</code> class</b>
 *
 * @author Erik Erskine
 * @see    org.locomotive.util.SQL
 */

public interface SqlRenderer {


  public String getSql(Object value);

  public String getSql(UniqueStringId uid);

  public String getSql(UniqueIntegerId uid);

  public String getSql(DataObject value);

  public String getSql(DataObject value, boolean upperCase);


  public String getSqlAssignment(String column, UniqueStringId u);

  public String getSqlAssignment(String column, UniqueIntegerId u);

  public String getSqlAssignment(String column, DataObject value);

  public String getSqlAssignment(String column, DataObject value,
                                 boolean upperCase);

  public String getSqlAssignment(String column, Object value);


  public String getSqlCompare(String column, UniqueStringId u);

  public String getSqlCompare(String column, UniqueIntegerId u);
  
  public String getSqlCompare(String column, DataObject value);

  public String getSqlCompare(String column, DataObject value,
                              boolean upperCase);

  public String getSqlCompare(String column, Object value);


  public String getSqlCompareOriginal(String column,
                                      EditableDataObject value);

  public String getSqlCompareOriginal(String column, EditableDataObject value,
                                      boolean upperCase);


}


