/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL are Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
 */

package org.locomotive.obf.server;

import org.locomotive.obf.ObfException;
import org.locomotive.obf.data.UniqueId;
import java.sql.Connection;


/**
 * Interface for unique identifier generators.
 * <p>
 * Note that each handler can have it's own unique id generator, so if
 * different types of identifier are required (eg, string, integer, etc)
 * for different handlers, then each one can have it's own generator.
 *
 * @author Erik Erskine
 * @author David Warnock
 */

public interface UniqueIdGenerator {

  /**
   * Returns the next unique identifier.  Once this method has been called,
   * the result must never returned again by this method in any class
   * implementing <code>UniqueIdGenerator</code> that uses the same database.
   *
   * @param      con a database connection to use
   * @exception  ObfException if there was some error
   */

  public UniqueId getNextUid(Connection con) throws ObfException;

}

