/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
*/

package org.locomotive.router;

import java.io.*;
import java.net.*;
import java.util.*;

class ClientConnection implements Runnable {


  private int connectionNo;
  private Socket clientSocket;
  private InputStream fromClient;
  private OutputStream toClient;
  private Router router;


  ClientConnection(Router router, Socket clientSocket, int connectionNo) {
    this.router = router;
    this.clientSocket = clientSocket;
    this.connectionNo = connectionNo;
  }








  public void run () {
    try {

      //collect request from client
      fromClient = clientSocket.getInputStream();
      toClient  = clientSocket.getOutputStream();


      HttpRequest request = null;
      try {
        request = new HttpRequest(fromClient);
      } catch (IOException e) {
        Router.showErrorMessage("<b>That did not appear to be a valid HTTP request, router could not determine where to send it.</b>", toClient);
        throw e;
      }


/*
      System.out.println(request.getContentLength());

      InputStream in = request.getInputStream();
      
      if (request.getContentLength() != null) {
        for (int x=0; x<request.getContentLength().intValue(); x++) {

          int ch = in.read();
          System.out.write(ch);

        }
      }
*/


      // do we know how to handle this request?

      // what is all this about????
      // if the line starts with LOCO-CONSOLE the request should goto the Locomotive console (this would be a Java Client connection not browser)
      // else if the line starts with LOCO-CLIENT the request should goto a Locomotive (using the load balancing etc) (this would be a Java Client connection not browser)
      // else if the URL includes loco-console the request should goto the Locomotive console (this would be a browser connection)
      // else if the URL includes locolink the request should goto a Locomotive (using the load balancing etc) (this would be a browser connection)

      //System.out.println(request.getPath());
      if (request.getUri().startsWith("/locolink")) {
        request.translatePath("/locolink", "");
        new LocomotiveConnection(router.getLocomotiveAddress(), request, toClient);
      } else {
        new WebServerConnection(router.getWebServerAddress(), request, toClient);
      }
      



    } catch (IOException e) {
      System.err.println("Run client connection exception: " + e);
    } finally {

      try {
        fromClient.close();
      } catch (IOException e) {
        e.printStackTrace();
      }
      try {
        toClient.close();
      } catch (IOException e) {
        e.printStackTrace();
      }

      try {
        clientSocket.close();
      } catch (IOException e) {
        e.printStackTrace();
      }

      System.out.println("client connection "+connectionNo+" done");


    }
  }


}

