/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
*/

package org.locomotive.router;

import java.io.*;
import java.net.*;
import java.util.*;

class LocomotiveConnection {


  private String getMethod(String request) {
    int end = request.indexOf(' ');

    return request.substring(0, end);
  }


  private String getName(String request) {
    int start = request.indexOf(' ') + 1;
    int end = request.indexOf(' ', start);

    if (start > -1 && end > -1)
      return request.substring(start, end);
    else
      return "/";
  }



  private String getPathInfo(String name) {
    int offset = name.indexOf("/locolink");
    int start = name.indexOf("/", offset+1);
    int end = name.indexOf("?", start);

    if (start > -1) {

      if (end == -1)
        return name.substring(start);
      else
        return name.substring(start, end);

    } else
      return "/";
  }



  private String getQueryString(String request) {
    int start = request.indexOf("?") + 1;
    int end = request.indexOf(" ", start);

    if (start > 0)
      return request.substring(start, end);
    else
      return "";
  }





/*
  // if the router is running on a different machine to locomotive the cookie domain may need to change
  // or this could be set in loco.conf
  // the cookie's domain must allow it to be sent back to the router

  private String modifyCookie(String cookie) {
    int d = cookie.indexOf("domain");

    if (d > -1) {
      int start = cookie.indexOf('=', d) + 1;
      int end = cookie.indexOf(';', d);

      System.out.println(cookie.substring(0, start) + ".sundayta.co.uk" + cookie.substring(end));
      return cookie.substring(0, start) + ".sundayta.co.uk" + cookie.substring(end);

    } else {
      return cookie;
    }

  }
*/



  private String[] getLocomotiveRequest(HttpRequest request) {
    Vector v = new Vector();

    System.out.println("*** "+getPathInfo("GET /locolink/base?a HTTP/1.0"));

    //SERVER_SOFTWARE =;
    //SERVER_NAME = router.getServerName();
    //SERVER_PORT = router.getServerPort();
    //GATEWAY_INTERFACE =;

    v.addElement("SERVER_PROTOCOL="+request.getVersion());
    v.addElement("REQUEST_METHOD="+request.getMethod());
    v.addElement("PATH_INFO="+request.getPath());
    v.addElement("QUERY_STRING="+request.getQueryString());
    
    String cookie = request.getCookie();
    if (cookie != null)
      v.addElement("HTTP_COOKIE="+cookie);

    String accept = request.getAccept();
    if (accept != null)
      v.addElement("HTTP_ACCEPT="+accept);

    String contentType = request.getContentType();
    if (contentType != null)
      v.addElement("CONTENT_TYPE="+contentType);

    Integer contentLength = request.getContentLength();
    if (contentLength != null)
      v.addElement("CONTENT_LENGTH="+contentLength);


    Enumeration e = v.elements();
    while (e.hasMoreElements()) {
      System.out.println("---> "+e.nextElement());
    }


    String[] result = new String[v.size()];
    v.copyInto(result);
    return result;
  }



  private String[] getLocomotiveRequest(String[] httpRequest) {

    // convert the http header into something locomotive will understand
    // need to make this more robust to stop it crashing
    String firstLine = httpRequest[0];
    String method = getMethod(firstLine);
    String name = getName(firstLine);
    String pathInfo = getPathInfo(name);
    String queryString = getQueryString(firstLine);

    System.out.println("path info: "+pathInfo);
    System.out.println("query string: "+queryString);


    // find a cookie line if there is one
    String cookie = null;
    for (int x=0; x<httpRequest.length; x++) {
      if (httpRequest[x].toLowerCase().startsWith("cookie")) {
        cookie = httpRequest[x].substring(8);
      }
    }

    // find a content-length line if there is one
    String contentLength = null;
    for (int x=0; x<httpRequest.length; x++) {
      if (httpRequest[x].toLowerCase().startsWith("content-length")) {
        contentLength = httpRequest[x].substring(16);
      }
    }



    Vector v = new Vector();
    v.addElement("SERVER_PROTOCOL=HTTP/1.0");

    if (cookie != null)
      v.addElement("HTTP_COOKIE="+cookie);

    if (contentLength != null)
      v.addElement("CONTENT_LENGTH="+contentLength);

    v.addElement("QUERY_STRING="+queryString);
    v.addElement("PATH_INFO="+pathInfo);
    v.addElement("REQUEST_METHOD="+method);


    String[] locoHeader = new String[v.size()];
    for (int x=0; x<v.size(); x++)
      locoHeader[x] = (String) v.elementAt(x);

    return locoHeader;
  }






  // need to put some more in here to make sure everything is closed if something goes wrong, that's why the
  // try block is here

  LocomotiveConnection(Address address, HttpRequest request, OutputStream toClient) throws UnknownHostException, IOException {

    // the input stream should be already have been read past the header by ClientConnection
    // so we just need to get anything after the header

    if (address == null) {
      // show no web server message
      Router.showErrorMessage("<b>Router has no locomotive defined</b>", toClient);

    } else {
      try {

        // connect to the locomotive
        Socket sock = new Socket(address.getUrl(), address.getPort());
        DataInputStream fromServer = new DataInputStream(sock.getInputStream());
        DataOutputStream toServer = new DataOutputStream(sock.getOutputStream());
        System.out.println("connected to locomotive");

        // send the header to the locomotive
        toServer.writeInt(2);
        //String[] s = getLocomotiveRequest(httpRequest);
        String[] s = getLocomotiveRequest(request);
        for (int x=0; x<s.length; x++) {
          System.out.println("sending to locomotive: "+s[x]);
          toServer.writeInt(s[x].length());
          toServer.writeBytes(s[x]);
        }
        toServer.writeInt(0);


        System.out.println("sent header to locomotive");


        // send the POST data to the locomotive exactly as is
        Integer contentLength = request.getContentLength();
        if (contentLength != null) {
          Router.copyStream(request.getInputStream(), toServer, contentLength.intValue());
        }



          //System.out.println("POST not fully supported yet!");
          /*
          int i;
          while ((i = clientReader.read()) != -1 && i != 13) {
            if (i != 10)
              toServer.writeByte((byte) i);
          }
          toServer.flush();
          */




/*
      // if the client is sending any more data, pass it on unchanged to loco
      // use a DataInputStream for this
      // is dis.available the best way to stop this from blocking???
      DataInputStream dis = new DataInputStream(fromClient);
      try {
        while (dis.available() > 0) {
          System.out.print(".");
          char ch = dis.readChar();
          toServer.writeChar(ch);
        }
      } catch (EOFException ee) {
        // ok, just stop looping, we've got everything from the client
      }
      toServer.flush();
*/

        System.out.println("finished sending data to locomotive");
        System.out.println("reading response header from locomotive");


        // start listening to locomotive to get the partly-http response header
        String line;
        Vector v = new Vector();

        // use BufferedReader
        BufferedReader br = new BufferedReader(new InputStreamReader(fromServer));
        do {
          if ((line = br.readLine()) != null)
            v.addElement(line);
        } while (line != null && line.length() > 0);



        System.out.println("writing http header to client");

        // write proper http response header to client
        DataOutputStream dos = new DataOutputStream(toClient);
        dos.writeBytes("HTTP/1.0 200 OK\n");

        String content = "Content-type: text/html";
        Enumeration e = v.elements();
        while (e.hasMoreElements()) {
          String str = (String) e.nextElement();
          if (str.toLowerCase().startsWith("content-type"))
            // remember this line and put it at the end
            content = str;
          //else if (str.toLowerCase().startsWith("set-cookie"))
          //  toClient.writeBytes(modifyCookie(str)+"\n");
          else if (str.toLowerCase().startsWith("status")) ;
          // throw away
          else if (str.length() > 0)
            dos.writeBytes(str+"\n");
        }

        dos.writeBytes(content+"\n\n");


        System.out.println("reading from locomotive and writing to client");

        // listen to locomotive for the rest of the data
        // write to client
        try {
          while ((line = br.readLine()) != null) {
            dos.writeBytes(line+"\n");
          }
        } catch (EOFException ee) {
          System.out.println(ee);
          dos.flush();
          // ok, just stop looping
        }

        System.out.println("closing connection to locomotive");


        // close down the connection to the locomotive
        toServer.close();
        fromServer.close();
        sock.close();

      } catch (IOException e) {
        Router.showErrorMessage("<b>Router was unable to connect to locomotive at "+address.getUrl()+" on port "+address.getPort()+" because of "+e+"</b>", toClient);
      }

    } // end if address

  }



}

