/*
 * The contents of this file are subject to the Locomotive Public License
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You
 * may not use this file except in compliance with the License; You may
 * obtain a copy of the License at http://www.locomotive.org/license/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The initial developers of this code under the LPL is Sundayta Ltd.
 * Portions created by Sundayta are Copyright (C) 1999 Sundayta Ltd.
 * All Rights reserved.
*/

package org.locomotive.router;

import java.io.*;
import java.net.*;
import java.util.*;

class WebServerConnection {


  WebServerConnection(Address address, HttpRequest request, OutputStream toClient) {

    if (address == null) {
      // show no web server message
      Router.showErrorMessage("<b>Router has no web server defined</b>", toClient);

    } else {

      Socket sock = null;
      OutputStream toServer = null;
      InputStream fromServer = null;

      try {

        // connect to the web server
        sock       = new Socket(address.getUrl(), address.getPort());
        toServer   = sock.getOutputStream();
        fromServer = sock.getInputStream();


        PrintWriter pw = new PrintWriter(toServer);
        
        // send the first line
        String firstLine = request.getMethod()+" "+request.getUri()+" "+request.getVersion();
        pw.println(firstLine);

        // send the rest of the headers
        Enumeration e = request.headerKeys();
        while (e.hasMoreElements()) {
          String key = (String) e.nextElement();
          pw.println(key+": "+request.getHeader(key));
        }

        // send a blank line
        pw.println("");
        pw.flush();


        // if there is post data in the request, forward this to the web server
        Integer contentLength = request.getContentLength();
        System.out.println(contentLength);
        if (contentLength != null) {
          try {
            Router.copyStream(request.getInputStream(), toServer, contentLength.intValue());
          } catch (Exception ex) {
            ex.printStackTrace();
          }
        }



        //Return response to client
        byte buf[] = new byte[1024];
        int count;

        count = fromServer.read(buf);
        BufferedOutputStream bufferedToClient = new BufferedOutputStream(toClient);
        while (count != -1) {
          bufferedToClient.write(buf, 0, count);
          bufferedToClient.flush();
          System.out.println("Passed " + count + " response bytes");
          count = fromServer.read(buf);
        }

        //System.out.println("Connection " + ConnectionCount + " Response has been forwarded to client");


      } catch (IOException e) {
        Router.showErrorMessage("<b>Router was unable to connect to web server at "+address.getUrl()+" on port "+address.getPort()+" because of "+e+"</b>", toClient);

      } finally {
        System.out.println("closing socket");
        try { if (sock != null) sock.close(); } catch (IOException e) {}
      }

    } // end if address
  }

}

