/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.server;

import java.util.*;

public class PooledThread implements Runnable 
{
    private ThreadPoolManager tpm;
    private Log log;
    private boolean quit_running = false;
    private int runcount = 0;
    private int id;

    private Runnable runnable_to_run;
    private boolean runnable = false;
    
    public PooledThread (ThreadPoolManager parent_tpm, Log l, int i)
    {
	tpm = parent_tpm;
	log = l;
	id = i;
    }
    
    public void run ()
    {
	if (log != null) log.log (8, "PT(" + id + "): awake");

	while (! quit_running) 
	{
	    waitForGo ();

	    if (log != null) log.log (5, "PT(" + id + "): firing");
	    // it all happens here
	    runnable_to_run.run ();
	    runcount++;
	    runnable = false;
	    tpm.returnToPool (this);
	}
    }

    public void loadRunnable (Runnable r)
    {
	runnable_to_run = r;
    }

    public synchronized void go ()
    {
	runnable = true;
	notifyAll ();
    }

    private synchronized void waitForGo ()
    {
	while (! runnable)
	{
	    try
	    {
		wait ();
	    }
	    catch (InterruptedException ie)
	    {
		if (log != null) log.log (0, "ERROR: PT(" + id + "): got interrupt on wait()");
	    }
	}	
    }

    public int getRunCount ()
    {
	return (runcount);
    }

    public void markToQuitRunning ()
    {
	quit_running = true;
    }
}
