/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 2000 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.servlets.test;

import org.locomotive.loco.*;
import org.locomotive.servlets.MultiThreadedHttpServlet;
import org.locomotive.server.Log;

import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;
import java.util.Hashtable;


/** 
 * This servlet is a test servlet that always throws UnavailableException
 * during initialization in an attempt to tell the servlet container that
 * this multithreaded servlet should be unavailable for two minutes.
 *
 * @author Jason Brittain <brittainj@diamtech.com>
 */
public class TemporarilyUnavailable1 extends MultiThreadedHttpServlet
{
    // We'll override the init() method and always throw UnavailableException.
    public void init() throws ServletException
    {
	// Always throw an UnavailableException during initialization!
	throw new UnavailableException(120, this, "Just Testing..");

	// FIXME: For Servlet 2.2 compliance, comment out the one above, and
	// uncomment this:
	//throw new UnavailableException("Just Testing..", 120);
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
	throws ServletException, IOException
    {
	String response_content = 
	    "<HTML>\n<HEAD>\n<TITLE>Locomotive - " +
            "TemporarilyUnavailable1 servlet</TITLE>\n</HEAD>\n" +
	    "<BODY>\n\n<H1>Error.</H1>\n" +
	    "The servlet container should never be able to send this " +
	    "message because this servlet is supposed to always be " +
	    "unavailable.  If you're seeing this message, it's due to " +
            "the servlet container not being fully compliant with SUN " +
            "Microsystems' Servlet specification.\n";
	response_content = response_content + "<BR>\n<BR>\n</BODY>\n</HTML>";

	PrintWriter print_writer = response.getWriter();
	print_writer.print(response_content);
	print_writer.flush();
    }
    
    public void doPost(HttpServletRequest request,
		       HttpServletResponse response)
	throws ServletException, IOException
    {
	doGet(request, response);
    }
}
