/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.steam;

import org.locomotive.util.GrowableChar;
import java.util.*;

		
/**
 * This class is NOT meant to be used publicly.  It's just a superclass for
 * defining >, >=, <, <=, and != operations.  Those are contained a little
 * lower in the file.
 */
class CompareTwoNumsExpr extends MixedExpr
{
    String error_string = null;
    
    /**
     * Override this method to create your own inequality comparisons.
     */
    boolean CompareFunction (int left, int right)
    {
	return (left == right);
    }
    
    public void evaluate (Hashtable ht, GrowableChar growchar)
    {
	if (notEmpty (ht))
	    growchar.add ("true");
	else if (error_string != null)
	    growchar.add (error_string);
    }
    
    public boolean notEmpty (Hashtable ht)
    {
	if (expr != null && expr.length > 1 &&
	    expr[0] != null && expr[1] != null)
	{
	    GrowableChar left_op_result = new GrowableChar(32);
	    GrowableChar right_op_result = new GrowableChar(32);
	    
	    expr[0].evaluate(ht, left_op_result);
	    expr[1].evaluate(ht, right_op_result);
	    
	    try {
		int left_value = 
		    Integer.parseInt(left_op_result.toString());
		int right_value = 
		    Integer.parseInt(right_op_result.toString());
		
		return (CompareFunction (left_value, right_value));
	    }
	    catch (NumberFormatException nfe) {
		error_string = "(NOT A NUMBER!)";
		return false;
	    }
	}
	else
	{
	    error_string = "(NULL NUMBERS!)";
	    return false;
	}
    }
}

