/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.steam;

import org.locomotive.util.GrowableChar;
import java.util.*;

	
/** puts an object keyed by "loop_index" into the hash table, which holds 
 * the current interation in the loop.  
 * Presently loops without set limits are looped 1 time.  
 * This should be changed to loop as many times as one of the arrays in the 
 * loop, but it's not implemented yet. 
 */
class LoopExpr extends MixedExpr
{
    int loop_limit = 0;

    public void evaluate(Hashtable ht, GrowableChar growchar)
    {
	if (expr != null && expr[0] != null && expr[0].notEmpty(ht) ) 
	{
	    try 
	    {
		GrowableChar g = new GrowableChar(40);
		expr[0].evaluate(ht, g);
		loop_limit = Integer.parseInt(g.toString().trim());
	    }
	    catch (NumberFormatException e)
	    {
		loop_limit = 1;
	    }
	}
	else  // default number of times to loop - should look to find the
	{    // longest array, or something like that.  No implemented yet
	    loop_limit = 1;
	}

	for (int loop_index = 0; loop_index < loop_limit; loop_index++)
	{
	    ht.put("loop_index", new Integer(loop_index) );

	    for (int i=1; i < expr.length; i++) 
	    {
		if (expr[i] == null)
		    continue;

		expr[i].evaluate(ht, growchar);
	    }
	}
	ht.remove("loop_index");
    }

}
	    
