/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.steam;

import org.locomotive.util.GrowableChar;
import java.util.*;


/**
 * This is the granddaddy of all template language expressions.
 * A MixedExpr contains a list of other MixedExprs.  It expands by expanding
 * all its children.
 * 
 * <P>OrExpr class does everything exactly the same as MixedExpr:
 * [IF [OR [VAR1] [VAR2] ...]] is exactly equivalent to:
 * [IF [QUOTE [VAR1] [VAR2] ...]].
 */
class MixedExpr
{
    MixedExpr expr[] = null;
    private Vector v = null;


    public MixedExpr ()
    {
	expr = new MixedExpr[0];
	v = new Vector(20);
    }
    
    public void insert (MixedExpr new_expr)
    {
	v.addElement (new_expr);
    }
    
    void convertVector ()
    {
	if (v != null)
	{
	    expr = new MixedExpr [v.size()];
	    for (int i=0; i < expr.length; i++)
	    {
		expr[i] = (MixedExpr) v.elementAt (i);
		expr[i].convertVector();
	    }
	    v = null;
	}
    }
    

    /* XXXXXXX TAKE OUT LATER!!!! DEBUG */
    void print (String s)
    {
	System.out.println(s);
    }

    /**
     * XXXXXXX TAKE OUT LATER!!!! DEBUG
     */
    void printTree (String s)
    {
	if (v != null)
	{
	    convertVector();
	    for (int i=0; i < expr.length; i++)
	    {
		expr[i].convertVector();
	    }
	}
	
	System.out.println("\n" + s + this.getClass().getName());
	for (int i=0; i < expr.length; i++)
	{
	    expr[i].printTree ("\t" + s + " " + i + ": ");
	}
    }
    
    
    /**
     * places the value of the output inside growchar, substituting 
     * dynamically for variables.
     */
    public void evaluate (Hashtable ht, GrowableChar growchar)
    {
	// If we haven't converted Vector() to char[] yet, then convert
	// ourselves, but also convert all our children, too.  This is
	// by far the easiest, most efficient way to go -- "Black Box".
	
	if (v != null) convertVector();
	
	if (expr != null)
	{
	    for (int i=0; i < expr.length; i++)
	    {
		if (expr[i] != null)
		    expr[i].evaluate (ht, growchar);
	    }
	}
    }


    /*
     * notEmpty () tells if the expression evaluates to "" or not.  It
     * helps speed up the evaluation of conditionals and helps to avoid
     * unnecessary string copying operations.
     *
     * MixedExpr.notEmpty() acts like an OR: it returns true if any
     * of its subcomponents are non-empty, else it returns false.
     */
    public boolean notEmpty (Hashtable ht)
    {
	if (v != null) convertVector();
	
	if (expr != null)
	{
	    for (int i=0; i < expr.length; i++)
	    {
		if (expr[i] != null && expr[i].notEmpty (ht))
		    return true;
	    }
	}
	
	return false;
    }
}

