/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.steam;

import java.util.*;

import org.locomotive.util.SortableTable;
import org.locomotive.util.GrowableChar;
			
/** 
 * Iteratively reads columns out of a SortableTable object stored in the 
 * hash table passed to it.  It puts an object keyed by "loop_index" into 
 * the hash table, which holds the current row number it is reading from.
 */
class TableExpr extends MixedExpr
{
    SortableTable table = null;
    int index = 0;
    protected TemplateCacheEntry parent = null;

    public void evaluate(Hashtable ht, GrowableChar growchar)
    {
	if (expr != null && expr[0] != null && expr[0].notEmpty(ht) ) 
	{
	    GrowableChar g = new GrowableChar(40);
	    expr[0].evaluate(ht, g);
	    String table_name = g.toString();
//		System.out.println("Table name: " + table_name);
	    if (ht.containsKey(table_name))
	    {
		table = (SortableTable) ht.get(table_name);
		if (table != null)
		    parent.table_stack.push(this);
		else
		    return;
	    }
	    else // table keyed by that name isn't in hash table- 
	    {
		return;
	    }
	}
	else  // no table name- we're done
	{
	    return;
	}

	for (index = 0; index < table.getNumRows(); index++)
	{
	    ht.put("loop_index", new Integer(index) );

	    for (int i=1; i < expr.length; i++) 
	    {
		if (expr[i] != null)
		    expr[i].evaluate(ht, growchar);
	    }
	}
	ht.remove("loop_index");
	parent.table_stack.pop();
	table = null;
	index = 0;
    }

    public boolean notEmpty(Hashtable ht) 
    {
	// if the first expression is null or empty, then the table is empty
	if (expr[0] == null || ! expr[0].notEmpty(ht) )
	    return false;
	    
	GrowableChar g = new GrowableChar(40);
	expr[0].evaluate(ht, g);
	String table_name = g.toString();
//		System.out.println("Table name: " + table_name);
	if (ht.containsKey(table_name))
	{
//		    System.out.println("Into table expression");
	    table = (SortableTable) ht.get(table_name);
	}
	else // table keyed by that name isn't in hash table- return
	    return false;

	return ( (table != null) && (table.getNumRows() > 0) );
    }

}


