/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util;

/**
 * BlockLock - allows one thread to hold and release another 
 * efficiently.  Warning: having multiple threads call block could 
 * result in bizzarre behaviour.  Calling block after unblock will 
 * have no effect and return immediately, since a BlockLock is only 
 * good for a one time usage.
 */
public class BlockLock
{
    private boolean ready_to_go = false;  
    private boolean timed_out = true;
    
    /**
     * creates a new BlockLock.  The constructor itself is non blocking.
     */
    public BlockLock ()
    {
	ready_to_go = false;
    }

    /**
     * thread that wants to block calls this
     */
    public synchronized void block ()
    {
	while (ready_to_go == false)
	{
	    try 
	    {
		wait ();
	    }
	    catch (InterruptedException ie) {}
	}
    }

    /**
     * blocks for a certain maximum time period.  Will return true 
     * if the block timed out.
     * @param time - the time in milliseconds to wait before timing out
     */
    public synchronized boolean blockForTime (long time)
    {
	timed_out = true;

	while (ready_to_go == false)
	{
	    try
	    {
		wait (time);
		ready_to_go = true;
	    }
	    catch (InterruptedException ie) {}
	}

	return (timed_out);
    }
    
    /**
     * unblocks the thread that called block()
     */
    public synchronized void unblock ()
    {
	ready_to_go = true;
	timed_out = false;
	notify();
    }
}

