/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util;

/*----------------------------------------------------------------------------
 * GrowableChar is an all-purpose, fast substitute for Strings or 
 * StringBuffers, designed mainly for adding to, not for removing from.
 * Whenever it runs out of space, it grows by doubling its space.
 * Derived from the BufferedCharBin class in DynamicPageLoader.java.
 * ---------------------------------------------------------------------------
 */
public class GrowableChar
{
    public char buf[];
    public int cursorpos;

    public GrowableChar (int initialsize)
    {
	buf = new char[initialsize];
	cursorpos = 0;
    }
    
    
    public void resize()
    {
	char newarray[] = new char [buf.length * 2];
	System.arraycopy (buf, 0, newarray, 0, cursorpos);
	buf = newarray;
    }

    
    void resize(int newsize)
    {
	char newarray[] = new char [newsize];
	System.arraycopy (buf, 0, newarray, 0, cursorpos);
	buf = newarray;
    }
    
    
    public void add (char newchars[])
    {
	if ((cursorpos + newchars.length) > buf.length)
	{
	    resize ((cursorpos + newchars.length) * 2);
	}

	System.arraycopy (newchars, 0, buf, cursorpos, newchars.length);
	cursorpos += newchars.length;
    }

    
    public void add (String s)
    {
	add (s.toCharArray());
    }

    
    public void add (GrowableChar growchar_more)
    {
	if ((cursorpos + growchar_more.cursorpos) > buf.length)
	{
	    resize ((cursorpos + growchar_more.cursorpos) * 2);
	}
	
	System.arraycopy (growchar_more.buf, 0,
			  buf, cursorpos, growchar_more.cursorpos);
	cursorpos += growchar_more.cursorpos;
    }
    
    
    public String toString()
    {
	return ( new String (buf, 0, cursorpos));
    }
    
    
    public char[] toCharArray()
    {
	char newarray[] = new char[cursorpos];
	System.arraycopy (buf, 0, newarray, 0, cursorpos);
	return (newarray);
    }
    
    
    public int length()
    {
	return cursorpos;
    }
    
    
    public void clear()
    {
	cursorpos = 0;
    }
}
