/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util;

import java.util.StringTokenizer;
import java.net.*;

/**
 * Holds library calls for internet-related networking stuff. Links in
 * liblevinet.so for native calls to support the verification of email
 * addresses via DNS. Potential source for problems here (hanging from
 * threads built up waiting for long-to-timeout DNS lookups) so watch the
 * hell out.
 */
public class INet
{
    // Load the library for the native methods
    static { System.loadLibrary("levinet"); }

    // The native method prototype
    private native static boolean verifyHost(String hostname);

    /**
     * Expects a full user@domain format address, not just domain.
     * Performs DNS lookup on domain for email address but WILL NOT
     * check syntax! Use StringConstraints instead. Some side-effect
     * syntax checking will be done here, however. <B>Note</B>: will make
     * a native call to DNS accessors.
     * @param email the string containing the email address to be looked up
     * @see org.locomotive.util.StringConstraint
     */
    public static boolean verifyEmailAddress(String email)
    {
	if (email == null) return (false);

	StringTokenizer tok = new StringTokenizer(email, "@");

	if (tok.hasMoreTokens())
	{
	    tok.nextToken();

	    if (tok.hasMoreTokens())
	    {
		return(verifyDomain(tok.nextToken()));
	    }
	    else return(false);
	}
	else return(false);
    }

    /**
     * Verify, via a native DNS call, that the given domain exists.
     */
    public static boolean verifyDomain(String domain)
    {
	if (domain == null) return(false);
	if (domain.equals("")) return(false);

	return(verifyHost(domain));
    }
}

