/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util;

import java.lang.Exception;

/**
 * This Exception class is used primarily by the StringUtil class for
 * signaling when a string value does not fit a particular format.
 * Here is an explanation of each of the error codes.
 *
 * <ol>
 * <li> If the string is null, sets an error code of IS_NULL.
 * <li> if the string is not null, but it's an empty string "", and the minimum length argument is not 0, then the FormData generates IS_EMPTY.
 * <li> if the string is shorter than the minimum length, sets TOO_SHORT.
 * <li> if it is longer than the maximum length, sets TOO_LONG.
 * <li> if the string does not fit the specific type you specified -- for example, if you expected digits, and there were alphabetical letters in the string -- then sets DOESNT_FIT_TYPE.
 * <li> if the string is numeric, and the number is smaller than the lower bound which you specified, then sets TOO_SMALL.
 * <li> however, if a numeric string is larger than the upper bound, then sets TOO_LARGE.
 * <li> if there was no error at all, sets NO_ERROR.
 * </ol>
 * 
 * You simply call errorCode() to get the error code for this 
 * ParseStringException.
 */
public class ParseStringException extends Exception
{
    public static final int NO_ERROR = 0;
    public static final int IS_NULL = 1;
    public static final int IS_EMPTY = 2;
    public static final int TOO_SHORT = 3;
    public static final int TOO_LONG = 4;
    public static final int TOO_SMALL = 5;
    public static final int TOO_LARGE = 6;
    public static final int DOESNT_FIT_TYPE = 7;
    
    int error_code;
    
    public ParseStringException (int code)
    {
	this.error_code = code;
    }
    
    public int errorCode ()
    {
	return error_code;
    }
}
