/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util;

import java.util.*;

/**
 */
public class SearchableVector
{
    // also make these public so that their methods can be usefully called.
    
    public Hashtable ht = new Hashtable();
    public Vector v = new Vector();
    
    
    /**
     * This method is called to find the element in this SearchableVector.
     * Think of it like the key for this element in a hash table.
    
     * !!!!Your subclass should override this method in order to search
     * for your own type of element!
     */
    public Object getKey (Object element)
    {
	return (element);
    }
    
    
    /**
     * This version of addElement() will check for uniqueness, as well as
     * index the element into a hash table for easier access later.
     */
    public void addElement (Object element)
    {
	if (! ht.containsKey (getKey(element)))
	{
	    v.addElement (element);
	    ht.put (getKey(element), element);
	}
    }
    
    public void addElements (Object element)
    {
	addElement (element);
    }
    
        
    public void addElements (Object elements[])
    {
	for (int i=0; i < elements.length; i++)
	{
	    addElement (elements[i]);
	}
    }
    
    
    public void removeElement (Object element)
    {
	Object key = getKey(element);
	if (ht.containsKey (key))
	{
	    v.removeElement (ht.get (key));
	    ht.remove (key);
	}
    }	

    
    public void removeElements (Object element)
    {
	removeElement (element);
    }
    
    
    public void removeElements (Object elements[])
    {
	for (int i=0; i < elements.length; i++)
	{
	    removeElement (elements[i]);
	}
    }
    

    public void removeAllElements ()
    {
	v.removeAllElements();
	ht.clear();
    }
	
    public Object[] getArrayOfElements ()
    {
	Object[] elements = new Object[v.size()];
	v.copyInto (elements);
	return (elements);
    }
    
    
    public Enumeration getEnumOfElements ()
    {
	return (v.elements());
    }
    
    
    public Object getElementByKey (Object element)
    {
	if (ht.containsKey (getKey(element)))
	    return (ht.get (getKey(element)));
	else
	    return null;
    }
    
    
    public boolean contains (Object element)
    {
	return (ht.containsKey (getKey(element)));
    }


    public int size()
    {
	return (v.size());
    }
}
