/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util.data;

import java.util.Date;

/**
 * this class provides a comparison key for dates with times.
 */
public class DateKey implements ComparisonKey {

    public Date date;			  
    
    /**
     * Constructor creates a new DateKey from a date
     * @param date the date to be used
     */
    public DateKey(Date date) {
	this.date = date;
    }
    
    /**
     * Compare myself to another object
     * @param key - the key to which this object is to be
     * compared
     * @return -1 - if this dateKey comes before otherKey <BR>
     * 1 - if this this dateKey comes after otherKey <BR>
     * 0 - if this dateKey is equivalent to otherKey
     * @exception ComparisonKeyException <BR> 
     * if otherKey cannot be compared with this object
     */
    public int compare(ComparisonKey otherKey) 
	throws ComparisonKeyException
    {
	if (! (otherKey instanceof DateKey) )
	    throw new ComparisonKeyException("Incompatible ComparisonKey types: " +
				      "DateKeyKey, " + otherKey.getClass().getName());

	DateKey key = (DateKey) otherKey;
	
	if (this.date.after(key.date) )
	    return 1;
	else if (this.date.before(key.date) )
	    return -1;
	else if (this.date.equals(key.date) )
	    return 0;
	else 
	    throw new ComparisonKeyException("ComparisonKeys could not be compared");					 
    }
    
    /** returns a string representation of the DateKey */
    public String toString() 
    {
	return date.toString();
    }
    
} // end of DateKey class






























