/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util.data;


/*========================== NAME KEY CLASS ======================*/
/**
  * this class provides a comparison key for names.  
  */
public class NameKey implements ComparisonKey {

    public String lastName;				
    public String firstName;
    
    /**
     * Constructor creates a new NameKey from a full name
     *
     * @param lastName the last name of the name
     * @param firstName the first name of the name
     */
    public NameKey(String lastName, String firstName) {
	this.lastName = lastName;
	this.firstName = firstName;
    }
    
    
    /**
     * Compare myself to another object
     * @param key - the key to which this object is to be
     * compared
     * @return -1 - if this nameKey comes before otherKey <BR>
     * 1 - if this this nameKey comes after otherKey <BR>
     * 0 - if this nameKey is equivalent to otherKey
     * @exception ComparisonKeyException <BR> 
     * if otherKey cannot be compared with this object
     */
    public int compare(ComparisonKey otherKey) 
	throws ComparisonKeyException 
    {
	if (! (otherKey instanceof NameKey) )
	    throw new ComparisonKeyException("Incompatible ComparisonKey types: " +
				      "NameKey, " + otherKey.getClass().getName());

	NameKey key = (NameKey) otherKey;
	
	if (this.lastName.compareTo(key.lastName) > 0)
	    return 1;
	else if (this.lastName.compareTo(key.lastName) < 0)
	    return -1;
	else if (this.lastName.equals(key.lastName) ) {
	    if (this.firstName.compareTo(key.firstName) > 0)
		return 1;
	    else if (this.firstName.compareTo(key.firstName) < 0)
		return -1;
	    else if (this.firstName.equals(key.firstName) )
		return 0;
	    else
		throw new ComparisonKeyException(
		    "ComparisonKeys could not be compared");	
	} else
	    throw new ComparisonKeyException("ComparisonKeys could not be compared");
    }

    
    /** returns a string representation of the object, of the
     *   form:  'lastName, firstName'
     */
    public String toString() {
	return new String (lastName + ", " + firstName);
    }

} // end of NameKey class

