/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util.data;

/**
 * this class provides a comparison key for most primitive data types
 */
public class NumberKey implements ComparisonKey {

    public double key;
    
    /**
     * Constructor creates a new NumberKey from a double
     * @param value the number to be used
     */
    public NumberKey(double value)
    {
	key = value;
    }

    /**
     * Constructor creates a new NumberKey from a char
     * @param value the number to be used
     */
    public NumberKey(char value)
    {
	key = value;
    }

    /**
     * Constructor creates a new NumberKey from a byte
     * @param value the number to be used
     */
    public NumberKey(byte value)
    {
	key = value;
    }

    /**
     * Constructor creates a new NumberKey from a short
     * @param value the number to be used
     */
    public NumberKey(short value)
    {
	key = value;
    }
    
    /**
     * Constructor creates a new NumberKey from a int
     * @param value the number to be used
     */
    public NumberKey(int value)
    {
	key = value;
    }

    /**
     * Constructor creates a new NumberKey from a long
     * @param value the number to be used
     */
    public NumberKey(long value)
    {
	key = value;
    }

    /**
     * Constructor creates a new NumberKey from a float
     * @param value the number to be used
     */
    public NumberKey(float value)
    {
	key = value;
    }

    /**
     * Constructor creates a new NumberKey from a Number object
     * @param value the number to be used
     */
    public NumberKey(Number value)
    {
	key = value.doubleValue();
    }
    
    


    /**
     * Compare myself to another object
     * @param key - the key to which this object is to be
     * compared
     * @return -1 - if this dateKey comes before otherKey <BR>
     * 1 - if this this dateKey comes after otherKey <BR>
     * 0 - if this dateKey is equivalent to otherKey
     * @exception ComparisonKeyException <BR> 
     * if otherKey cannot be compared with this object
     */
    public int compare(ComparisonKey value) 
	throws ComparisonKeyException 
    {
	if (! (value instanceof NumberKey) )
	    throw new ComparisonKeyException("Incompatible ComparisonKey types: " +
				      "NumberKey, " + value.getClass().getName());
	double otherKey = ((NumberKey) value).key;
	if (this.key > otherKey )
	    return 1;
	else if (this.key == otherKey)
	    return 0;
	else if (this.key < otherKey )
	    return -1;
	else 
	    throw new ComparisonKeyException("ComparisonKeys could not be compared");
    }
    
    public String toString() {
	return String.valueOf(key);
    }
    
}  // end of NumberKey class
		
