/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util.data;


/** This class contains a basic implementation of a stack using arrays
 * At this point, resizing the array will simply double the present size
 * of the array.  The main benefit of this over java.util.Stack is that
 * it isn't synchronized.
 */
public class Stack 
{
    private int index = -1;
    private int size = 20;     // default size
    private Object [] theArray;


    /** Creates a stack with an initial size of 20 */
    public Stack() 
    {
	theArray = new Object[size];
    }


    /** Creates a stack whose initial size is the specified parameter */
    public Stack(int initialSize)
    {
	size = initialSize;
	theArray = new Object[initialSize];
    }


    /** pushes an item onto the stack */
    public void push(Object obj)
    {
	if (++index == theArray.length)
	    resize();
	
	theArray[index] = obj;
    }

    /** pops an item off of the stack.  Returns null if the stack is 
     * empty */
    public Object pop()
    {
	if (index >= 0)
	{
	    return theArray[index--];
	}
	else
	    return null;
    }


    /** returns true if empty, false otherwise */
    public boolean isEmpty()
    {
	return (index < 0);
    }


    /* doubles the array size */
    private void resize()
    {
	size *=2;
	Object [] newArray = new Object [size];
	System.arraycopy(theArray, 0, newArray, 0, theArray.length);
	theArray = newArray;
    }

}
