/*
 * The contents of this file are subject to the Locomotive Public License 
 * (LPL), a derivative of the Mozilla Public License, version 1.0. You 
 * may not use this file except in compliance with the License; You may 
 * obtain a copy of the License at http://www.locomotive.org/license/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the 
 * License. 
 *
 * The initial developers of this code under the LPL is Leverage Information 
 * Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
 * Information Systems. All Rights reserved.
 */

package org.locomotive.util.data;


/*========================== NAME KEY CLASS ======================*/
/**
  * this class provides a comparison key for strings
  */
public class StringKey implements ComparisonKey {

    public String value;				
    
    /**
     * Constructor creates a new StringKey 
     * @param value the String to compare
     */
    public StringKey(String value) {
	this.value = value;
    }
    
    
    /**
     * Compare myself to another object
     * @param key - the key to which this object is to be compared
     * @return -1 - if this StringKey comes before otherKey <BR>
     * 1 - if this this StringKey comes after otherKey <BR>
     * 0 - if this StringKey is equivalent to otherKey
     * @exception ComparisonKeyException <BR> 
     * if otherKey cannot be compared with this object
     */
    public int compare(ComparisonKey otherKey) 
	throws ComparisonKeyException 
    {
	if (! (otherKey instanceof StringKey) )
	    throw new ComparisonKeyException("Incompatible ComparisonKey types: " +
				      "StringKey, " + otherKey.getClass().getName());

	StringKey key = (StringKey) otherKey;
	
	if (this.value.compareTo(key.value) > 0)
	    return 1;
	else if (this.value.compareTo(key.value) < 0)
	    return -1;
	else if (this.value.equals(key.value) )
	    return 0;
	else
	    throw new ComparisonKeyException(
		"ComparisonKeys could not be compared");	
    }

    
    /** returns a string representation of the object, of the
     *   form:  'value, firstName'
     */
    public String toString() {
	return new String(value);
    }

} // end of StringKey class

