#!/usr/bin/perl

# The contents of this file are subject to the Locomotive Public License 
# (LPL), a derivative of the Mozilla Public License, version 1.0. You 
# may not use this file except in compliance with the License; You may 
# obtain a copy of the License at http://www.locomotive.org/license/.
# 
# Software distributed under the License is distributed on an "AS IS" basis, 
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
# for the specific language governing rights and limitations under the 
# License. 
#
# The initial developers of this code under the LPL is Leverage Information 
# Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage 
# Information Systems. All Rights reserved.


# This little script will save you typing time by creating Java methods for
# objects for you.  It will take a simple declaration of a Java class, such as:
#
# public class MyClass
# {
#     int id;  // it assumes that the first field is the primary key ID
#     String str;
# }
# 
# and print out a class with the generic methods for instantiation and 
# accessing internal variables filled in, as well as database read/write
# operations, for example:
# 
# public class MyClass
# {
#     int id;
#     String str;
# 
#     public MyClass (int id,
#                     String str)
#     {
#         this.id = id;
#         this.str = str;
#     }
# 
#     public MyClass (Connection conn,
#                     int id)
#     {
#         this.id = id;
#         read (conn);
#     }
# 
#     public int myInt ()
#     {
#         return (id);
#     }
# 
#     public String str ()
#     {
#         return (str);
#     }
#
#     public boolean read (Connection conn) throws SQLException
#     {
#         String statement =
#             "select str " +
#             " from my_class where id = " + id;
#         
#         Statement stmt = conn.createStatement ();
#         ResultSet rs = stmt.executeQuery (statement);
#
#         if (rs.next())
#         {
#             str = rs.getString ("str");
#	      found = true;
#         }
#         else
#         {
#             found = false;
#         }
#
#         rs.close();
#         stmt.close();
#
#	  return found;
#     }
# 
#     public void update (Connection conn) throws SQLException
#     {
#         String statement =
#             "update my_class set " +
#             " id = " + id +
#	      ", str = " + str +
#             " where id = " + id;
#         
#         Statement stmt = conn.createStatement ();
#         stmt.executeUpdate (statement);
#         stmt.close();
#     }
# 
#     public void insert (Connection conn) throws SQLException
#     {
#         String statement =
#             "insert into my_class (str) values (" +
#	      str + ")" +
#             " where id = " + id;
#         
#         Statement stmt = conn.createStatement ();
#         stmt.executeUpdate (statement);
#         stmt.close();
#     }
# 
# }
#
# It reads directly from the input and writes directly to the output, so use
# redirection to create your new file.  For example:
#
# % getmethods.pl < myclass.java > newclass.java
#

$num_vars = 0;

while (<>)
{
    if (/public class (\w+)/)
    {
	$classname = $1;
    }
    elsif (/(\w+) (\w+).*=?.*;/)
    {
	$var_type[$num_var] = $1;
	$var[$num_var++] = $2;
    }
    elsif (/\}/)
    {
	print<<ENDBLOCK;

    /**
     * this constructor creates a new $classname without saving it
     * to the database.
     */
ENDBLOCK

	print "    public $classname ($var_type[0] $var[0]";
	for $i (1..$#var)
	{
	    print ",\n        $var_type[$i] $var[$i]";
	}
	print ")\n    {\n";
	for $i (0..$#var)
	{
	    print "        this.$var[$i] = $var[$i];\n";
	}
	print "    }\n\n";

	print<<ENDBLOCK;

    /**
     * this constructor load a new $classname from the database.
     */
    public $classname (Connection conn, $var_type[0] $var[0])
	throws SQLException
    {
        this.$var[0] = $var[0];
        read (conn);
    }


ENDBLOCK

	for $i (0..$#var)
	{
	    $varMethodName = "get_" . $var[$i];
	    convert_underbar_ToCaps ($varMethodName);
	    print "    public $var_type[$i] $varMethodName ()\n";
	    print "    {\n";
	    print "        return ($var[$i]);\n";
	    print "    }\n\n";
	    
	    $varMethodName = "set_" . $var[$i];
	    convert_underbar_ToCaps ($varMethodName);
	    print "    public void $varMethodName ($var_type[$i] $var[$i])\n";
	    print "    {\n";
	    print "        this.$var[$i] = $var[$i];\n";
	    print "    }\n\n";
        }
	
	print_DB_methods ();
    }
    print $_;
}



sub convert_underbar_ToCaps
{
    @name_with_underbars = split(/_/, @_[0]);
    $mixedCaseName = $name_with_underbars[0];
    foreach $part (1..$#name_with_underbars)
    {
	substr($name_with_underbars[$part], 0, 1) =~ tr/a-z/A-Z/;
	$mixedCaseName = $mixedCaseName . $name_with_underbars[$part];
    }
    
    @_[0] = $mixedCaseName;
}

sub convertCaps_to_underbar
{
    @mixedCaseName = split(//, @_[0]);
    $name_with_underbars = $mixedCaseName[0];
    $name_with_underbars =~ tr/A-Z/a-z/;
    foreach $part (1..$#mixedCaseName)
    {
	if ($mixedCaseName[$part] =~ tr/A-Z/a-z/)
	{
	    $name_with_underbars .= "_";
	}
	$name_with_underbars .= $mixedCaseName[$part];
    }
    
    @_[0] = $name_with_underbars;
}

    
sub print_DB_methods
{
    $underbar_classname = $classname;
    convertCaps_to_underbar ($underbar_classname);

    if ($#var > 0)
    {
	$varlist = $var[1];
    }
    for $i (2..$#var)
    {
	$varlist .= ", $var[$i]";
# this kicks in when there are very many fields:
	if ($i % 4 == 0)
	{
	    $varlist .= "\" +\n            \"";
	}
    }

    print <<ENDBLOCK;
    boolean read (Connection conn) throws SQLException
    {
	boolean found = false;
        String statement =
	    "select $varlist from $underbar_classname " +
            " where $var[0] = " + $var[0];\n

        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = conn.createStatement();
            rs = stmt.executeQuery (statement);\n
            if (rs.next())
            {
ENDBLOCK
    
    for $i (1..$#var)
    {
	$get_type = "get_$var_type[$i]";
	convert_underbar_ToCaps ($get_type);
	print "                $var[$i] = rs.$get_type (\"$var[$i]\");\n";
    }

    print <<ENDBLOCK;
                found = true;
            }
            else
            {
                found = false;
            }
        }
        finally
        {
            if (rs != null)
		rs.close();
            if (stmt != null)
		stmt.close();
        }
        return found;
    }


    /**
     * saves an existing $classname to the database.
     */
    public boolean update (Connection conn) throws SQLException
    {
        String statement =
	    "update $underbar_classname set " +
ENDBLOCK
    
    if ($#var > 0)
    {
	if ($var_type[$i] eq "String")
	{
	    print "            \" $var[0] = \" + SQL.quoteString($var[0]) ";
	}
	else
	{
	    print "            \" $var[0] = \" + $var[0] ";
	}
    }
    for $i (1..$#var)
    {
	if ($var_type[$i] eq "String")
	{
	    print " +\n            \", $var[$i] = \" + SQL.quoteString($var[$i])";
	}
	else
	{
	    print " +\n            \", $var[$i] = \" + $var[$i]";
	}
    }
    print "+\n            \" where $var[0] = \" + $var[0];\n\n";

    print <<ENDBLOCK;
        Statement stmt = null;
        try {
            stmt = conn.createStatement ();
            return (stmt.executeUpdate (statement) > 0);
        }
        finally
        {
            if (stmt != null)
                stmt.close();
        }
    }


    /**
     * saves a new $classname to the database.
     */
    public boolean insert (Connection conn) throws SQLException
    {
	String statement = "select ${underbar_classname}_seq.nextVal from dual";
	
        Statement stmt = null;
        ResultSet rs = null;
	try {
	    stmt = conn.createStatement ();
	    rs = stmt.executeQuery (statement);
	    if (rs.next())
	    {
		$var[0] = rs.getInt(1);
	    }
	    else
	    {
		return false;
	    }
	}
	finally {
	    if (rs != null)
		rs.close();
	    if (stmt != null)
		stmt.close();
	}

        String statement =
	    "insert into $underbar_classname" +
            "($varlist) values (" +
ENDBLOCK
    
    for $i (1..$#var - 1)
    {
	if ($var_type[$i] eq "String")
	{
	    print "            SQL.quoteString($var[$i]) + \",\" +\n";
	}
	else
	{
	    print "            $var[$i] + \",\" +\n";
	}
    }
    print "            $var[$#var] + \")\";\n\n";

    
    print <<EOF
        Statement stmt = null;
        try {
            stmt = conn.createStatement ();
            return (stmt.executeUpdate (statement) > 0);
        }
        finally
        {
            if (stmt != null)
                stmt.close();
        }
    }
EOF
}
