#!/usr/bin/perl

# The contents of this file are subject to the Locomotive Public License  
# (LPL), a derivative of the Mozilla Public License, version 1.0. You  
# may not use this file except in compliance with the License; You may  
# obtain a copy of the License at http://www.locomotive.org/license/.
#  
# Software distributed under the License is distributed on an "AS IS" basis,  
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License  
# for the specific language governing rights and limitations under the  
# License.  
#
# The initial developers of this code under the LPL is Leverage Information  
# Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage  
# Information Systems. All Rights reserved.


$LOCOPORT = 5434;
$LOCOIPADDRESS = '127.0.0.1';
$BUFSIZE = 4096;

############################################################
# open a connection to the LOCO
############################################################

$sock_addr_in = pack ('S n C4 x8', 2, $LOCOPORT,         
                      split(/\./, $LOCOIPADDRESS));

($name, $aliases, $proto) = getprotobyname ('tcp');

if (! socket (SFD, 2, 2, $proto))
{
    # unable to connect
    &exitwitherror("Connect failed: unable to allocate socket");
}

if (!connect (SFD, $sock_addr_in))
{
    &exitwitherror("Connect failed: $!");
}

alarm (180);

############################################################
# send version
############################################################

$buf = pack ('N', 1);

if ( &writebuf_sfd ($buf, length ($buf)) == -1)
{
    &exitwitherror("write failed for version");
}

############################################################
# Send the CGI variables, pack them first, then send the whole lot
############################################################

# reset the buffer to empty
$buf = "";

# add each key value pair to the buffer.
foreach $name (keys (%ENV))
{
    # write name
    $buf .= pack ('N a*', length ($name), $name);
    $value = $ENV{$name};
    $buf .= pack ('N a*', length ($value), $value);
}

# add terminator
$buf .= pack ('N', 0);

if ( &writebuf_sfd ($buf, length($buf)) == -1)
{
    &exitwitherror("write failed on env block");
}
    
############################################################
# write the stdin 
############################################################

if ($ENV{"CONTENT_LENGTH"})
{
    local ($total) = 0;
    local ($c_length) = $ENV{"CONTENT_LENGTH"};

    while ($total < $c_length)
    {
        $nr = sysread (STDIN, $buf, $BUFSIZE, 0);

        if (! defined ($nr))
        {
            &exitwitherror("read error for STDIN: $!");
        }

        if ($nr == 0)
        {
            &exitwitherror("premature EOF on STDIN");
        }

        if ( &writebuf_sfd ($buf, $nr) == -1)
        {
            &exitwitherror("write STDIN failed");
        }
        
        $total += $nr;
    }
}

############################################################
# read the results back from the LOCO
############################################################

while (($nr = sysread (SFD, $buf, $BUFSIZE)) != 0)
{
    if (defined ($nr))
    {
        if ( &writebuf_stdout ($buf, $nr) == -1)
        {
            &exitwitherror("response write failed to STDOUT");
        }
    }
    else
    {
        &exitwitherror("output read error");
    }
}

close (SFD);
close (STDIN);
close (STDOUT);
close (STDERR);

exit (0);

############################################################
############################################################
# Begin subroutines
############################################################

sub writebuf_sfd
{
    local ($buf, $len) = @_;
    local ($written, $totalwritten) = (0, 0);
    
    while ($totalwritten < $len)
    {
        $written = syswrite (SFD, $buf, 
                                $len - $totalwritten, $totalwritten);
        
        if (defined ($written) )
        {
            $totalwritten += $written;
        }
        else
        {
            return (-1);
        }
    }
    
    return (0);
}

sub writebuf_stdout
{
    local ($buf, $len) = @_;
    local ($written, $totalwritten) = (0, 0);

    while ($totalwritten < $len)
    {
        $written = syswrite (STDOUT, $buf, 
                                $len - $totalwritten, $totalwritten);
        
        if (defined ($written) )
        {
            $totalwritten += $written;
        }
        else
        {
            return (-1);
        }
    }

    return (0);
}


sub exitwitherror
{
    local ($errstring) = @_;

    print ("Content-Type: text/html\n\n");

    print ("<HTML>\n<HEAD>\n<TITLE>LOCO Error</TITLE>\n</HEAD>\n<BODY>\n");
    print ("<H1>Request Failed at LocoLink: </H1>\n");
    print ($errstring . "\n");
    print ("</BODY>\n</HTML>\n");
    exit (0);
}

