#!/usr/bin/perl

 
# The contents of this file are subject to the Locomotive Public License  
# (LPL), a derivative of the Mozilla Public License, version 1.0. You  
# may not use this file except in compliance with the License; You may  
# obtain a copy of the License at http://www.locomotive.org/license/.
#  
# Software distributed under the License is distributed on an "AS IS" basis,  
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License  
# for the specific language governing rights and limitations under the  
# License.  
#
# The initial developers of this code under the LPL is Leverage Information  
# Systems.  Portions created by Leverage are Copyright (C) 1998 Leverage  
# Information Systems. All Rights reserved.


$loopcnt = 0;
while ($line = <STDIN>)
{
    chop ($line);

    if ($loopcnt % 1000 == 0)
    {
	print ("line: $loopcnt\n");
    }
    $loopcnt++;
    
    ($date, $sid, $userid, $username, $major, $minor, $details) 
	= split (/\t/, $line);

    $hash_sid{$sid}++;
    $hash_sid_anonymous{$sid}++;

    if ($userid != 0)
    {
	$hash_auth_sid{$sid}++;
	$hash_userid_impressions{$userid}++;

	if (($major eq "login") && ($minor eq "success") ||
	    ($major eq "reg") && ($minor eq "success"))
	{
	    $hash_userid_sessions{$userid}++;
	}
    }

    $hash_major{$major}++;

    $storeval = "$major" . "_" . "$minor";
    $hash_major_minor{$storeval}++;

    if ($storeval eq "content_view")
    {
	$hash_content_view{$details}++;
    }

    if ($storeval eq "stat_sent")
    {
	$hash_stat_sent{$details}++;
    }
}

print ("SESSIONS AND USER SUMMARY\n");

# print total number of users
@userid_list = keys (%hash_userid_impressions);
$userid_count = $#userid_list + 1;

print ($userid_count . "\tunique users\n");

# print total number of sessions
@sid_list = keys (%hash_sid);
$sid_count = $#sid_list + 1;

print ($sid_count . "\ttotal sessions\n");

# print number of authenticated sessions
@auth_sid_list = keys (%hash_auth_sid);
$auth_sid_count = $#auth_sid_list + 1;

print ($auth_sid_count . "\tauthenticated sessions\n");

# determine and print number of anonymous only sessions
$anon_only_sid_count = $sid_count - $auth_sid_count;
print ($anon_only_sid_count . "\tanonymous only sessions\n");

# sum total number of impressions
foreach $sid (@sid_list)
{
    $impressions_total = $impressions_total + $hash_sid{$sid};
}

print ($impressions_total . "\ttotal impressions\n");

# zero authenticated sessions hash to give anonymous only hits
foreach $sid (@auth_sid_list)
{
    $hash_sid_anonymous{$sid} = 0;
}

# sum pages viewed for anonymous only sessions
foreach $sid (keys (%hash_sid_anonymous))
{
    $impressions_anon_only = 
	$impressions_anon_only + $hash_sid_anonymous{$sid};
}

print ($impressions_anon_only . 
       "\timpressions by anonymous only sessions\n");

# sum the impressions by users after authenticating
foreach $sid (keys (%hash_auth_sid))
{
    $impressions_post_auth = $impressions_post_auth + $hash_auth_sid{$sid};
}

# determine number of impressions for authenticated users before authenticating
$impressions_pre_auth = 
    $impressions_total - $impressions_anon_only - $impressions_post_auth;

print ($impressions_pre_auth . 
       "\timpressions by users before authenticating\n");
print ($impressions_post_auth . 
       "\timpressions by users after authenticating\n");

# second order values
$authenticated_sessions_per_user = $auth_sid_count / $userid_count;
$impressions_per_session = $impressions_total / $sid_count;
$impressions_per_anon_only_session = 
    $impressions_anon_only / $anon_only_sid_count;
$impressions_per_auth_session_before = $impressions_pre_auth / $auth_sid_count;
$impressions_per_auth_session_after = $impressions_post_auth / $auth_sid_count;
$impressions_per_auth_session_total = 
    ($impressions_pre_auth + $impressions_post_auth) / $auth_sid_count;


$out = sprintf ("%.2f", $authenticated_sessions_per_user);
print ($out . "\tnumber of authenticated sessions per user\n");

$out = sprintf ("%.2f", $impressions_per_session);
print ($out . "\timpressions per session (all types)\n");

$out = sprintf ("%.2f", $impressions_per_anon_only_session);
print ($out . "\timpressions per anon only session\n");

$out = sprintf ("%.2f", $impressions_per_auth_session_before);
print ($out . "\timpressions per authenticated session before athenticating\n");

$out = sprintf ("%.2f", $impressions_per_auth_session_after);
print ($out . "\timpressions per authenticated session after athenticating\n");

$out = sprintf ("%.2f", $impressions_per_auth_session_total);
print ($out . "\timpressions per authenticated session total\n");

print ("\n");

############################################################
# Major type summary section
############################################################

print ("MAJOR TYPE SUMMARY\n");

foreach $key (sort (keys %hash_major))
{
    print ($hash_major{$key} . "\t$key\n");
}

print ("\n");

############################################################
# Minor type summary section
############################################################

print ("MINOR TYPE BREAKDOWNSUMMARY\n");

foreach $key (sort (keys %hash_major_minor))
{
    print ($hash_major_minor{$key} . "\t$key\n");
}

print ("\n");

############################################################
# Content Viewing Breakdown
############################################################

print ("CONTENT VIEW BREAKDOWN\n");

foreach $key (sort (keys %hash_content_view))
{
    print ($hash_content_view{$key} . "\t$key\n");
}

print ("\n");

############################################################
# Static page view breakdown
############################################################

print ("STATIC PAGE VIEW BREAKDOWN\n");

foreach $key (sort (keys %hash_stat_sent))
{
    print ($hash_stat_sent{$key} . "\t$key\n");
}

print ("\n");

############################################################
# User login count distribution
############################################################


print ("USER VISIT DISTRIBUTION\n");
print ("users\tvisits\n");

foreach $userid (keys (%hash_userid_sessions))
{
    $count = $hash_userid_sessions{$userid};
    $distribution{$count}++;
}

foreach $num_visits (sort by_number (keys (%distribution)))
{
    $num_users = $distribution{$num_visits};
    print ("$num_users\t$num_visits\n");
}

sub by_number 
{
    $a <=> $b;
}

sub by_number_reverse
{
    if ($a < $b)
    {
	1;
    }
    elsif ($a == $b)
    {
	0;
    }
    elsif ($a > $b)
    {
	-1;
    }
}
