#!/usr/bin/perl -w

# The contents of this file are subject to the Locomotive Public License 
# (LPL), a derivative of the Mozilla Public License, version 1.0. You 
# may not use this file except in compliance with the License; You may 
# obtain a copy of the License at http://www.locomotive.org/license/.
# 
# Software distributed under the License is distributed on an "AS IS" basis, 
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
# for the specific language governing rights and limitations under the 
# License. 
#
# The initial developers of this code under the LPL is Leverage Information 
# Systems.  Portions created by Leverage are Copyright (C) 1999 Leverage 
# Information Systems. All Rights reserved.

# Test the ability of the Locomotive to handle virtual sessions.
#
# Let's repeatedly send a request.
# We'll send a "virgin" request each time, i.e., without cookies.  This allows
# the tunnel to assign us a locomotive.  Then we'll make a second request,
# returning all the cookies we were given.
#
# If the response comes from a different server, it would indicate that virtual
# sessions are not working, or that the first server went down.

use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;
use HTTP::Headers;

$url = "http://boson.leverage.com:8080/cgi-bin/locolink/helloworld";
$ua = new LWP::UserAgent;

$virgin_request = new HTTP::Request (GET, $url);
#print $virgin_request->as_string();

while (1) {
    $first_response = $ua->request($virgin_request);
    #print $first_response->as_string();

    if ($first_response->is_success) {
	#
	# fish out the cookies and insert them into a header for the 2nd req.
	#
	$cookie_string = $first_response->header('Set-cookie');
	#print "First response: $cookie_string\n\n";
	@cookies = split (/\,/, $cookie_string);

	my @out_cookies;
	foreach (@cookies) {
	    @tokens = split (/\;/);
	    ($name, $value) = split(/\=/, $tokens[0]);

	    if($name && $value) {
		#print "Name: $name -- Value: $value\n";
		push (@out_cookies, "$name=$value");
	    }
	}
	my $out_cookie_string = join('; ', @out_cookies);
	undef @out_cookies;
	print "$out_cookie_string\n";
	$header = new HTTP::Headers (('Cookie' => $out_cookie_string));
	undef $out_cookie_string;

	$second_request = new HTTP::Request (GET, $url, $header);
	#print $second_request->as_string();
	$second_response = $ua->request($second_request);
	#print $second_response->as_string();

	if ($second_response->is_success) {
	    #print $second_response->header('Set-cookie');
	} else {
	    print $second_response->error_as_HTML;
	}

	#print "\n\n";
    } else {
	print $first_response->error_as_HTML;
    }

    sleep(1);
}
